
var BerUI = require("../../index");
var CellBase = require("./CellBase");

// ====

var BerGrid = (function() {
    //

    //
    class BerGrid extends BerUI.BasePanel {
        constructor(parentView, options, itemOpts) {
            super(parentView, options);
            var _this = this;

            this.data = [];
            this.cells = [];

            var Cell = null;


            // == ui ==
            if(this.opts.showScrollbar) this.scrollPanel.setClassName("wkScroll");

            if(itemOpts) {
                if(itemOpts.cell) buildCell(itemOpts.cell);

            } else {
                console.error("invalid params");
            }


            // == methods ==

            function buildCell(itemOpts) {
                Cell = (function(ItemOpts) {
                    //
                    class Cell extends CellBase {
                        constructor(parentView, options) {
                            super(parentView, options, ItemOpts);
                            var _this = this;


                            // == ui ==


                            // == methods ==


                            // == events ==
                            for(var id in this.items) {
                                (function(ID) {
                                    _this.items[ID].on("itemClicked", function() {
                                        console.log(_this.data, _this.index, ID);
                                        _this.emit("itemClicked", _this.data, _this.index, ID);
                                    });

                                    // TODO: 可能還需要承接別的事件

                                })(id);
                            }


                        }

                        init() {
                            super.init();

                        }

                        load(data, index) {
                            super.load(data, index);

                            for(var id in data) {
                                if(this.items[id]) this.items[id].Value = data[id];
                            }

                        }

                    }

                    //
                    return Cell;
                })(itemOpts);
            }

            function clear() {
                clearCells();
                clearData();
            }
            function clearCells() {
                for(var i=0; i<_this.cells.length; i++) {
                    _this.cells[i].dispose();
                }
                _this.cells = [];
            }
            function clearData() {
                _this.data = [];
            }

            function sliceCell(index) {
                // NOTE: 留下從 0 到 index-1 的 item
                if(index > _this.cells.length) return;

                for(var i=index; i<_this.cells.length; i++) {
                    _this.cells[i].dispose();
                }

                _this.cells = _this.cells.splice(0, index);

            }
            function addCell(cnt) {
                var row;
                for(var i=0; i<cnt; i++) {
                    //
                    row = new Cell(_this.scrollPanel);
                    row.on("itemClicked", function(data, index, itemID) {
                        _this.emit("itemClicked", data, index, itemID);
                    });

                    //
                    _this.cells.push(row);

                }
            }

            function updateCells() {
                //
                var dataLength = _this.data.length;
                var cellsLength = _this.cells.length;

                if(dataLength > cellsLength) {
                    // data 較長, 新增 row
                    addCell(dataLength - cellsLength);

                } else if(dataLength < cellsLength) {
                    // data 較短, 捨去 row
                    sliceCell(dataLength);

                } else {
                    // 長度相等, 直接更新
                }

                // 更新介面
                for(var i=0; i<dataLength; i++) {
                    _this.cells[i].load(_this.data[i], i); // NOTE: item.load 需要 item 自己實作設定 BerPanel 內部元件的方法
                }

                //
                _this.redraw(true);

            }

            function load(data, opts) {
                //
                if(opts && opts.append) {
                    if(opts.append == "toTop") {
                        _this.data = data.concat(_this.data);
                    } else { // "toBottom"
                        _this.data = _this.data.concat(data);
                    }

                } else {
                    // 無 opts 則直接取代資料
                    if(Array.isArray(data)) {
                        _this.data = data;
                    } else {
                        _this.data.push(data);
                    }

                }
                //
                updateCells();
            }

            function remove(index) {
                //
                _this.cells[index].dispose();
                _this.cells.splice(index, 1);
                _this.data.splice(index, 1);
                //
                updateCells();
            }

            function setItemsLook(id, look) {
                for(var i=0; i<_this.cells.length; i++) {
                    if(_this.cells[i].items[id]) {
                        _this.cells[i].items[id].setLook(look);
                    }
                }

                if(look.value.bgColor) this.setLook({ bgColor: look.value.bgColor });

            }

            function scrollToTop() {
                _this.scrollPanel.scrollToTop();
            }

            //
            this.clear = clear;
            this.clearCells = clearCells;
            this.clearData = clearData;
            this.load = load;
            this.remove = remove;

            this.setItemsLook = setItemsLook;

            this.scrollToTop = scrollToTop;


            // == events ==
            window.addEventListener("resize", function () {
                if(_this.opts.visible) {
                    _this.redraw(true);
                }
            }, false);

            this.scrollPanel.on("atBottom", function() {
                _this.emit("atBottom");
            });


        }

        init() {
            super.init();

            //
            this.look.cellBgColor = BerColor.Bright;

            // NOTE: 預設是 [固定尺寸] 模式. 若 rowNum 與 colNum 皆被設定, 則改成 [固定視野內數量]. 兩種模式不會並存.
            // 以下項目用於 [固定尺寸] 的模式
            this.rect.baseCellSize = 150;
            // rect [固定視野內數量] 的模式
            this.rect.rowNum = 0;
            this.rect.colNum = 0;

            //
            this.look.bgColor = BerColor.Trasparent;//BerColor.Mid;
            this.opts.animate = true;
            this.opts.showScrollbar = false;


            // == ui ==
            var scrollPanel = new BerUI.BasePanel(this, {
                look: {
                    // padding: 1,
                    // bgColor: BerColor.Mid
                },
                overflow: {
                    x: "scroll",
                    y: "scroll"
                }
            });
            // scrollPanel.setClassName("wkScroll");


            //
            this.scrollPanel = scrollPanel;


        }

        redrawRect() {
            super.redrawRect();

            if(!isUsable(this.cells) || this.cells.length == 0) return;

            //
            if(this.rect.rowNum && this.rect.colNum) {
                // 固定視野內數量
                var ROW = this.rect.rowNum;
                var COL = this.rect.colNum;
                var CELL_W = 100 / COL;
                var CELL_H = 100 / ROW;
                for(var i=0; i<this.cells.length; i++) {
                    this.cells[i].setRect({
                        x: CELL_W * (i % COL),
                        y: CELL_H * Math.floor(i / COL),
                        w: CELL_W,
                        h: CELL_H
                    });
                }


            } else {
                // 固定尺寸
                var CELL_SIZE;
                var COL = Math.floor(this.PxW / this.rect.baseCellSize);
                CELL_SIZE = this.PxW / COL;
                var idxDraw = 0
                for(var i=0; i<this.cells.length; i++) {
                    // NOTE: 註解掉的是能隱藏部分 cell 的機制
                    // if(checkVisible(this.ui_state, this.user_group, this.cells[i].data)) {
                    //     this.cells[i].show();
                        this.cells[i].setRect({
                            x: CELL_SIZE * (idxDraw % COL) + "px",
                            y: CELL_SIZE * Math.floor(idxDraw / COL) + "px",
                            w: CELL_SIZE + "px",
                            h: CELL_SIZE + "px"
                        });
                        idxDraw++;
                    //
                    // } else {
                    //     this.cells[i].hide();
                    //     continue;
                    //
                    // }
                }

            }

        }

        redrawLook() {
            super.redrawLook();

            for(var i in this.cells) {
                this.cells[i].setLook({ bgColor: this.look.cellBgColor });
            }

            this.scrollPanel.setLook({
                bgColor: this.look.bgColor
            });

        }

    }


    //
    return BerGrid;
})();


//
module.exports = BerGrid;
