
var BerUI = require("../../index");
var RowBase = require("./RowBase");

// ====

var BerList = (function() {
    //

    //
    class BerList extends BerUI.BasePanel {
        constructor(parentView, options, itemOpts, customItems) {
            super(parentView, options);
            var _this = this;

            this.data = [];
            this.rows = [];

            var Header = null;
            var Row = null;


            // == ui ==
            if(this.opts.showScrollbar) this.scrollPanel.setClassName("wkScroll");

            if(itemOpts) {
                if(itemOpts.header) buildHeader(itemOpts.header);
                if(itemOpts.row) buildRow(itemOpts.row, this.opts.rowClickable);

            } else {
                console.error("invalid params");
            }


            // == methods ==

            function buildHeader(itemOpts) {
                Header = (function(ItemOpts) {
                    //
                    class Header extends RowBase {
                        constructor(parentView, options) {
                            super(parentView, options, ItemOpts, customItems);
                            var _this = this;


                            // == ui ==


                            // == methods ==


                            // == events ==


                        }

                        init() {
                            super.init();

                        }

                        load(data, index) {
                            super.load(data, index);

                            for(var id in data) {
                                if(this.items[id]) {
                                    if(typeof data[id] === "number" || typeof data[id] === "string") {
                                        // 簡單數值或文字設定
                                        if(this.items[id]) this.items[id].Value = data[id];

                                    } else {

                                        if(data[id].look) {
                                            // 包含外觀設定, 或較複雜的內部元件
                                            this.items[id].setLook(data[id].look);

                                        } else {
                                            // 多重數值設定
                                            this.items[id].setData(data[id]);

                                        }

                                    }
                                }
                            }

                        }

                    }

                    //
                    return Header;
                })(itemOpts);

                // init header
                _this.header = new Header(_this.cellHeader);

            }
            function buildRow(itemOpts, rowClickable) {
                Row = (function(ItemOpts, rowClickable) {
                    //
                    class Row extends RowBase {
                        constructor(parentView, options) {
                            super(parentView, options, ItemOpts, customItems);
                            var _this = this;


                            // == ui ==


                            // == methods ==


                            // == events ==
                            for(var id in this.items) {
                                (function(ID) {
                                    _this.items[ID].on("itemClicked", function() {
                                        console.log(_this.data, _this.index, ID);
                                        _this.emit("itemClicked", _this.data, _this.index, ID);
                                    });

                                    // TODO: 可能還需要承接別的事件

                                })(id);
                            }

                            // ====
                            // NOTE: BerList.opts.rowClickable == true
                            if(rowClickable) {

                                var clickMask = new BerUI.BasePanel(this, {
                                    opts: {
                                        zIndex: 10,
                                        cursor: "pointer"
                                    }
                                });

                                let target;

                                function setMaskColor(active) {
                                    //
                                    if(active) {
                                        clickMask.setLook({
                                            bgColor: "rgba(255, 255, 255, 0.5)",
                                        });
                                    } else {
                                        clickMask.setLook({
                                            bgColor: "rgba(255, 255, 255, 0)",
                                        });
                                    }

                                }

                                function onTouchStart(event) {
                                    if(_this.opts.disabled) return;
                                    //
                                    setMaskColor(true);
                                    target = event.target;
                                }
                                function onTouchMove(event) {
                                    if(_this.opts.disabled) return;
                                    //
                                    var moveTarget = document.elementFromPoint(
                                        event.changedTouches[0].pageX,
                                        event.changedTouches[0].pageY
                                    );
                                    if(target !== moveTarget) { // NOTE leave
                                        setMaskColor(false);
                                        target = null;
                                    }
                                }
                                function onTouchEnd(event) {
                                    if(_this.opts.disabled) return;
                                    setMaskColor(false);
                                    //
                                    // if(scrollPanel && scrollPanel.isScrolling) {
                                    //     console.log("yo");
                                    //     return;
                                    // }
                                    //
                                    var endTarget = document.elementFromPoint(
                                        event.changedTouches[0].pageX,
                                        event.changedTouches[0].pageY
                                    );
                                    if(target === endTarget) {
                                        if(event) {
                                            // if(consumeEvent) {
                                                event.preventDefault();
                                                event.stopPropagation();
                                            // }
                                            _this.emit("rowClicked", _this.data, _this.index);
                                        }
                                    }

                                }

                                function onMouseDown() {
                                    if(_this.opts.disabled) return;
                                    setMaskColor(true);
                                }
                                function onMouseEnter() {
                                    if(_this.opts.disabled) return;
                                    // _this.txt.setLook({ bgColor: _this.look.bgNormal }); // TODO add EnterStyle
                                }
                                function onMouseLeave() {
                                    if(_this.opts.disabled) return;
                                    setMaskColor(false);
                                }
                                function onMouseUp() {
                                    if(_this.opts.disabled) return;
                                    setMaskColor(false);
                                }
                                function onClick() {
                                    if(_this.opts.disabled) return;
                                    _this.emit("rowClicked", _this.data, _this.index);
                                }


                                //
                                if("ontouchstart" in window) {
                                    clickMask.domOn("touchstart", onTouchStart, false);
                                    clickMask.domOn("touchmove", onTouchMove, false);
                                    clickMask.domOn("touchend", onTouchEnd, false);
                                } else {
                                    clickMask.domOn("mousedown", onMouseDown);
                                    clickMask.domOn("mouseenter", onMouseEnter);
                                    clickMask.domOn("mouseleave", onMouseLeave);
                                    clickMask.domOn("mouseup", onMouseUp);
                                    clickMask.domOn("click", onClick);
                                }


                            }



                        }

                        init() {
                            super.init();

                        }

                        load(data, index) {
                            super.load(data, index);

                            for(var id in data) {
                                if(this.items[id]) {
                                    if(typeof data[id] === "number" || typeof data[id] === "string") {
                                        // 簡單數值或文字設定
                                        if(this.items[id]) this.items[id].Value = data[id];

                                    } else {

                                        if(data[id].look) {
                                            // 包含外觀設定, 或較複雜的內部元件
                                            this.items[id].setLook(data[id].look);

                                        } else {
                                            // 多重數值設定
                                            this.items[id].setData(data[id]);

                                        }

                                    }
                                }

                            }

                        }

                    }

                    //
                    return Row;
                })(itemOpts, rowClickable);
            }

            function clear() {
                clearRows();
                clearData();
            }
            function clearRows() {
                for(var i=0; i<_this.rows.length; i++) {
                    _this.rows[i].dispose();
                }
                _this.rows = [];
            }
            function clearData() {
                _this.data = [];
            }

            function sliceRow(index) {
                // NOTE: 留下從 0 到 index-1 的 item
                if(index > _this.rows.length) return;

                for(var i=index; i<_this.rows.length; i++) {
                    _this.rows[i].dispose();
                }

                _this.rows = _this.rows.splice(0, index);

            }
            function addRow(cnt) {
                var row;
                for(var i=0; i<cnt; i++) {
                    //
                    row = new Row(_this.scrollPanel);
                    row.on("itemClicked", function(data, index, itemID) {
                        console.log("[Row][itemClicked]", data, index, itemID);
                        _this.emit("itemClicked", data, index, itemID);
                    });
                    row.on("rowClicked", function(data, index) { // NOTE: opts.rowClickable == true
                        console.log("[Row][rowClicked]", data, index);
                        _this.emit("rowClicked", data, index);
                    });

                    //
                    _this.rows.push(row);

                }
            }

            function updateRows() {
                //
                var dataLength = _this.data.length;
                var rowsLength = _this.rows.length;

                if(dataLength > rowsLength) {
                    // data 較長, 新增 row
                    addRow(dataLength - rowsLength);

                } else if(dataLength < rowsLength) {
                    // data 較短, 捨去 row
                    sliceRow(dataLength);

                } else {
                    // 長度相等, 直接更新
                }

                // 更新介面
                for(var i=0; i<dataLength; i++) {
                    _this.rows[i].load(_this.data[i], i); // NOTE: item.load 需要 item 自己實作設定 BerPanel 內部元件的方法
                }

                //
                _this.redraw(true);

            }

            function load(data, opts) {
                //
                if(opts && opts.append) {
                    if(opts.append == "toTop") {
                        if(Array.isArray(data)) {
                            // 是 array 則 concat 在舊資料前面
                            _this.data = data.concat(_this.data);

                        } else {
                            // 是單一資料則 unshift
                            _this.data.unshift(data);

                        }

                    } else { // "toBottom"
                        if(Array.isArray(data)) {
                            // 是 array 則 concat 在舊資料後面
                            _this.data = _this.data.concat(data);

                        } else {
                            // 是單一資料則 push
                            _this.data.push(data);

                        }

                    }

                } else {
                    // 無 opts
                    if(Array.isArray(data)) {
                        // 是 array 則直接取代資料
                        _this.data = data;
                    } else {
                        // 是單一資料則用新的 array 取代
                        _this.data = [data];
                    }

                }
                //
                updateRows();
            }

            function remove(index) {
                //
                _this.rows[index].dispose();
                _this.rows.splice(index, 1);
                _this.data.splice(index, 1);
                //
                updateRows();
            }

            function setItemsLook(id, look) {
                for(var i=0; i<_this.rows.length; i++) {
                    if(_this.rows[i].items[id]) {
                        _this.rows[i].items[id].setLook(look);
                    }
                }

                if(look.value.bgColor) this.setLook({ bgColor: look.value.bgColor });

            }

            function scrollToTop() {
                _this.scrollPanel.scrollToTop();
            }
            function scrollTo(idx, animate, duration) {
                // NOTE 若沒空間捲了, 不一定會到頂
                var _animate = (isUsable(animate) ? animate : false);
                var _duration = (isUsable(duration) ? duration : 500);

                // var ref = 0;
                // if(this.opts.orientation == "horizontal") {
                //     for(var i=0; i<this.itemArr.length; i++) {
                //         if(i < idx) {
                //             ref += this.itemArr[i].PxW;
                //         } else {
                //             break;
                //         }
                //     }
                //
                //     if(_animate) {
                //         this.animate({ scroll: { x: ref }, duration: _duration });
                //     } else {
                //         this.core.scrollleft = ref;
                //     }
                //
                // } else {                    // vertical
                //     for(var i=0; i<this.itemArr.length; i++) {
                //         if(i < idx) {
                //             ref += this.itemArr[i].PxH;
                //         } else {
                //             break;
                //         }
                //     }
                //
                //     if(_animate) {
                //         this.animate({ scroll: { y: ref }, duration: _duration });
                //     } else {
                //         this.core.scrollTop = ref;
                //     }
                //
                // }

                var ref = 0;
                for(var i=0; i<_this.rows.length; i++) {
                    if(i < idx) {
                        ref += _this.rows[i].PxH;
                    } else {
                        break;
                    }
                }

                if(_animate) {
                    _this.scrollPanel.animate({ scroll: { y: ref }, duration: _duration });
                } else {
                    _this.scrollPanel.core.scrollTop = ref;
                }

                // TODO: 像上面例子一樣, 做 horizontal 版

            }

            //
            this.clear = clear;
            this.clearRows = clearRows;
            this.clearData = clearData;
            this.load = load;
            this.remove = remove;
            this.updateRows = updateRows;

            this.setItemsLook = setItemsLook;

            this.scrollToTop = scrollToTop;
            this.scrollTo = scrollTo;


            // == events ==
            window.addEventListener("resize", function () {
                if(_this.opts.visible) {
                    _this.redraw(true);
                }
            }, false);

            this.scrollPanel.on("atBottom", function() {
                _this.emit("atBottom");
            });


        }

        init() {
            super.init();

            // NOTE: 預設是 [固定尺寸] 模式. 若 rowNum 與 colNum 皆被設定, 則改成 [固定視野內數量]. 兩種模式不會並存.
            // 以下項目用於 [固定尺寸]的模式
            this.rect.rowPxH = 40; // NOTE: row 的高度 (廢話)
            // 以下兩項用於 [固定視野內數量] 的模式
            this.rect.rowNum = 0;

            this.rect.headerPxH = 0; // 0:跟著 row 的設定. 其他數字:獨立設定.

            this.look.headerBgColor = "white";
            this.look.rowBgColor = BerColor.Bright;
            this.look.scrollPadding = 0;

            this.opts.rowClickable = false;

            //
            this.look.bgColor = BerColor.Trasparent;//BerColor.Mid;
            this.opts.animate = true;
            this.opts.showScrollbar = true;


            // == ui ==
            var cellHeader = new BerUI.BasePanel(this, {
                // look: { bgColor: BerColor.Mid },
                opts: { zIndex: 1 },
                boxShadow: {
                    visible: true,
                    blur: 1,
                    spread: 0,
                    color: BerColor.Dark
                }
            });
            // var header = new Header(cellHeader);

            //
            var scrollPanel = new BerUI.BasePanel(this, {
                look: {
                    // padding: 1,
                    // bgColor: BerColor.Mid
                },
                overflow: {
                    // x: "scroll",
                    y: "scroll"
                }
            });
            // scrollPanel.setClassName("wkScroll");


            //
            this.cellHeader = cellHeader;
            this.scrollPanel = scrollPanel;

            this.header = null;

        }

        redrawRect() {
            super.redrawRect();

            var ROW_H, HEADER_H;

            if(this.rect.rowNum) {
                // 固定視野內數量
                var ROW = this.rect.rowNum;
                ROW_H = this.PxH / ROW;
                HEADER_H = (this.rect.headerPxH ? this.rect.headerPxH : ROW_H);

                if(this.header) {
                    this.cellHeader.show();

                    //
                    this.cellHeader.setRect({
                        h: HEADER_H + "px"
                    });
                    if(this.opts.showScrollbar) {
                        this.header.setRect({
                            w: (this.PxW - 12) + "px" // NOTE: 12 = width of the scrollbar
                        });
                    } else {
                        this.header.setRect({
                            w: this.PxW + "px"
                        });
                    }

                    this.scrollPanel.setRect({
                        y: HEADER_H + "px",
                        h: (this.PxH - HEADER_H) + "px"
                    });

                } else {
                    this.cellHeader.hide();
                    this.scrollPanel.setRect({
                        y: 0, h: 100
                    });

                }

            } else {
                // 固定尺寸
                ROW_H = this.rect.rowPxH;
                HEADER_H = (this.rect.headerPxH ? this.rect.headerPxH : ROW_H);

                if(this.header) {
                    this.cellHeader.show();

                    //
                    this.cellHeader.setRect({
                        h: HEADER_H + "px"
                    });
                    if(this.opts.showScrollbar) {
                        this.header.setRect({
                            w: (this.PxW - 12) + "px" // NOTE: 12 = width of the scrollbar
                        });
                    } else {
                        this.header.setRect({
                            w: this.PxW + "px"
                        });
                    }


                    this.scrollPanel.setRect({
                        y: HEADER_H + "px",
                        h: (this.PxH - HEADER_H) + "px"
                    });


                } else {
                    this.cellHeader.hide();
                    this.scrollPanel.setRect({
                        y: 0, h: 100
                    });

                }

            }

            //
            if(this.rows) {
                for(var i=0; i<this.rows.length; i++) {
                    this.rows[i].setRect({
                        y: (ROW_H * i) + "px",
                        h: ROW_H + "px"
                    });
                    // this.rows[i].redraw(true);
                }
            }


        }

        redrawLook() {
            super.redrawLook();

            this.cellHeader.setLook({
                bgColor: this.look.headerBgColor
            });

            for(var i in this.rows) {
                this.rows[i].setLook({ bgColor: this.look.rowBgColor });
            }

            this.scrollPanel.setLook({
                bgColor: this.look.bgColor,
                padding: this.look.scrollPadding
            });

        }

    }


    //
    return BerList;
})();


//
module.exports = BerList;
