
var BerUI = require("../../index");
var TextWithArrow = require("./TextWithArrow");

// ====

var ButtonWithArrow = (function() {
    //

    //
    class ButtonWithArrow extends TextWithArrow {
        constructor(parentView, options, scrollPanel) { // NOTE: 當作為 list 的 item 時, 用 scrollPanel.isScrolling 確保 touch 相關事件的捲動檢查運作正常
            super(parentView, options);
            var _this = this;

            var target;


            // == ui ==

            // NOTE: 初始化狀態相關的參數
            if(this.look.value.bgColor) {
                this.look.value.normalBgColor = this.look.value.bgColor;
                this.look.value.activeTxtColor = this.look.value.bgColor;
            }
            if(this.look.value.txtColor) {
                this.look.value.normalTxtColor = this.look.value.txtColor;
                this.look.value.activeBgColor = this.look.value.txtColor;
            }


            // == methods ==
            function setBtnColor(active) {
                //
                if(active) {
                    _this.look.value.txtColor = _this.look.value.activeTxtColor;
                    _this.look.value.bgColor = _this.look.value.activeBgColor;
                } else {
                    _this.look.value.txtColor = _this.look.value.normalTxtColor;
                    _this.look.value.bgColor = _this.look.value.normalBgColor;
                }
                //
                _this.cellBG.setLook({
                    bgColor: _this.look.value.bgColor,
                });
                _this.txtValue.setLook({
                    txtColor: _this.look.value.txtColor,
                });
                _this.txtArrow.setLook({
                    txtColor: _this.look.value.txtColor,
                });
            }

            function onTouchStart(event) {
                if(_this.opts.disabled) return;
                //
                setBtnColor(true);
                target = event.target;
            }
            function onTouchMove(event) {
                if(_this.opts.disabled) return;
                //
                var moveTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target !== moveTarget) { // NOTE leave
                    setBtnColor(false);
                    target = null;
                }
            }
            function onTouchEnd(event) {
                if(_this.opts.disabled) return;
                setBtnColor(false);
                //
                if(scrollPanel && scrollPanel.isScrolling) {
                    console.log("yo");
                    return;
                }
                //
                var endTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target === endTarget) {
                    if(event) {
                        // if(consumeEvent) {
                            event.preventDefault();
                            event.stopPropagation();
                        // }
                        _this.emit("itemClicked", _this.txtValue.Value);
                    }
                }

            }

            function onMouseDown() {
                if(_this.opts.disabled) return;
                setBtnColor(true);
            }
            function onMouseEnter() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal }); // TODO add EnterStyle
            }
            function onMouseLeave() {
                if(_this.opts.disabled) return;
                setBtnColor(false);
            }
            function onMouseUp() {
                if(_this.opts.disabled) return;
                setBtnColor(false);
            }
            function onClick() {
                if(_this.opts.disabled) return;
                _this.emit("itemClicked", _this.txtValue.Value);
            }


            // == events ==
            if("ontouchstart" in window) {

                this.txtValue.domOn("touchstart", onTouchStart, false);
                this.txtValue.domOn("touchmove", onTouchMove, false);
                this.txtValue.domOn("touchend", onTouchEnd, false);

                this.txtArrow.domOn("touchstart", onTouchStart, false);
                this.txtArrow.domOn("touchmove", onTouchMove, false);
                this.txtArrow.domOn("touchend", onTouchEnd, false);

            } else {

                this.txtValue.domOn("mousedown", onMouseDown);
                this.txtValue.domOn("mouseenter", onMouseEnter);
                this.txtValue.domOn("mouseleave", onMouseLeave);
                this.txtValue.domOn("mouseup", onMouseUp);
                this.txtValue.domOn("click", onClick);

                this.txtArrow.domOn("mousedown", onMouseDown);
                this.txtArrow.domOn("mouseenter", onMouseEnter);
                this.txtArrow.domOn("mouseleave", onMouseLeave);
                this.txtArrow.domOn("mouseup", onMouseUp);
                this.txtArrow.domOn("click", onClick);

            }

        }

        init() {
            super.init();

            //
            // this.look.value.txt = "";
            this.look.value.txtColor = BerColor.BtnText;
            this.look.value.bgColor = BerColor.BtnBg;
            this.look.value.txtAlign = "center";
            // this.look.value.txtScale = 0.4;
            //
            // this.look.value.padding = 2;
            // this.look.value.borderRadius = 4;

            // for btn (不需要讓外部設定)
            this.look.value.normalBgColor = this.look.value.bgColor;
            this.look.value.activeTxtColor = this.look.value.bgColor;
            this.look.value.normalTxtColor = this.look.value.txtColor;
            this.look.value.activeBgColor = this.look.value.txtColor;

            //
            this.txtValue.setOpts({ cursor: "pointer" });
            this.txtArrow.setOpts({ cursor: "pointer" });


        }

        setLook(look, setOnly) {
            if(look.value) {
                if(look.value.bgColor) {
                    this.look.value.normalBgColor = look.value.bgColor;
                    this.look.value.activeTxtColor = look.value.bgColor;
                }
                if(look.value.txtColor) {
                    this.look.value.normalTxtColor = look.value.txtColor;
                    this.look.value.activeBgColor = look.value.txtColor;
                }
            }
            super.setLook(look, setOnly);

        }

        //
        set Value(val) {
            this.look.value.txt = val;
            this.txtValue.Value = val;
            this.core.title = val; // TODO: 做成可被設定是否要更新 title. 確認其他類似元件是否需要.
        }
        get Value() {
            return this.txtValue.Value;
        }

    }


    //
    return ButtonWithArrow;
})();


//
module.exports = ButtonWithArrow;
