
var BerUI = require("../../index");
var ButtonWithArrow = require("./ButtonWithArrow");
var DropdownPanel = require("./DropdownPanel");

// ====

var DropdownButtonWithArrow = (function() {
    //

    //
    class DropdownButtonWithArrow extends ButtonWithArrow {
        constructor(parentView, options, scrollPanel) { // NOTE: 當作為 list 的 item 時, 用 scrollPanel.isScrolling 確保 touch 相關事件的捲動檢查運作正常
            super(parentView, options, scrollPanel);
            var _this = this;


            // == ui ==
            // console.log(_this.look.dropdown.mode || _this.opts.mode);
            var dropdownPanel = new DropdownPanel(BerRoot, {
                opts: {
                    visible: false,
                    mode: _this.look.dropdown.mode || _this.opts.mode
                }
            });


            // == methods ==
            function loadList(dataList) {
                _this.dataList = dataList;
            }
            function showList() {
                var itemsLook = {
                    value: {
                        txtColor: _this.look.value.txtColor,
                        bgColor: _this.look.value.bgColor,
                        txtAlign: "center",
                    }
                };
                dropdownPanel.setLook({ maxRows: _this.look.dropdown.maxVisibleRows });
                dropdownPanel.show(_this.cellBG, itemsLook, _this.dataList);
            }
            //
            this.loadList = loadList;
            this.showList = showList;


            // == events ==
            this.on("itemClicked", function() {
                showList();
                _this.btnMask.show();
            });

            dropdownPanel.on("close", function() {
                _this.btnMask.hide();
            });
            dropdownPanel.on("itemClicked", function(data, index, itemID) {
                console.log(data, index, itemID);

                _this.Data = data;
                // NOTE: 因應兩種設定方式, 所以 data 會有兩種格式. 但 emit 出去同一種.
                if(typeof data.value == "string") {
                     _this.Value = data.value;
                     _this.emit("dropdownClicked", data, index, itemID);
                } else {
                    _this.Value = data.value.value.txt;
                    _this.emit("dropdownClicked", { value: data.value.value.txt }, index, itemID);
                }

                dropdownPanel.hide();
                _this.btnMask.hide();
            });


        }

        init() {
            super.init();

            //
            this.dataList = []; // { value: "這個會顯示為按鈕文字" }

            //
            this.look.dropdown = {
                maxVisibleRows: 5,
                mode: null,         // NOTE: 與下面同時用來判斷, 但目前主要通用的是這個
            };

            this.opts.mode = null;


            //
            var btnMask = new BerUI.BasePanel(this.cellValue, {
                look: { bgColor: "rgba(0, 0, 0, 0.2)" },
                opts: { visible: false }
            });


            //
            this.btnMask = btnMask;

        }


        //
        set Value(val) {
            super.Value = val;
        }
        get Value() {
            return super.Value;
        }

        set Data(data) {
            this.data = data;
            if(data.value) this.Value = data.value;
        }
        get Data() {
            return this.data;
        }


    }


    //
    return DropdownButtonWithArrow;
})();


//
module.exports = DropdownButtonWithArrow;
