
var BerUI = require("../../../index");
var DropdownList = require("./DropdownList");
var PopupList = require("./PopupList");
var TriggerPanel = require("../../TriggerPanel");

// ====

var DropdownPanel = (function() {
    //

    //
    class DropdownPanel extends BerUI.BasePanel {
        constructor(parentView, options, data) {
            super(parentView, options);
            var _this = this;
            var target;


            // == ui ==


            // == methods ==


            // == events ==
            this.closeTrigger.on("trigger", function() {
                _this.emit("close");
                _this.hide();
            });

            // var del = true; // TEMP: test remove
            this.dropdownList.on("itemClicked", function(data, index, itemID) {
                _this.emit("itemClicked", data, index, itemID);
                // if(del) { // TEMP: test remove
                //     del = false;
                //     _this.dropdownList.remove(index);
                // }
            });
            this.popupList.on("itemClicked", function(data, index, itemID) {
                _this.emit("itemClicked", data, index, itemID);
                // if(del) { // TEMP: test remove
                //     del = false;
                //     _this.dropdownList.remove(index);
                // }
            });


            //
            // this.redraw(true);

        }

        init() {
            super.init();

            this.look.maxRows = 5; // 只對 dropdown 有效

            this.opts.zIndex = 1000;
            this.opts.visible = false;
            this.opts.mode; // dropdown or popup (若未設定則自動根據設備判斷)

            this.boxShadow.visible = true;
            this.boxShadow.h = 0;
            this.boxShadow.v = 8;
            this.boxShadow.blur = 16;
            this.boxShadow.spread = 0;
            this.boxShadow.color = "rgba(0, 0, 0, 0.2)";


            //
            var dropdownList = new DropdownList(this, {
                opts: {
                    visible: false,
                    animate: false,
                    showScrollbar: true
                }
            });
            var popupList = new PopupList(this, {
                look: {
                    bgColor: BerColor.Transparent
                },
                opts: {
                    visible: false,
                    animate: false,
                    showScrollbar: false
                }
            });

            //
            var closeTrigger = new TriggerPanel(BerRoot, {
                opts: {
                    zIndex: this.opts.zIndex - 1
                },
            });

            //
            this.dropdownList = dropdownList;
            this.popupList = popupList;
            this.closeTrigger = closeTrigger;


        }

        show(targetView, itemsLook, data) {
            super.show();

            //
            if(!this.opts.mode) {
                if("ontouchstart" in window) {
                    this.opts.mode = "popup";
                } else {
                    this.opts.mode = "dropdown";
                }
            }

            //
            if(this.opts.mode == "popup") {
                // popup

                this.setRect({
                    x: 10,
                    y: 10,
                    w: 80,
                    h: 80
                });

                var rows = ((data && data.length <= 8) ? data.length : 8);
                var rowPxH = this.PxH / rows;
                if(rowPxH > 100) rowPxH = 100;
                this.popupList.setRect({ rowPxH: rowPxH });
                this.popupList.load(data);
                if(itemsLook) {
                    this.popupList.setItemsLook("value", itemsLook);
                    this.popupList.setLook({ bgColor: BerColor.Transparent });
                }
                this.popupList.show();
                this.popupList.scrollToTop();

                this.closeTrigger.setLook({ bgColor: BerColor.TransparentDarker });

                // TEMP:
                var totalH = rowPxH * rows;
                if(totalH < this.PxH * 0.8) {
                    this.setRect({
                        y: (this.PxH * 0.1 + (this.PxH - totalH)/2) + "px",
                        h: totalH + "px"
                    });
                }


            } else {
                // dropdown

                var refRect = targetView.core.getBoundingClientRect();

                var H = refRect.height;

                var MAX_H = H * this.look.maxRows;
                var targetH = H * data.length;
                var h = ((targetH > MAX_H) ? MAX_H : targetH) + this.look.padding * 2;
                this.setRect({
                    x: refRect.x + "px",
                    y: (refRect.bottom - this.look.padding) + "px",
                    w: refRect.width + "px",
                    h: h + "px"
                });

                // console.log(H, MAX_H, targetH, h);

                this.dropdownList.setRect({ rowPxH: H });
                this.dropdownList.load(data);
                if(itemsLook) {
                    this.dropdownList.setItemsLook("value", itemsLook);
                    if(itemsLook.value.bgColor) {
                        this.dropdownList.setLook({ rowBgColor: itemsLook.value.bgColor });
                    }
                }
                this.dropdownList.show();
                this.dropdownList.scrollToTop();

                this.closeTrigger.setLook({ bgColor: BerColor.Transparent });

            }

            //
            // this.dropdownList.load(data);
            // if(itemsLook) {
            //     this.dropdownList.setItemsLook("value", itemsLook);
            //     if(itemsLook.value.bgColor) {
            //         this.dropdownList.setLook({ rowBgColor: itemsLook.value.bgColor });
            //     }
            // }
            // this.dropdownList.scrollToTop();   // TODO: 或許不需要每次都捲到頂, 待確認

            //
            if(!data || data.length == 0) {
                this.hide();
            } else {
                this.closeTrigger.show();
            }

        }

        hide() {
            super.hide();
            this.dropdownList.hide();
            this.popupList.hide();
            this.closeTrigger.hide();
        }

    }


    //
    return DropdownPanel;
})();


//
module.exports = DropdownPanel;
