
var BerUI = require("../../index");

// ====

// NOTE: 高度根據 parentView 和載入資料的 row 切
//       寬度則根據載入的資料切 (每個 row 不一定相同)

var BerPanel = (function() {
    //
    // itemSettings = [
    //     [col, col, ...], // row
    //     [col, col, ...], // row
    //     ...
    // ];

    //
    class BerPanel extends BerUI.BasePanel {
        constructor(parentView, options, itemSettings, customItems) { // NOTE: 當作為 list 的 item 時, parentView 必須是 scrollPanel, 以確保 touch 相關事件的捲動檢查運作正常
            super(parentView, options);
            var _this = this;

            var scrollPanel = parentView;
            var Item = {

                text: require("./Text"),
                button: require("./Button"),
                buttonwitharrow: require("./ButtonWithArrow"),//
                check: require("./Check"),
                textbox: require("./TextBox"),
                switch: require("./Switch"),
                canvas: require("./Canvas"),
                canvasbutton: require("./CanvasButton"),
                canvascheck: require("./CanvasCheck"),
                canvasswitch: require("./CanvasSwitch"),

                titletext: require("./TitleText"),
                titlebutton: require("./TitleButton"),
                titlecheck: require("./TitleCheck"),
                titlebox: require("./TitleBox"),
                titleswitch: require("./TitleSwitch"),
                titlecanvas: require("./TitleCanvas"),
                titlecanvasbutton: require("./TitleCanvasButton"),
                titlecanvascheck: require("./TitleCanvasCheck"),
                titlecanvasswitch: require("./TitleCanvasSwitch"),

                dropdownbutton: require("./DropdownButton"),
                dropdownbuttonwitharrow: require("./DropdownButtonWithArrow"),//
                dropdownbox: require("./DropdownBox"),

                titledropdownbutton: require("./TitleDropdownButton"),
                titledropdownbuttonwitharrow: require("./TitleDropdownButtonWithArrow"),//
                titledropdownbox: require("./TitleDropdownBox"),

                //
                separator: require("./Separator"),

            };
            if(customItems) {
                for(var i in customItems) {
                    if(Item[customItems[i].id]) {
                        console.log("[duplicate id]", customItems[i].id);
                    } else {
                        Item[customItems[i].id] = customItems[i].item;
                    }
                }
            }


            // == ui ==
            this.cells = [];

            this.rows = [];
            var refY = 0;
            var H = 100 / itemSettings.length;
            for(var i=0; i<itemSettings.length; i++) {
                this.rows.push(new BerUI.BasePanel(this.panel, {
                    rect: { y: refY, h: H }
                }));
                refY += H;
            }
            //
            this.items = {};
            for(var i=0; i<itemSettings.length; i++) {
                initItems(this.rows[i], itemSettings[i]);
            }


            // == methods ==
            function initItems(parentView, colSettings) {

                // TODO: 做個可以設定寬度跟位置的機制 >> 現在只有寬度, 位置要用隱形元件填補來達成
                // >>>> 預設情況下, 元件不會設定高度, 所以全部等高. 若要能設定高度, 必須讓 item 的 parentView 不再是 row (這對直接設定位置也有幫助). >> 載入時根據 itemSettings 的 i 決定所在的 y
                // >>>> 呃... 如果沒有好的設定方法, 做出來也會很難使用, 暫緩


                var COL = colSettings.length;
                var widths = getColWidths(colSettings);
                var refX = 0;
                var cell, item, options;
                for(var i=0; i<COL; i++) {
                    //
                    cell = new BerUI.BasePanel(parentView, {
                        rect: { x: refX, w: widths[i] },
                        // look: { padding: _this.look.cellPadding }
                    });
                    refX += widths[i];

                    //
                    item = createItem(cell, colSettings[i], scrollPanel);

                    //
                    _this.items[colSettings[i].id] = item;

                    _this.cells.push(cell);

                }

                _this.redraw(true);
            }

            function getColWidths(colSettings) {
                // NOTE: 在此將 colSettings.widthWeight 換算成每個欄位需要的寬度
                //       colSettings.widthWeight 是比重, 預設值都是 1
                //       (在沒額外設定的情況下, 每欄等寬)
                var weights = [];
                var weightSum = 0;
                var widths = [];
                var COL = colSettings.length;
                for(var i=0; i<COL; i++) {
                    if(isUsable(colSettings[i].widthWeight)) {
                        weights.push(colSettings[i].widthWeight);
                        weightSum += colSettings[i].widthWeight;
                    } else {
                        weights.push(1);
                        weightSum += 1;
                    }
                }
                for(var i=0; i<COL; i++) {
                    widths.push(weights[i] / weightSum * 100);
                }
                return widths;
            }

            function createItem(cell, itemData, scrollPanel) {
                // NOTE: 根據 type 選取元件, 元件內部會轉換 itemData 成設定元件用的 options
                // return new Item[itemData.type](cell, {}, itemData);

                // TEMP: deving
                // console.log(itemData.type);
                return new Item[itemData.type](cell, itemData, scrollPanel);


            }


            // == events ==


        }

        init() {
            super.init();

            this.look.padding = 2;
            this.look.cellPadding = 2;


            //
            var panel = new BerUI.BasePanel(this);

            //
            this.panel = panel;

        }

        // redrawRect() {
        //     super.redrawRect();
        //
        // }

        redrawLook() {
            super.redrawLook();

            // this.panel.setLook({ bgColor: this.look.bgColor });
            //
            // if(this.rows) {
            //     for(var i=0; i<this.rows.length; i++) {
            //         this.rows[i].setLook({ bgColor: this.look.bgColor });
            //     }
            // }

            for(var i in this.cells) {
                this.cells[i].setLook({ padding: this.look.cellPadding });
            }

        }


        //
        static getItem() {

            return {

                text: require("./Text"),
                button: require("./Button"),
                check: require("./Check"),
                textbox: require("./TextBox"),
                switch: require("./Switch"),
                canvas: require("./Canvas"),
                canvasbutton: require("./CanvasButton"),
                canvascheck: require("./CanvasCheck"),
                canvasswitch: require("./CanvasSwitch"),

                titletext: require("./TitleText"),
                titlebutton: require("./TitleButton"),
                titlecheck: require("./TitleCheck"),
                titlebox: require("./TitleBox"),
                titleswitch: require("./TitleSwitch"),
                titlecanvas: require("./TitleCanvas"),
                titlecanvasbutton: require("./TitleCanvasButton"),
                titlecanvascheck: require("./TitleCanvasCheck"),
                titlecanvasswitch: require("./TitleCanvasSwitch"),

                dropdownbutton: require("./DropdownButton"),
                dropdownbox: require("./DropdownBox"),

                titledropdownbutton: require("./TitleDropdownButton"),
                titledropdownbox: require("./TitleDropdownBox"),

                //
                separator: require("./Separator"),

            }

        };

    }


    //
    return BerPanel;
})();


//
module.exports = BerPanel;
