
var BerUI = require("../index");

// ====

// NOTE 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.


var CalendarNightSelector = (function() {
    //

    //
    class NightsPanel extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == ui ==
            var nightsPicker;


            // == methods ==
            function initNightsPicker() {
                nightsPicker = new BerUI.GenericPicker(BerRoot, {
                    rect: {
                        box: { w: 50, h: 50 }
                    },
                    look: {
                        rowNum: 5,
                        colNum: 5,
                        itemTxtScale: 0.2,
                        box: {
                            // overflowX: "scroll",
                            overflowY: "scroll"
                        }
                    },
                    opts: { zIndex: 99 }
                }, [
                    { name: "1 晚" },
                    { name: "2 晚" },
                    { name: "3 晚" },
                    { name: "4 晚" },
                    { name: "5 晚" },

                    { name: "6 晚" },
                    { name: "7 晚" },
                    { name: "8 晚" },
                    { name: "9 晚" },
                    { name: "10 晚" },

                    { name: "11 晚" },
                    { name: "12 晚" },
                    { name: "13 晚" },
                    { name: "14 晚" },
                    { name: "15 晚" },

                    { name: "16 晚" },
                    { name: "17 晚" },
                    { name: "18 晚" },
                    { name: "19 晚" },
                    { name: "20 晚" },

                    { name: "21 晚" },
                    { name: "22 晚" },
                    { name: "23 晚" },
                    { name: "24 晚" },
                    { name: "25 晚" },

                ]);

                nightsPicker.on("itemClicked", function(item) {
                    // console.log(item);
                    nightsPicker.hide();
                    _this.btnNight.setLook({ txt: item.data.name });
                    //
                    setNights(parseInt(item.data.name));
                    _this.emit("nightsChanged", _this.nights);
                });

            }
            function showNightsPicker() {
                if(!nightsPicker) initNightsPicker();
                nightsPicker.show();
            }

            function setNights(nights) {
                _this.nights = nights;
                if(_this.nights < 1) _this.nights = 1;
                _this.btnNight.setLook({ txt: _this.nights + " 晚" });
            }
            //
            this.showNightsPicker = showNightsPicker;
            this.setNights = setNights;


            // == events ==
            this.btnUp.on("click", function() {
                _this.nights--;
                if(_this.nights < 1) _this.nights = 1;
                _this.btnNight.setLook({ txt: _this.nights + " 晚" });
                //
                _this.emit("nightsChanged", _this.nights);

            });
            this.btnDown.on("click", function() {
                _this.nights++;
                _this.btnNight.setLook({ txt: _this.nights + " 晚" });
                //
                _this.emit("nightsChanged", _this.nights);
            });
            this.btnNight.on("click", showNightsPicker);

        }


        // == init ==
        init() {
            super.init();

            this.nights = 1;

            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            this.look.padding = 1;
            //
            this.look.btnBgNormal = BerColor.BtnBright;
            this.look.btnBgEnter = "rgb(200, 200, 200)";
            this.look.btnBgDown = "rgb(100, 100, 100)";
            this.look.btnBgColor = BerColor.BtnBright;
            this.look.btnTxtColor = "rgb(240, 240, 240)";
            this.look.btnTxtColor_down = "rgb(100, 100, 100)";


            // == ui ==
            var UNIT_H = 100 / 7.5;

            var cellUp = new BerUI.BasePanel(this, {
                rect: { h: UNIT_H * 2 },
                look: {
                    bgColor: this.look.btnBgNormal,
                    padding: 1
                }
            });
            var btnUp = new BerUI.ButtonView(cellUp, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/up_white.png')",
                    imgEnter: "url('img/sys/up_white.png')",
                    imgDown: "url('img/sys/up_light_gray.png')"
                }
            });

            var cellNight = new BerUI.BasePanel(this, {
                rect: { y: UNIT_H * 2, h: (100 - UNIT_H * 4) },
                look: { padding: 1 }
            });
            var btnNight = new BerUI.ButtonView(cellNight, {
                look: {
                    txtScale: 0.15,
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    txtNormal: this.look.btnTxtColor,
                    txtEnter: this.look.btnTxtColor,
                    txtDown: this.look.btnTxtColor_down,
                    txt: this.nights + " 晚"
                }
            });

            var cellDown = new BerUI.BasePanel(this, {
                rect: { y: (100 - UNIT_H * 2), h: UNIT_H * 2 },
                look: {
                    bgColor: this.look.btnBgNormal,
                    padding: 1
                }
            });
            var btnDown = new BerUI.ButtonView(cellDown, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/down_white.png')",
                    imgEnter: "url('img/sys/down_white.png')",
                    imgDown: "url('img/sys/down_light_gray.png')"
                }
            });


            //
            this.cellUp = cellUp;

            this.btnUp = btnUp;
            this.btnDown = btnDown;
            this.btnNight = btnNight;

        }

        //
        redrawRect() {
            super.redrawRect();

            var refRect = this.cellUp.RectPx;
            var refSize = ((refRect.w > refRect.h) ? refRect.h : refRect.w) * 0.8;

            this.btnUp.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

            this.btnDown.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

        }

    }

    class DateStringPanel extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            setTitle(this.look.title);
            setValue(this.look.value);


            // == methods ==
            function setTitle(title) {
                _this.txtTitle.setLook({ txt: title });
            }
            function setValue(value) {
                _this.txtValue.setLook({ txt: value });
            }
            //
            this.setTitle = setTitle;
            this.setValue = setValue;

            // == events ==

        }


        // == init ==
        init() {
            super.init();


            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.title = "";
            this.look.value = "";


            // == ui ==
            var UNIT_W = 100 / 9.5;

            var cellTitle = new BerUI.BasePanel(this, {
                rect: { w: UNIT_W * 2 }
            });
            var txtTitle = new BerUI.TextView(cellTitle, {
                look: {
                    txtScale: 0.4,
                    txtColor: "rgb(220, 220, 220)"
                }
            });

            var cellValue = new BerUI.BasePanel(this, {
                rect: { x: UNIT_W * 2, w: (100 - UNIT_W * 2) }
            });
            var txtValue = new BerUI.TextView(cellValue, {
                look: {
                    txtScale: 0.4,
                    txtColor: "white"
                }
            });


            //
            this.txtTitle = txtTitle;
            this.txtValue = txtValue;

        }

    }


    //
    class CalendarNightSelector extends BerUI.Calendar {
        constructor(parentView, options) {
            super(parentView, options, "toggle");
            var _this = this;

            this.nights = 1;
            this.inDate;
            this.outDate;

            // == ui ==


            // == methods ==
            function selectNights(date) {
                _this.clearActiveDates();
                //
                var newActiveDates = [date];
                for(var i=1; i<=_this.nights; i++) {
                    newActiveDates.push(date.addDays(i));
                }
                _this.loadActiveDates(newActiveDates);
                //
                _this.inDate = date;
                _this.outDate = newActiveDates[newActiveDates.length-1];
                _this.inPanel.setValue(_this.inDate.toYYYYMMdd());
                _this.outPanel.setValue(_this.outDate.toYYYYMMdd());
            }
            function setNights(nights) {
                _this.nights = nights;
                if(_this.nights < 1) _this.nights = 1;
                _this.nightsPanel.setNights(_this.nights);
            }

            function showNightsPicker() {
                _this.nightsPanel.showNightsPicker();
            }

            //
            this.selectNights = selectNights;
            this.setNights = setNights;

            this.showNightsPicker = showNightsPicker;


            // == events ==
            this.on("dateSelected", selectNights);
            this.on("dateUnselected", selectNights);
            this.nightsPanel.on("nightsChanged", function(nights) {
                _this.nights = nights;
                if(_this.inDate) selectNights(_this.inDate);
                //
                _this.emit("nightsChanged", nights);
            });

            this.btnDone.on("click", function() {
                _this.emit("maskClicked"); // NOTE parent 收到這事件會關閉本介面
            });

            // this.boxPanel.domOn("click", null);
            this.topPanel.domOn("click", null);
            this.rightPanel.domOn("click", null);
            this.bottomPanel.domOn("click", null);

        }


        // == init ==
        init() {
            super.init();


            // == ui ==
            var topPanel = new BerUI.BasePanel(this, {
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    blur: 4,
                    visible: true
                }
            });
            var inPanel = new DateStringPanel(topPanel, {
                rect: { h: 50 },
                look: { title: "入住" }
            });
            var outPanel = new DateStringPanel(topPanel, {
                rect: { y: 50, h: 50 },
                look: { title: "退房" }
            });


            var rightPanel = new BerUI.BasePanel(this, {
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    blur: 4,
                    visible: true
                }
            });
            var nightsPanel = new NightsPanel(rightPanel);


            var bottomPanel = new BerUI.BasePanel(this, {
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    blur: 4,
                    visible: true
                }
            });
            var btnDone = new BerUI.ButtonView(bottomPanel, {
                look: {
                    txtScale: 0.4,
                    bgNormal: BerColor.BtnBright,
                    bgEnter: BerColor.BtnBright,
                    bgDown: BerColor.BtnBright,
                    txtNormal: BerColor.Brighter,
                    txtEnter: BerColor.Brighter,
                    txtDown: BerColor.Bright,
                    txt: "Done"
                }
            });


            // ====
            this.topPanel = topPanel;
            this.rightPanel = rightPanel;
            this.bottomPanel = bottomPanel;

            this.nightsPanel = nightsPanel;

            this.inPanel = inPanel;
            this.outPanel = outPanel;

            this.btnDone = btnDone;

        }

        //
        redrawLook() {
            super.redrawLook();
            this.boxPanel.setBoxShadow({ blur: 4 });
        }
        redrawRect() {
            super.redrawRect();

            var BOX_X = this.boxPanel.PxX;
            var BOX_Y = this.boxPanel.PxY;
            var BOX_W = this.boxPanel.PxW;
            var BOX_H = this.boxPanel.PxH;

            var SPACE = 4;

            this.topPanel.setRect({
                x: BOX_X,
                y: BOX_Y - (BOX_H * (2/7.5) + SPACE), // y: BOX_Y + BOX_H + SPACE,
                w: BOX_W * (1 + 2/7) + SPACE,
                h: BOX_H * (2/7.5),
                mode: "px"
            });

            this.rightPanel.setRect({
                x: BOX_X + BOX_W + SPACE,
                y: BOX_Y,
                w: BOX_W * (2/7),
                h: BOX_H,
                mode: "px"
            });

            this.bottomPanel.setRect({
                x: BOX_X,
                 y: BOX_Y + BOX_H + SPACE,
                w: BOX_W * (1 + 2/7) + SPACE,
                h: BOX_H * (1/7.5),
                mode: "px"
            });


        }

        //
        setDate(year, month) {
            super.setDate(year, month);
            // NOTE 初始話顯示月份時, 也選取以當天為起點的天數
            this.selectNights(new Date(new Date().toYYYYMMdd()));
        }

        loadActiveDates(activeDates) {
            super.loadActiveDates(activeDates);
        }
        clearActiveDates() {
            super.clearActiveDates();
        }



    }



    // ====
    return CalendarNightSelector;
})();


// ====
module.exports = CalendarNightSelector;
