
var BerUI = require("../index");

// ====

var CheckButton = (function() {
    //

    //
    class CheckButton extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var target;
            this.checked = false;


            // == ui ==


            // == methods ==
            function onTouchStart(event) {
                if(_this.opts.disabled) return;
                //
                target = event.target;
            }
            function onTouchMove() {
                if(_this.opts.disabled) return;
                //
                var moveTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target !== moveTarget) { // NOTE leave
                    _this.txtValue.setLook({
                        bgColor: _this.look.normalBgColor,
                        txtColor: _this.look.activeBgColor
                    });
                    target = null;
                }
            }
            function onTouchEnd(event) {
                if(_this.opts.disabled) return;
                //
                var endTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target === endTarget) {
                    if(event) {
                        // if(consumeEvent) {
                            event.preventDefault();
                            event.stopPropagation();
                        // }
                        _this.checked = !_this.checked;
                        _this.redrawLook();

                        //
                        _this.emit("itemChecked", _this.txtValue.Value, _this.checked);
                    }
                }


            }

            function onMouseDown() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgActive });
            }
            function onMouseEnter() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal }); // TODO add EnterStyle
            }
            function onMouseLeave() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal });
            }
            function onMouseUp() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal });
            }
            function onClick() {
                if(_this.opts.disabled) return;
                _this.checked = !_this.checked;
                _this.redrawLook();
                _this.emit("itemChecked", _this.txtValue.Value, _this.checked);
            }


            // == events ==
            if("ontouchstart" in window) {
                this.txtValue.domOn("touchstart", onTouchStart, false);
                this.txtValue.domOn("touchmove", onTouchMove, false);
                this.txtValue.domOn("touchend", onTouchEnd, false);
            } else {
                this.txtValue.domOn("mousedown", onMouseDown);
                this.txtValue.domOn("mouseenter", onMouseEnter);
                this.txtValue.domOn("mouseleave", onMouseLeave);
                this.txtValue.domOn("mouseup", onMouseUp);
                this.txtValue.domOn("click", onClick);
            }



            //
            this.redraw(true);

        }

        init() {
            super.init();

            //
            this.look.txtScale = 0.4;
            this.look.normalBgColor = BerColor.Brighter;
            this.look.activeBgColor = BerColor.BtnBright;
            this.look.normalTxt = "";
            this.look.activeTxt = "";


            // == ui ==
            var txtValue = new BerUI.TextView(this, {
                look: {
                    borderRadius: 4,
                    txtAlign: "center"
                }
            });

            //
            this.txtValue = txtValue;

        }

        redrawLook() {
            super.redrawLook();

            if(this.checked) {
                this.txtValue.setLook({
                    txtScale: this.look.txtScale,
                    bgColor: this.look.activeBgColor,
                    txtColor: this.look.normalBgColor,
                    txt: this.look.activeTxt
                });
            } else {
                this.txtValue.setLook({
                    txtScale: this.look.txtScale,
                    bgColor: this.look.normalBgColor,
                    txtColor: this.look.activeBgColor,
                    txt: this.look.normalTxt
                });
            }

        }

        set Checked(checked) {
            this.checked = checked;
            this.redrawLook();
        }
        get Checked() {
            return this.checked;
        }


    }


    //
    return CheckButton;
})();


//
module.exports = CheckButton;
