
var BerUI = require("../../index");
var FuncBar = require("../FuncBar");
var TextTextRow = require("../TextTextRow");
var TextBoxRow = require("../TextBoxRow");
var SetPanel = require("../SetPanel");
var MenuItem = require("../MenuItem");

// ====

var DataPanelBase = (function() {
    //

    //
    class DataPanelBase extends BerUI.BasePanel {
        constructor(parentView, options, itemSettings) {
            super(parentView, options);
            var _this = this;


            // == ui ==
            this.funcBar.Title = this.look.funcbarTitle;
            this.funcBar.LeftText = this.look.funcbarLeftText;
            this.funcBar.RightText = this.look.funcbarRightText;

            // init rows
            this.rows = [];
            for(var i=0; i<itemSettings.length; i++) {
                this.rows.push(new BerUI.BasePanel(this.panel));
            }

            // init items
            this.items = {};
            for(var i=0; i<itemSettings.length; i++) {
                initItems(this.rows[i], itemSettings[i]);
            }


            // == methods ==
            function initItems(parentView, data) {
                var COL = data.items.length;
                var ITEM_W = 100 / COL;
                var refX = 0;
                var cell, item;

                for(var i=0; i<COL; i++) {
                    cell = new BerUI.BasePanel(parentView, {
                        rect: { x: refX, w: ITEM_W },
                        look: { padding: 3 }
                    });

                    item = createItem(cell, data.items[i]);

                    _this.items[data.items[i].id] = item;
                    refX += ITEM_W;

                }

            }

            function createItem(cell, itemData) {
                var item;
                switch (itemData.type) { // TODO: 支援更多類型
                    case "text":
                        item = new TextTextRow(cell, {
                            look: {
                                title: itemData.title,
                                value: itemData.value
                            }
                        });
                        break;

                    case "textbox":
                        item = new TextBoxRow(cell, {
                            look: {
                                title: itemData.title
                            }
                        });
                        if(itemData.textType) item.setOpts({ textType: itemData.textType });
                        break;

                    case "setPanel":
                        item = new SetPanel(cell, {
                            look: {
                                title: itemData.title,
                                value: itemData.value
                            }
                        });
                        break;

                    default:

                }
                return item;
            }

            function reset() {
                for(var id in _this.items) {
                    _this.items[id].reset();
                }
            }
            function load(data) {
                for(var id in data) {
                    if(_this.items[id]) _this.items[id].Value = data[id];
                }
            }

            function getData() {
                var data = {};
                for(var id in _this.items) {
                    data[id] = _this.items[id].Value;
                }
                return data;
            }

            //
            this.reset = reset;
            this.load = load;
            this.getData = getData;



            // == events ==
            this.funcBar.on("leftClicked", function() {
                _this.emit("back");
            });


            // ==
            this.redraw(true);

        }

        init() {
            super.init();

            this.look.funcBarPxH = 64;
            this.look.funcbarTitle = "";
            this.look.funcbarLeftText = "";
            this.look.funcbarRightText = "";

            this.look.imgSend = "img/temp.png";
            this.look.txtSend = "送出";


            //
            var funcBar = new FuncBar(this, {
                look: {
                    txtLeft: "返回"
                }
            });

            var panel = new BerUI.BasePanel(this, {
                look: { padding: 8 }
            });

            //
            var cellSend = new BerUI.BasePanel(panel, {
                look: { padding: 4 }
            });
            var btnSend = new MenuItem(cellSend, {
                look: {
                    borderRadius: 4,
                    txtColor: BerColor.BtnBright
                }
            }, {
                // iconSrc: "img/temp.png",
                name: "送出",
            });


            //
            this.funcBar = funcBar;
            this.panel = panel;

            this.cellSend = cellSend;

            this.btnSend = btnSend;


        }

        redrawRect() {
            super.redrawRect();

            //
            this.funcBar.setRect({ h: this.look.funcBarPxH + "px" });

            this.panel.setRect({
                y: this.look.funcBarPxH + "px",
                h: (this.PxH - this.look.funcBarPxH) + "px"
            });



            // ==
            if(!this.rows) return;

            var BASE_H = 80;
            var refY = 0;
            for(var i=0; i<this.rows.length; i++) {
                this.rows[i].setRect({
                    y: refY + "px",
                    h: BASE_H + "px"
                });
                refY += BASE_H;
            }

            //
            refY += BASE_H * 1.4;
            var SEND_SIZE = BASE_H * 1.7;
            this.cellSend.setRect({
                x: (this.parentView.PxW - SEND_SIZE) / 2 + "px",
                y: refY + "px",
                w: SEND_SIZE + "px",
                h: SEND_SIZE + "px"
            });

        }

        redrawLook() {
            super.redrawLook();

            this.btnSend.IconSrc = this.look.imgSend;
            this.btnSend.Value = this.look.txtSend;

        }

    }


    //
    return DataPanelBase;
})();


//
module.exports = DataPanelBase;
