
var BerUI = require("../index");
var SimpleBtn = require("./SimpleBtn");

// ====

var FuncBar = (function() {
    //

    //
    class FuncBar extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            // == ui ==
            this.btnLeft.Value = this.look.txtLeft;
            this.btnLeftSub.Value = this.look.txtLeftSub;
            this.btnRight.Value = this.look.txtRight;
            this.btnRightSub.Value = this.look.txtRightSub;
            this.txtTitle.Value = this.look.txtTitle;

            if(!isUsable(this.look.txtLeft) || this.look.txtLeft.length == 0) this.look.visibleLeft = false;
            if(!isUsable(this.look.txtLeftSub) || this.look.txtLeftSub.length == 0) this.look.visibleLeftSub = false;
            if(!isUsable(this.look.txtRight) || this.look.txtRight.length == 0) this.look.visibleRight = false;
            if(!isUsable(this.look.txtRightSub) || this.look.txtRightSub.length == 0) this.look.visibleRightSub = false;
            if(!isUsable(this.look.txtTitle) || this.look.txtTitle.length == 0) this.look.visibleTitle = false;

            this.btnLeft.setOpts({ visible: this.look.visibleLeft });
            this.btnLeftSub.setOpts({ visible: this.look.visibleLeftSub });
            this.btnRight.setOpts({ visible: this.look.visibleRight });
            this.btnRightSub.setOpts({ visible: this.look.visibleRightSub });
            this.txtTitle.setOpts({ visible: this.look.visibleTitle });


            // == methods ==
            function load(data) {
                _this.Title = data.name;
            }
            function showLeft(show) {
                _this.btnLeft.setOpts({ visible: show });
            }
            function showLeftSub(show) {
                _this.btnLeftSub.setOpts({ visible: show });
            }
            function showRight(show) {
                _this.btnRight.setOpts({ visible: show });
            }
            function showRightSub(show) {
                _this.btnRightSub.setOpts({ visible: show });
            }
            function showTitle(show) {
                _this.txtTitle.setOpts({ visible: show });
            }

            //
            this.load = load;
            this.showLeft = showLeft;
            this.showLeftSub = showLeftSub;
            this.showRight = showRight;
            this.showRightSub = showRightSub;
            this.showTitle = showTitle;


            // == events ==
            this.btnLeft.on("clicked", function() {
                _this.emit("leftClicked");
            });
            this.btnLeftSub.on("clicked", function() {
                _this.emit("leftSubClicked");
            });
            this.btnRight.on("clicked", function() {
                _this.emit("rightClicked");
            });
            this.btnRightSub.on("clicked", function() {
                _this.emit("rightSubClicked");
            });

        }


        // == init ==
        init() {
            super.init();

            this.look.bgColor = BerColor.Brighter;
            this.opts.zIndex = 10;

            this.boxShadow.visible = true;
            this.boxShadow.blur = 1;
            this.boxShadow.spread = 0;
            this.boxShadow.color = BerColor.Dark;

            //
            this.look.txtLeft = "";
            this.look.txtLeftSub = "";
            this.look.txtRight = "";
            this.look.txtRightSub = "";
            this.look.txtTitle = "";
            this.look.visibleLeft = true;
            this.look.visibleLeftSub = false;
            this.look.visibleRight = true;
            this.look.visibleRightSub = false;
            this.look.visibleTitle = true;


            // == ui ==
            var panel = new BerUI.BasePanel(this, {
                // look: { padding: 8 }
            });

            var BTN_W = 15;
            var TITLE_W = 100 - BTN_W * 4;

            var btnLeft = new SimpleBtn(panel, {
                rect: { w: BTN_W },
                look: {
                    txtScale: 0.3,
                    txt: this.look.txtLeft
                }
            });
            var btnLeftSub = new SimpleBtn(panel, {
                rect: { x: BTN_W, w: BTN_W },
                look: {
                    txtScale: 0.3,
                    txt: this.look.txtLeftSub
                }
            });

            var cellTitle = new BerUI.BasePanel(panel, {
                rect: { x: BTN_W * 2, w: TITLE_W }
            });
            var txtTitle = new BerUI.TextView(cellTitle, {
                look: {
                    txtColor: BerColor.Dark,
                    txtScale: 0.3,
                    txtAlign: "center"
                }
            });

            var btnRightSub = new SimpleBtn(panel, {
                rect: { x: BTN_W * 2 + TITLE_W, w: BTN_W },
                look: {
                    txtScale: 0.3,
                    txt: this.look.txtRightSub
                }
            });
            var btnRight = new SimpleBtn(panel, {
                rect: { x: BTN_W * 3 + TITLE_W, w: BTN_W },
                look: {
                    txtScale: 0.3,
                    txt: this.look.txtRight
                }
            });

            //
            this.btnLeft = btnLeft;
            this.btnLeftSub = btnLeftSub;
            this.txtTitle = txtTitle;
            this.btnRightSub = btnRightSub;
            this.btnRight = btnRight;


        }

        set Title(val) {
            this.txtTitle.Value = val;
            this.look.txtTitle = val;
            if(!isUsable(this.look.txtTitle) || this.look.txtTitle.length == 0) {
                this.look.visibleTitle = false;
            } else {
                this.look.visibleTitle = true;
            }
            this.txtTitle.setOpts({ visible: this.look.visibleTitle });
        }
        set LeftText(val) {
            this.btnLeft.Value = val;
            this.look.txtLeft = val;
            if(!isUsable(this.look.txtLeft) || this.look.txtLeft.length == 0) {
                this.look.visibleLeft = false;
            } else {
                this.look.visibleLeft = true;
            }
            this.btnLeft.setOpts({ visible: this.look.visibleLeft });
        }
        set LeftSubText(val) {
            this.btnLeftSub.Value = val;
            this.look.txtLeftSub = val;
            if(!isUsable(this.look.txtLeftSub) || this.look.txtLeftSub.length == 0) {
                this.look.visibleLeftSub = false;
            } else {
                this.look.visibleLeftSub = true;
            }
            this.txtTitle.setOpts({ visible: this.look.visibleLeftSub });
        }
        set RightText(val) {
            this.btnRight.Value = val;
            this.look.txtRight = val;
            if(!isUsable(this.look.txtRight) || this.look.txtRight.length == 0) {
                this.look.visibleRight = false;
            } else {
                this.look.visibleRight = true;
            }
            this.btnRight.setOpts({ visible: this.look.visibleRight });
        }
        set RightSubText(val) {
            this.btnRightSub.Value = val;
            this.look.txtRightSub = val;
            if(!isUsable(this.look.txtRightSub) || this.look.txtRightSub.length == 0) {
                this.look.visibleRightSub = false;
            } else {
                this.look.visibleRightSub = true;
            }
            this.btnRightSub.setOpts({ visible: this.look.visibleRightSub });
        }

        set BgColor(val) {
            this.look.bgColor = val;
            this.btnLeft.look.bgNormal = val;
            this.btnLeftSub.look.bgNormal = val;
            this.btnRight.look.bgNormal = val;
            this.btnRightSub.look.bgNormal = val;
            this.redraw(true);
        }

    }



    //
    return FuncBar;
})();


//
module.exports = FuncBar;
