
var BerUI = require("../index");
var TextView = BerUI.TextView;
var ButtonView = BerUI.ButtonView;
var BasePanel = BerUI.BasePanel;
var ListItem = BerUI.ListItem;

// ====

var COLOR_BTN_BRIGHT = "rgb(56, 147, 170)";

var GenericPicker = (function() {
    //

    //
    class PickerItem extends ListItem {
        constructor(parentView, options, data) {
            super(parentView, options, data);
            var _this = this;

            var target;

            this.data = data;


            // == ui ==
            // this.btn.setLook({
            //     bgColor: this.look.bgNormal,
            //     txtColor: this.look.txtNormal,
            //     txtScale: this.look.txtScale,
            //     borderRadius: this.look.borderRadius,
            //     txt: data.name
            // });


            // == methods ==
            function onTouchStart(event) {
                if(_this.opts.disabled) return;
                //
                _this.btn.setLook({
                    bgColor: _this.look.bgDown,
                    txtColor: _this.look.txtDown
                });
                target = event.target;
            }
            function onTouchMove(event) {
                if(_this.opts.disabled) return;
                //
                _this.btn.setLook({
                    bgColor: _this.look.bgNormal,
                    txtColor: _this.look.txtNormal
                });
                //
                target = null;
            }
            function onTouchEnd(event) {
                if(_this.opts.disabled) return;
                _this.btn.setLook({
                    bgColor: _this.look.bgNormal,
                    txtColor: _this.look.txtNormal
                });
                //
                if(_this.parentView.parentView.isScrolling) {
                    console.log("yo");
                    return;
                }
                //
                var endTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target === endTarget) {
                    if(event) {
                        // if(consumeEvent) {
                            event.stopPropagation();
                            event.preventDefault();
                        // }
                         _this.emit("itemClicked", _this);
                    }
                }

            }

            function onMouseDown() {
                if(_this.opts.disabled) return;
                _this.btn.setLook({
                    bgColor: _this.look.bgDown,
                    txtColor: _this.look.txtDown
                });
            }
            function onMouseEnter() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal }); // TODO add EnterStyle
            }
            function onMouseLeave() {
                if(_this.opts.disabled) return;
                _this.btn.setLook({
                    bgColor: _this.look.bgNormal,
                    txtColor: _this.look.txtNormal
                });
            }
            function onMouseUp() {
                if(_this.opts.disabled) return;
                _this.btn.setLook({
                    bgColor: _this.look.bgNormal,
                    txtColor: _this.look.txtNormal
                });
            }
            function onClick() {
                if(_this.opts.disabled) return;
                _this.emit("itemClicked", _this.data);
            }



            // == events ==
            if("ontouchstart" in window) {
                this.btn.domOn("touchstart", onTouchStart, false);
                this.btn.domOn("touchmove", onTouchMove, false);
                this.btn.domOn("touchend", onTouchEnd, false);
            } else {
                this.btn.domOn("mousedown", onMouseDown);
                this.btn.domOn("mouseenter", onMouseEnter);
                this.btn.domOn("mouseleave", onMouseLeave);
                this.btn.domOn("mouseup", onMouseUp);
                this.btn.domOn("click", onClick);
            }



        }

        init() {
            super.init();

            // inherit
            this.look.padding = 2;
            this.look.borderRadius = 4;
            this.look.txtScale = 0.4;
            // extend
            this.look.bgNormal = COLOR_BTN_BRIGHT;
            this.look.bgDown = "rgb(100, 100, 100)";
            this.look.txtNormal = "rgb(240, 240, 240)";
            this.look.txtDown = "rgb(200, 200, 200)";

            //
            var cellValue = new BasePanel(this, {
                look: { padding: 2 }
            });
            var txtValue = new TextView(cellValue, {
                look: {
                    borderRadius: 4,
                    bgColor: this.look.bgNormal,
                    txtColor: this.look.txtNormal,
                    txtScale: 0.4,
                    txtAlign: "center"
                }
            });

            //
            this.btn = txtValue;


        }

        redrawLook() {
            super.redrawLook();

            this.btn.setLook({
                bgColor: this.look.bgNormal,
                txtColor: this.look.txtNormal,
                txtScale: this.look.txtScale,
                borderRadius: this.look.borderRadius,
                txt: (this.data && this.data.name) ? this.data.name : ""
            });

        }

    }

    //
    class GenericPicker extends BasePanel {
        constructor(parentView, options, data) {
            super(parentView, options);
            var _this = this;

            // this.ROW = this.look.rowNum;
            // this.COL = this.look.colNum;
            // this.CELL_W = 100 / this.COL;
            // this.CELL_H = 100 / this.ROW;


            // == ui ==
            // this.panel.setRect({
            //     x: (100 - this.rect.box.w) / 2,
            //     y: (100 - this.rect.box.h) / 2,
            //     w: this.rect.box.w,
            //     h: this.rect.box.h,
            // })
            // this.panel.setOverflow({
            //     x: this.look.box.overflowX,
            //     y: this.look.box.overflowY
            // })
            load(data);


            // == methods ==
            function clear() {
                for(var i=0; i<_this.items.length; i++) {
                    _this.items[i].dispose();
                }
                _this.items = [];
            }
            function load(data) {
                var newItem;
                for(var i=0; i<data.length; i++) {
                    newItem = new PickerItem(_this.panel, {
                        look: _this.look.item
                    }, data[i]);
                    newItem.on("itemClicked", function() {
                        _this.emit("itemClicked", this);
                    });
                    _this.items.push(newItem);
                }
                // _this.redrawPanel();
                _this.redraw(true);
            }

            //
            this.clear = clear;
            this.load = load;


            // == events ==
            this.domOn("click", function() {
                _this.hide();
            });



            //
            this.redraw(true);

        }

        init() {
            super.init();

            this.items = [];

            //
            this.rect.box = {
                w: 100,
                h: 100
            };

            //
            this.look.bgColor = "rgba(50, 50, 50, 0.5)";
            //
            this.look.rowNum = 4;
            this.look.colNum = 1;
            this.look.item = {
                bgNormal: BerColor.Brighter,
                bgDown: BerColor.Dark,
                txtNormal: BerColor.Dark,
                txtDown: BerColor.Brighter,
                txtScale: 0.5
            };
            this.look.box = {
                overflowX: "hidden",
                overflowY: "hidden"
            };

            //
            this.panel = new BasePanel(this);

        }

        redrawRect() {
            super.redrawRect();

            this.panel.setRect({
                x: (100 - this.rect.box.w) / 2,
                y: (100 - this.rect.box.h) / 2,
                w: this.rect.box.w,
                h: this.rect.box.h,
            })

            //
            var ROW = this.look.rowNum;
            var COL = this.look.colNum;
            var CELL_W = 100 / COL;
            var CELL_H = 100 / ROW;
            for(var i=0; i<this.items.length; i++) {
                this.items[i].setRect({
                    x: CELL_W * (i % COL),
                    y: CELL_H * Math.floor(i / COL),
                    w: CELL_W,
                    h: CELL_H
                });
            }

        }

        // redrawPanel() {
        //     for(var i=0; i<this.items.length; i++) {
        //         this.items[i].setRect({
        //             x: this.CELL_W * (i % this.COL),
        //             y: this.CELL_H * Math.floor(i / this.COL),
        //             w: this.CELL_W,
        //             h: this.CELL_H
        //         });
        //     }
        //     this.redraw(true);
        // }

        redrawOverflow() {
            super.redrawOverflow();

            this.panel.setOverflow({
                x: this.look.box.overflowX,
                y: this.look.box.overflowY
            });

        }

    }

    //
    return GenericPicker;
})();


// ====
module.exports = GenericPicker;
