
var BerUI = require("../../index");
var TextBoxRow = require("../TextBoxRow");
var SetPanel = require("../SetPanel");
var TextTextRow = require("../TextTextRow");

// ====

// NOTE: 高度根據 parentView 和載入資料的 row 切
//       寬度則根據載入的資料切 (每個 row 不一定相同)

var GridFuncPanel = (function() {
    //
    // itemSettings = [
    //     [col, col, ...], // row
    //     [col, col, ...], // row
    //     ...
    // ];

    //
    class GridFuncPanel extends BerUI.BasePanel {
        constructor(parentView, options, itemSettings) {
            super(parentView, options);
            var _this = this;


            // == ui ==
            this.rows = [];
            var refY = 0;
            var H = 100 / itemSettings.length;
            for(var i=0; i<itemSettings.length; i++) {
                this.rows.push(new BerUI.BasePanel(this.panel, {
                    rect: { y: refY, h: H }
                }));
                refY += H;
            }
            //
            this.items = {};
            for(var i=0; i<itemSettings.length; i++) {
                initItems(this.rows[i], itemSettings[i]);
            }


            // == methods ==
            function initItems(parentView, colSettings) {
                var COL = colSettings.length;// TODO: 做個可以設定寬度跟位置的機制 >> 現在只有寬度, 位置要用隱形元件填補來達成
                var widths = getColWidths(colSettings);// var ITEM_W = 100 / COL;
                var refX = 0;
                var cell, item, options;
                for(var i=0; i<COL; i++) {
                    //
                    cell = new BerUI.BasePanel(parentView, {
                        rect: { x: refX, w: widths[i] },// rect: { x: refX, w: ITEM_W },
                        look: { padding: 2 }
                    });
                    refX += widths[i];// refX += ITEM_W;

                    //
                    item = createItem(cell, colSettings[i]);

                    //
                    _this.items[colSettings[i].id] = item;

                }

            }

            function getColWidths(colSettings) {
                // NOTE: 在此將 colSettings.widthWeight 換算成每個欄位需要的寬度
                //       colSettings.widthWeight 是比重, 預設值都是 1
                //       (在沒額外設定的情況下, 每欄等寬)
                var weights = [];
                var weightSum = 0;
                var widths = [];
                var COL = colSettings.length;
                for(var i=0; i<COL; i++) {
                    if(isUsable(colSettings[i].widthWeight)) {
                        weights.push(colSettings[i].widthWeight);
                        weightSum += colSettings[i].widthWeight;
                    } else {
                        weights.push(1);
                        weightSum += 1;
                    }
                }
                for(var i=0; i<COL; i++) {
                    widths.push(weights[i] / weightSum * 100);
                }
                return widths;
            }

            function createItem(cell, itemData) {
                var item, options;
                switch (itemData.type) { // TODO: 支援更多類型
                    case "textbox":
                        options = (
                            itemData.options ?
                            itemData.options :
                            {
                                look: {
                                    title: itemData.title,
                                    value: itemData.value
                                }
                            }
                        );
                        // if()options.look.title =

                        if(itemData.textType || itemData.readonly) {
                            options.opts = {};
                            if(itemData.textType) options.opts.textType = itemData.textType;
                            if(itemData.readonly) options.opts.readonly = itemData.readonly;
                        }

                        item = new TextBoxRow(cell, options);
                        break;

                    case "setPanel":
                        options = (
                            itemData.options ?
                            itemData.options :
                            {
                                look: {
                                    title: itemData.title,
                                    value: itemData.value
                                }
                            }
                        );
                        item = new SetPanel(cell, options);
                        break;

                    case "text":
                        options = (
                            itemData.options ?
                            itemData.options :
                            {
                                look: {
                                    title: itemData.title,
                                    value: itemData.value
                                }
                            }
                        );
                        item = new TextTextRow(cell, options);
                        break;

                    default:

                }
                return item;
            }


            // == events ==


        }

        init() {
            super.init();

            var panel = new BerUI.BasePanel(this, {
                // look: { padding: 8 }
            });

            //
            this.panel = panel;

        }

        // redrawRect() {
        //     super.redrawRect();
        //
        // }

    }


    //
    return GridFuncPanel;
})();


//
module.exports = GridFuncPanel;
