
var BerUI = require("../../index");
var CheckButton = require("../CheckButton");
var SimplePicBtn = require("../SimplePicBtn");

// ====

var ItemBase = (function() {
    //

    //
    class ItemBase extends BerUI.BasePanel {
        constructor(parentView, options, colSettings, data) {
            super(parentView, options);
            var _this = this;

            this.colSettings = colSettings;
            this.data = data;

            // == ui ==
            this.items = {};

            var COL = colSettings.length;// TODO: 做個可以設定寬度跟位置的機制 >> 現在只有寬度, 位置要用隱形元件填補來達成
            var widths = getColWidths(colSettings);// var ITEM_W = 100 / COL;
            var refX = 0;
            var cell, item, options;
            for(var i=0; i<COL; i++) {
                //
                cell = new BerUI.BasePanel(this.panel, {
                    rect: { x: refX, w: widths[i] },// rect: { x: refX, w: ITEM_W },
                    look: { padding: 2 }
                });
                refX += widths[i];// refX += ITEM_W;

                //
                switch (colSettings[i].type) { // TODO: 支援更多類型
                    case "text":
                        options = (
                            colSettings[i].options ?
                            colSettings[i].options :
                            {
                                look: {
                                    txtScale: 0.5,
                                    txtAlign: "center"
                                }
                            }
                        );
                        item = new BerUI.TextView(cell, options);
                        break;

                    case "textbox":
                        options = (colSettings[i].options ? colSettings[i].options : {});
                        item = new BerUI.TextBoxView(cell, options);
                        // item.domOn("click", function() {
                        //     console.log("test");
                        // }); // TEMP: test
                        break;

                    case "checkbutton":
                        options = (colSettings[i].options ? colSettings[i].options : {});
                        item = new CheckButton(cell, options);
                        break;

                    case "imagebutton":
                        options = (colSettings[i].options ? colSettings[i].options : {});
                        item = new SimplePicBtn(cell, options);
                        (function(ID) {
                            item.on("itemClicked", function() {
                                _this.emit("_cellClicked", ID);
                            });
                        })(colSettings[i].id);
                        break;

                    default:

                }

                //
                this.items[colSettings[i].id] = item;

            }


            // == methods ==
            function getColWidths(colSettings) {
                // NOTE: 在此將 colSettings.widthWeight 換算成每個欄位需要的寬度
                //       colSettings.widthWeight 是比重, 預設值都是 1
                //       (在沒額外設定的情況下, 每欄等寬)
                var weights = [];
                var weightSum = 0;
                var widths = [];
                var COL = colSettings.length;
                for(var i=0; i<COL; i++) {
                    if(isUsable(colSettings[i].widthWeight)) {
                        weights.push(colSettings[i].widthWeight);
                        weightSum += colSettings[i].widthWeight;
                    } else {
                        weights.push(1);
                        weightSum += 1;
                    }
                }
                for(var i=0; i<COL; i++) {
                    widths.push(weights[i] / weightSum * 100);
                }
                return widths;
            }


            // == events ==


        }

        init() {
            super.init();

            this.look.bgColor = BerColor.Brighter;
            this.look.padding = 1;

            this.look.txtColor = BerColor.Mid;


            // == ui ==
            var panel = new BerUI.BasePanel(this, {
                look: {
                    bgColor: BerColor.Brighter
                }
            });

            //
            this.panel = panel;

        }

        redrawLook() {
            super.redrawLook();

            if(!isUsable(this.colSettings)) return;
            for(var i=0; i<this.colSettings.length; i++) {
                if(!isUsable(this.items[this.colSettings[i].id])) continue;

                switch (this.colSettings[i].type) { // TODO: 支援更多類型
                    case "text":
                        this.items[this.colSettings[i].id].setLook({
                            txtColor: this.look.txtColor,
                            txt: isUsable(this.data[this.colSettings[i].id]) ? this.data[this.colSettings[i].id] : "----"
                        });
                        break;

                    case "textbox":
                        this.items[this.colSettings[i].id].setLook({
                            txt: isUsable(this.data[this.colSettings[i].id]) ? this.data[this.colSettings[i].id] : ""
                        });
                        break;

                    case "checkbutton":
                        this.items[this.colSettings[i].id].Checked = (isUsable(this.data[this.colSettings[i].id]) ? this.data[this.colSettings[i].id] : false);
                        break;

                    case "imagebutton":
                        this.items[this.colSettings[i].id].setLook({
                            imgNormal: isUsable(this.data[this.colSettings[i].id]) ? this.data[this.colSettings[i].id] : "",
                            imgActive: isUsable(this.data[this.colSettings[i].id]) ? this.data[this.colSettings[i].id] : ""
                        });
                        break;

                    default:

                }


            }

        }


    }


    //
    return ItemBase;
})();


//
module.exports = ItemBase;
