
var BerUI = require("../../index");

// ====

var ListPanel = (function() {
    //

    //
    class ListPanel extends BerUI.BasePanel {
        constructor(parentView, options, Header, Item) {
            super(parentView, options);
            var _this = this;

            this.data;
            this.cells = [];
            this.items = [];


            // == ui ==
            if(this.opts.showScrollbar) this.scrollPanel.setClassName("wkScroll");


            // == methods ==
            function initHeader(data, colWidths) {
                if(_this.header) return;
                _this.header = new Header(_this.cellHeader, {}, data);
            }

            function clear(opts) {
                for(var i=0; i<_this.items.length; i++) {
                    _this.cells[i].dispose();
                    _this.items[i].dispose();
                }
                _this.cells = [];
                _this.items = [];

                if(opts) {
                    if(opts.clearData) {
                        _this.data = [];
                    }
                } else {
                    _this.data = [];
                }

            }
            function load(data) {
                if(_this.data) {
                    clear();
                    if(Array.isArray(data)) {
                        _this.data = _this.data.concat(data);
                    } else {
                        _this.data.push(data);
                    }
                } else {
                    _this.data = data;
                }

                //
                reload();
            }
            function append(data, toTop) {
                if(_this.data) {
                    clear({ clearData: false });
                    if(Array.isArray(data)) {
                        if(toTop) {
                            _this.data = data.concat(_this.data);
                        } else {
                            _this.data = _this.data.concat(data);
                        }
                    } else {
                        _this.data.push(data);
                    }
                } else {
                    _this.data = data;
                }

                //
                reload();
            }
            function reload() {
                var cell, item;
                for(var i=0; i<_this.data.length; i++) {
                    //
                    if(_this.opts.animate) {
                        cell = new BerUI.BasePanel(_this.scrollPanel, {
                            transform: { scale: 0.5 }
                        });
                    } else {
                        cell = new BerUI.BasePanel(_this.scrollPanel);
                    }

                    item = new Item(cell, {}, _this.data[i]);

                    (function(I) {
                        item.on("itemClicked", function(data) {
                            _this.emit("itemClicked", data, I);
                        });
                        item.on("cellClicked", function(cell_id, data) {
                            _this.emit("cellClicked", cell_id, data, I);
                        });
                    })(i);

                    //
                    _this.cells.push(cell);
                    _this.items.push(item);

                }
                _this.redraw(true);

                //
                if(_this.opts.animate) {
                    var delay = 0;
                    for(var i=0; i<_this.cells.length; i++) {
                        (function(CELL, I) {
                            setTimeout(function() {
                                CELL.animate([{
                                    duration: 150,
                                    transform: { scale: 1.2 }
                                }, {
                                    duration: 80,
                                    transform: { scale: 1 }
                                }]);
                            }, delay + 100*I)
                        })(_this.cells[i], i);
                    }
                }

            }
            function remove(index) {
                _this.cells[index].dispose();
                _this.items[index].dispose();
                _this.cells.splice(index, 1);
                _this.items.splice(index, 1);
                _this.data.splice(index, 1);

                // 為了重置 index
                clear({ clearData : false });
                reload();
            }

            function setItemsLook(look) {
                for(var i=0; i<_this.items.length; i++) {
                    _this.items[i].setLook(look);
                }
            }


            //
            this.initHeader = initHeader;
            this.clear = clear;
            this.load = load;
            this.append = append;
            this.remove = remove;

            this.setItemsLook = setItemsLook;


            // == events ==
            window.addEventListener("resize", function () {
                if(_this.opts.visible) {
                    _this.redraw(true);
                }
            }, false);

            this.scrollPanel.on("atBottom", function() {
                _this.emit("atBottom");
            });


        }


        // == init ==
        init() {
            super.init();

            this.look.bgColor = BerColor.Mid;
            this.opts.animate = true;
            this.opts.showScrollbar = true;

            this.rect.rowPxH = 40; // NOTE: row 的高度 (廢話)
            this.look.itemsLook = null;


            // == ui ==
            var cellHeader = new BerUI.BasePanel(this, {
                // look: { bgColor: BerColor.Mid },
                opts: { zIndex: 1 },
                boxShadow: {
                    visible: true,
                    blur: 1,
                    spread: 0,
                    color: BerColor.Dark
                }
            });
            // var header = new Header(cellHeader);

            //
            var scrollPanel = new BerUI.BasePanel(this, {
                look: {
                    padding: 1,
                    // bgColor: BerColor.Mid
                },
                overflow: {
                    // x: "scroll",
                    y: "scroll"
                }
            });
            // scrollPanel.setClassName("wkScroll");


            //
            this.cellHeader = cellHeader;
            this.scrollPanel = scrollPanel;

            // this.header = header;

        }

        redrawRect() {
            super.redrawRect();

            var ROW_H = this.rect.rowPxH;

            if(this.header) {
                this.cellHeader.show();

                //
                this.cellHeader.setRect({
                    h: ROW_H + "px"
                });
                if(this.opts.showScrollbar) {
                    this.header.setRect({
                        w: (this.PxW - 12) + "px" // NOTE: 12 = width of the scrollbar
                    });
                } else {
                    this.header.setRect({
                        w: this.PxW + "px"
                    });
                }


                this.scrollPanel.setRect({
                    y: ROW_H + "px",
                    h: (this.PxH - ROW_H) + "px"
                });


            } else {
                this.cellHeader.hide();
                this.scrollPanel.setRect({
                    y: 0, h: 100
                });

            }

            //
            if(this.cells && this.items) {
                for(var i=0; i<this.cells.length; i++) {
                    this.cells[i].setRect({
                        y: (ROW_H * i) + "px",
                        h: ROW_H + "px"
                    });
                    this.items[i].redraw(true);
                }
            }

        }

        redrawLook() {
            super.redrawLook();

            this.cellHeader.setLook({ bgColor: this.look.bgColor });
            this.scrollPanel.setLook({ bgColor: this.look.bgColor });

            // TODO: 設定 Item 風格
            if(this.look.itemsLook) {
                this.setItemsLook(this.look.itemsLook);
            }

        }

    }


    //
    return ListPanel;
})();


//
module.exports = ListPanel;
