
var BerUI = require("../index");

// ====

var MenuItem = (function() {
    //

    //
    class MenuItem extends BerUI.BasePanel {
        constructor(parentView, options, data) {
            super(parentView, options);
            var _this = this;
            var target;


            // == ui ==
            load(data);


            // == methods ==
            function onTouchStart() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: "white" });
                //
                target = event.target;
            }
            function onTouchMove() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: _this.look.bgColor });
                target = null;
            }
            function onTouchEnd() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: _this.look.bgColor });
                //
                var endTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target === endTarget) {
                    if(event) {
                        // if(consumeEvent) {
                            event.preventDefault();
                            event.stopPropagation();
                        // }
                        _this.emit("itemClicked", _this.data);
                    }
                }
            }

            function onMouseDown() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: "white" });
            }
            function onMouseEnter() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: _this.look.bgColor });
            }
            function onMouseLeave() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: _this.look.bgColor });
            }
            function onMouseUp() {
                if(_this.opts.disabled) return;
                _this.panel.setLook({ bgColor: _this.look.bgColor });
            }
            function onClick() {
                if(_this.opts.disabled) return;
                _this.emit("itemClicked", _this.data);
            }

            function load(data) {
                _this.data = data;
                if(isUsable(data)) {
                    if(isUsable(data.iconSrc)) _this.look.iconSrc = data.iconSrc;
                    if(isUsable(data.name)) _this.look.name = data.name;
                    if(isUsable(data.count)) setMsgCount(data.count);
                }
                //
                _this.canvasIcon.setLook({ src: _this.look.iconSrc });
                _this.txtName.setLook({
                    txtColor: _this.look.txtColor,
                    txt: _this.look.name
                });

            }
            function setMsgCount(count) {
                _this.data.count = count
                if(count) {
                    _this.txtMsgCount.Value = count;
                    _this.cellMsgCount.show();
                    //
                    _this.redraw(true);

                } else {
                    _this.txtMsgCount.Value = 0;
                    _this.cellMsgCount.hide();

                }
            }

            //
            this.load = load;
            this.setMsgCount = setMsgCount;


            // == events ==
            if("ontouchstart" in window) {
                this.panel.domOn("touchstart", onTouchStart, false);
                this.panel.domOn("touchmove", onTouchMove, false);
                this.panel.domOn("touchend", onTouchEnd, false);

            } else {
                this.panel.domOn("mousedown", onMouseDown);
                this.panel.domOn("mouseenter", onMouseEnter);
                this.panel.domOn("mouseleave", onMouseLeave);
                this.panel.domOn("mouseup", onMouseUp);
                this.panel.domOn("click", onClick);

            }


        }

        init() {
            super.init();

            this.look.bgColor = BerColor.Brighter;
            // extends
            this.look.iconSrc = "img/demo.png";
            this.look.name = "NAME NAME";
            this.look.txtColor = BerColor.Dark;


            // == ui ==
            var panel = new BerUI.BasePanel(this, {
                look: {
                    padding: 16,
                    bgColor: BerColor.Brighter
                }
            });

            // image
            var cellIcon = new BerUI.BasePanel(panel, {
                rect: { h: 75 },
                look: { padding: 8 }
            });
            var _cellIcon = new BerUI.BasePanel(cellIcon);
            var canvasIcon = new BerUI.CanvasView(_cellIcon);

            // name
            var cellName = new BerUI.BasePanel(panel, {
                rect: { y: 75, h: 25 },
                look: { padding: 2 }
            });
            var txtName = new BerUI.TextView(cellName, {
                look: {
                    txtScale: 0.5,
                    txtAlign: "center"
                }
            });

            // count
            var cellMsgCount = new BerUI.BasePanel(this, {
                opts: { visible: false }
            });
            var txtMsgCount = new BerUI.TextView(cellMsgCount, {
                look: {
                    bgColor: "red",
                    txtColor: "white",
                    txtScale: 0.5,
                    txtAlign: "center"
                },
            });


            //
            this.panel = panel;

            this.cellIcon = cellIcon;
            this.cellName = cellName;
            this.cellMsgCount = cellMsgCount;

            this.canvasIcon = canvasIcon;
            this.txtName = txtName;
            this.txtMsgCount = txtMsgCount;

        }

        redrawRect() {
            super.redrawRect();

            if(this.cellMsgCount.opts.visible) {
                var SPACE = 16;
                var CNT_SIZE = 30;
                this.cellMsgCount.setRect({
                    x: (this.PxW - CNT_SIZE - SPACE) + "px",
                    y: SPACE + "px",
                    w: CNT_SIZE + "px",
                    h: CNT_SIZE + "px"
                });
            }

        }

        redrawLook() {
            super.redrawLook();

            if(this.cellMsgCount.opts.visible) {
                this.txtMsgCount.setLook({
                    borderRadius: this.txtMsgCount.PxW / 2
                });
            }

        }

        //
        set IconSrc(src) {
            this.look.iconSrc = src;
            this.canvasIcon.setLook({ src: this.look.iconSrc });
        }
        set Value(val) {
            this.look.name = val;
            this.txtName.setLook({ txt: val });
        }


    }


    //
    return MenuItem;
})();


//
module.exports = MenuItem;
