
var BerUI = require("../../index");
var FuncBar = require("../FuncBar");
var MenuItem = require("./MenuItem");

// ====

var MenuPanel = (function() {
    //

    //
    class MenuPanel extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            this.extraData;

            this.data;
            this.items = [];

            this.ui_state;
            this.user_group;


            // == methods ==
            function clear() {
                for(var i=0; i<_this.items.length; i++) {
                    _this.items[i].dispose();
                }
                _this.items = [];
            }
            function load(data) {
                _this.data = data;
                //
                var cell, item;
                for(var i=0; i<_this.data.length; i++) {
                    //
                    item = new MenuItem(_this.panel, {
                        look: _this.look.item,
                        transform: { scale: 0.5 }
                    }, _this.data[i]);
                    item.on("itemClicked", function(itemData) {
                        _this.emit("itemClicked", itemData, _this.extraData);
                    });
                    //
                    _this.items.push(item);

                }
                _this.redraw(true);

                // NOTE: animation
                var delay = 0;
                for(var i=0; i<_this.items.length; i++) {
                    (function(ITEM, I) {
                        setTimeout(function() {
                            ITEM.animate([{
                                duration: 150,
                                transform: { scale: 1.2 }
                            }, {
                                duration: 80,
                                transform: { scale: 1 }
                            }]);
                        }, delay + 100*I)
                    })(_this.items[i], i);
                }
            }

            function setMsgCount(itemName, cnt) {
                for(var i=0; i<_this.items.length; i++) {
                    if(_this.items[i].data.name == itemName) {
                        _this.items[i].setMsgCount(cnt);
                        break;
                    }
                }
            }
            function clearMsgCount() {
                for(var i=0; i<_this.items.length; i++) {
                    _this.items[i].setMsgCount(0);
                }
                //
                if(isUsable(_this.data)) {
                    for(var i=0; i<_this.data.length; i++) {
                        _this.data[i].count = 0;
                    }
                }

            }

            function setState(ui_state, user_group) {
                // console.log("[ber-ui][setState]", ui_state, user_group);
                _this.ui_state = ui_state;
                _this.user_group = user_group;
                _this.redraw(true);
            }

            //
            this.clear = clear;
            this.load = load;
            this.setMsgCount = setMsgCount;
            this.clearMsgCount = clearMsgCount;
            this.setState = setState;


            // == events ==
            window.addEventListener("resize", function () {
                if(_this.opts.visible) {
                    _this.redraw(true);
                }
            }, false);

            this.funcBar.on("leftClicked", function() {
                _this.emit("funcBarLeftClicked");
            });
            this.funcBar.on("rightClicked", function() {
                _this.emit("funcBarRightClicked");
            });


            // ====
            this.redraw(true);

        }

        init() {
            super.init();

            this.look.funcBarTitle = "";
            this.look.funcBarLeft = "";
            this.look.funcBarRight = "";
            this.look.funcBarBgColor = BerColor.Brighter;

            this.rect.funcBarH = 64;
            this.rect.itemBaseSize = 150;

            this.look.item = {
                txtColor: BerColor.BtnBright,
                bgNormal: "white",
                bgActive: BerColor.Brighter,
            };


            // == ui ==
            var funcBar = new FuncBar(this);

            var panel = new BerUI.BasePanel(this, {
                // look: { paddingScale: 3 },
                overflow: {
                    // x: "scroll",
                    y: "scroll"
                }
            });


            //
            this.funcBar = funcBar;
            this.panel = panel;

        }

        redrawRect() {
            super.redrawRect();

            //
            var FUNCBAR_H = this.rect.funcBarH;//64;
            var PANEL_H = this.parentView.PxH - FUNCBAR_H;
            this.funcBar.setRect({ h: FUNCBAR_H + "px" });
            this.panel.setRect({
                y: FUNCBAR_H + "px",
                h: PANEL_H + "px"
            });

            //
            if(!isUsable(this.items) || this.items.length == 0) return;

            var BASE_SIZE = this.rect.itemBaseSize;//150;
            var CELL_SIZE;
            var COL = Math.floor(this.PxW / BASE_SIZE);
            CELL_SIZE = this.PxW / COL;
            var idxDraw = 0
            for(var i=0; i<this.items.length; i++) {
                if(checkVisible(this.ui_state, this.user_group, this.items[i].data)) {
                    this.items[i].show();
                    this.items[i].setRect({
                        x: CELL_SIZE * (idxDraw % COL) + "px",
                        y: CELL_SIZE * Math.floor(idxDraw / COL) + "px",
                        w: CELL_SIZE + "px",
                        h: CELL_SIZE + "px"
                    });
                    idxDraw++;

                } else {
                    this.items[i].hide();
                    continue;

                }
            }

            // ====
            function checkVisible(ui_state, user_group, item_data) {
                // console.log("[]", ui_state, user_group);
                //
                var pass_state = false;
                if(ui_state) {
                    var visibleState = item_data.visibleState;
                    if(visibleState) {
                        for(var i in visibleState) {
                            if(ui_state == visibleState[i]) {
                                pass_state = true;
                                break;
                            }
                        }
                    } else {
                        pass_state = true; // NOTE: 沒給 visibleState 的視為不篩選
                    }
                } else {
                    pass_state = true; // NOTE: 沒給 ui_state 的視為不篩選
                }

                //
                var pass_group = false;
                if(user_group) {
                    var visibleGroup = item_data.visibleGroup;
                    if(visibleGroup) {
                        for(var i in visibleGroup) {
                            if(user_group == visibleGroup[i]) {
                                pass_group = true;
                                break;
                            }
                        }
                    } else {
                        pass_group = true; // NOTE: 沒給 visibleGroup 的視為不篩選
                    }
                } else {
                    pass_group = true; // NOTE: 沒給 user_group 的視為不篩選
                }

                //
                // console.log(pass_state, pass_group);
                return (pass_state && pass_group);

            }


        }

        redrawLook() {
            super.redrawLook();

            this.funcBar.Title = this.look.funcBarTitle; // NOTE: 這會順便決定是否顯示元件
            this.funcBar.LeftText = this.look.funcBarLeft; // NOTE: 這會順便決定是否顯示元件
            this.funcBar.RightText = this.look.funcBarRight; // NOTE: 這會順便決定是否顯示元件

            this.funcBar.setLook({ bgColor: this.look.funcBarBgColor });

        }

        show(extraData) {
            super.show();
            if(isUsable(extraData)) {
                this.extraData = extraData;
                // this.funcBar.Title = extraData.name;
            }
        }

        //
        set FuncBarTitle(val) {
            this.look.funcBarTitle = val;
            this.funcBar.Title = val;
        }
        set FuncBarLeftText(val) {
            this.look.funcBarLeft = val;
            this.funcBar.LeftText = val;
        }
        set FuncBarRightText(val) {
            this.look.funcBarRight = val;
            this.funcBar.RightText = val;
        }

    }


    //
    return MenuPanel;
})();


//
module.exports = MenuPanel;
