
var BerUI = require("../../index");
var FuncBar = require("../FuncBar");
var MenuItem = require("./MenuItem");

// ====

var MenuPanel = (function() {
    //

    //
    class MenuPanel extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            this.extraData;

            this.data;
            this.cells = [];
            this.items = [];


            // == methods ==
            function clear() {
                for(var i=0; i<_this.items.length; i++) {
                    _this.cells[i].dispose();
                    _this.items[i].dispose();
                }
                _this.cells = [];
                _this.items = [];
            }
            function load(data) {
                _this.data = data;
                //
                var cell, item;
                for(var i=0; i<_this.data.length; i++) {
                    //
                    cell = new BerUI.BasePanel(_this.panel, {
                        // look: { paddingScale: 2 }
                        transform: { scale: 0.5 }
                    });
                    item = new MenuItem(cell, {
                        look: _this.look.item
                    }, _this.data[i]);
                    item.on("itemClicked", function(itemData) {
                        _this.emit("itemClicked", itemData, _this.extraData);
                    });
                    //
                    _this.cells.push(cell);
                    _this.items.push(item);

                }
                _this.redraw(true);

                // NOTE: animation
                var delay = 0;
                for(var i=0; i<_this.cells.length; i++) {
                    (function(CELL, I) {
                        setTimeout(function() {
                            CELL.animate([{
                                duration: 150,
                                transform: { scale: 1.2 }
                            }, {
                                duration: 80,
                                transform: { scale: 1 }
                            }]);
                        }, delay + 100*I)
                    })(_this.cells[i], i);
                }
            }

            function setMsgCount(itemName, cnt) {
                for(var i=0; i<_this.items.length; i++) {
                    if(_this.items[i].data.name == itemName) {
                        _this.items[i].setMsgCount(cnt);
                        break;
                    }
                }
            }
            function clearMsgCount() {
                for(var i=0; i<_this.items.length; i++) {
                    _this.items[i].setMsgCount(0);
                }
                //
                if(isUsable(_this.data)) {
                    for(var i=0; i<_this.data.length; i++) {
                        _this.data[i].count = 0;
                    }
                }

            }

            //
            this.clear = clear;
            this.load = load;
            this.setMsgCount = setMsgCount;
            this.clearMsgCount = clearMsgCount;


            // == events ==
            window.addEventListener("resize", function () {
                if(_this.opts.visible) {
                    _this.redraw(true);
                }
            }, false);

            this.funcBar.on("leftClicked", function() {
                _this.emit("funcBarLeftClicked");
            });
            this.funcBar.on("rightClicked", function() {
                _this.emit("funcBarRightClicked");
            });


            // ====
            this.redraw(true);

        }

        init() {
            super.init();

            this.look.funcBarTitle = "";
            this.look.funcBarLeft = "";
            this.look.funcBarRight = "";

            this.look.item = {
                txtColor: BerColor.BtnBright,
                bgNormal: "white",
                bgActive: BerColor.Brighter,
            };


            // == ui ==
            var funcBar = new FuncBar(this);

            var panel = new BerUI.BasePanel(this, {
                // look: { paddingScale: 3 },
                overflow: {
                    // x: "scroll",
                    y: "scroll"
                }
            });


            //
            this.funcBar = funcBar;
            this.panel = panel;

        }

        redrawRect() {
            super.redrawRect();

            //
            var FUNCBAR_H = 64;
            var PANEL_H = this.parentView.PxH - FUNCBAR_H;
            this.funcBar.setRect({ h: FUNCBAR_H + "px" });
            this.panel.setRect({
                y: FUNCBAR_H + "px",
                h: PANEL_H + "px"
            });


            //
            if(this.cells && this.items) {
                var BASE_SIZE = 150;
                var CELL_SIZE;
                var COL = Math.floor(this.PxW / BASE_SIZE);
                CELL_SIZE = this.PxW / COL;
                for(var i=0; i<this.cells.length; i++) {
                    this.cells[i].setRect({
                        x: CELL_SIZE * (i % COL) + "px",
                        y: CELL_SIZE * Math.floor(i / COL) + "px",
                        w: CELL_SIZE + "px",
                        h: CELL_SIZE + "px"
                    });
                    this.items[i].redraw(true);
                }
            }

        }

        redrawLook() {
            super.redrawLook();

            this.funcBar.Title = this.look.funcBarTitle; // NOTE: 這會順便決定是否顯示元件
            this.funcBar.LeftText = this.look.funcBarLeft; // NOTE: 這會順便決定是否顯示元件
            this.funcBar.RightText = this.look.funcBarRight; // NOTE: 這會順便決定是否顯示元件

        }

        show(extraData) {
            super.show();
            if(isUsable(extraData)) {
                this.extraData = extraData;
                // this.funcBar.Title = extraData.name;
            }
        }

        //
        set FuncBarTitle(val) {
            this.look.funcBarTitle = val;
            this.funcBar.Title = val;
        }
        set FuncBarLeftText(val) {
            this.look.funcBarLeft = val;
            this.funcBar.LeftText = val;
        }
        set FuncBarRightText(val) {
            this.look.funcBarRight = val;
            this.funcBar.RightText = val;
        }

    }


    //
    return MenuPanel;
})();


//
module.exports = MenuPanel;
