
var TextBoxRow = CusUI.TextBoxRow;
var SetPanel = CusUI.SetPanel;
var GenericPicker = CusUI.GenericPicker;
var SearchPanelBase = CusUI.SearchPanelBase;

// ====

var RecordSearchPanel = (function() {
    //
    var itemSettings = {
        row_0: {
            colNum: 6,
            items: [{
                id: "bookingCode",
                type: "textbox",
                title: "訂單編號"
            }, {
                id: "supplier",
                type: "setPanel",
                title: "供應商",
                value: "(載入供應商)"
            }, {
                id: "agentName",
                type: "textbox",
                title: "Agent"
            }, {
                id: "location",
                type: "textbox",
                title: "國家城市"
            }, {
                id: "hotelName",
                type: "textbox",
                title: "飯店"
            }, {
                id: "customerName",
                type: "textbox",
                title: "顧客"
            }]
        },

        row_1: {
            colNum: 6,
            items: [{
                id: "bookingDate",
                type: "setPanel",
                title: "下單",
                value: new Date().toYYYYMMdd()
            }, {
                id: "deadline",
                type: "setPanel",
                title: "保住",
                value: new Date().toYYYYMMdd()
            }, {
                id: "checkIn",
                type: "setPanel",
                title: "入住",
                value: new Date().toYYYYMMdd()
            }, {
                id: "checkOut",
                type: "setPanel",
                title: "退房",
                value: new Date().toYYYYMMdd()
            }, {
                id: "status",
                type: "setPanel",
                title: "狀態",
                value: "----"
            }]
        },

        // row_2: {
        //     colNum: 6,
        //     items: [{
        //         id: "bookingDate",
        //         type: "text"
        //     }, {
        //         id: "deadline",
        //         type: "text"
        //     }]
        // },

        row_2_control: {
            colNum: 12,
            fromRight: true,
            items: [{
                id: "search",
                type: "setPanel",
                title: "",
                value: "搜尋"
            }, {
                id: "reset",
                type: "setPanel",
                title: "",
                value: "重置"
            }]
        },

    };

    //
    class RecordSearchPanel extends SearchPanelBase {
        constructor(parentView, options) {
            super(parentView, options, itemSettings);
            var _this = this;


            // == ui ==
            var datePicker, datePickerTarget, supplierPicker, statusPicker;


            // == methods ==
            function showDatePicker(setPanel) {
                if(!datePicker) initDatePicker();
                datePickerTarget = setPanel;
                var date = new Date(setPanel.Value);
                datePicker.setDate(date.getFullYear(), date.getMonth()+1);
                datePicker.show();

                // ==
                function initDatePicker() {
                    var now = new Date();
                    datePicker = new BerUI.Calendar(BerRoot, {
                        rect: {
                            constW: 350,
                            constH: 375,
                        },
                        opts: { zIndex: 99 }
                    });
                    datePicker.setDate(now.getFullYear(), now.getMonth()+1);
                    datePicker.on("dateSelected", function(date) {
                        // console.log(date);
                        datePicker.hide();
                        datePickerTarget.Value = date.toYYYYMMdd();
                    });

                    datePicker.on("maskClicked", function() {
                        datePicker.hide();
                    });

                    var target;
                    datePicker.domOn("touchstart", function() {
                        target = event.target;
                    });
                    datePicker.domOn("touchmove", function() {
                        target = null;
                    });
                    datePicker.domOn("touchend", function() {
                        var endTarget = document.elementFromPoint(
                            event.changedTouches[0].pageX,
                            event.changedTouches[0].pageY
                        );
                        if(target === endTarget) {
                            if(event) {
                                event.preventDefault();
                                event.stopPropagation();
                                datePicker.hide();
                            }
                        }
                    });

                }
            }

            function showSupplierPicker() {
                if(!supplierPicker) initSupplierPicker();
                supplierPicker.show();

                // ==
                function initSupplierPicker() {
                    supplierPicker = new GenericPicker(BerRoot, {
                        rect: {
                            box: { w: 80, h: 80 }
                        },
                        look: {
                            rowNum: 3,
                            colNum: 1,
                            item: {
                                bgNormal: BerColor.Brighter,
                                bgDown: BerColor.BtnBright,
                                txtNormal: BerColor.BtnBright,
                                txtDown: BerColor.Brighter,
                                txtScale: 0.2,
                            },
                            box: {
                                // overflowX: "scroll",
                                overflowY: "scroll"
                            }
                        },
                        opts: {
                            zIndex: 99,
                            visible: false
                        }
                    }, [
                        { name: "AAA" },
                        { name: "BBB" },
                        { name: "CCC" },
                    ]);

                    supplierPicker.on("itemClicked", function(item) {
                        // console.log(item);
                        supplierPicker.hide();
                        _this.itemGridPanel.items.supplier.Value = item.data.name;

                        // //
                        // clear();
                        // reqList();

                    });

                }
            }

            function showStatusPicker() {
                if(!statusPicker) initStatusPicker();
                statusPicker.show();

                // ==
                function initStatusPicker() {
                    statusPicker = new GenericPicker(BerRoot, {
                        rect: {
                            box: { w: 80, h: 80 }
                        },
                        look: {
                            rowNum: 3,
                            colNum: 1,
                            item: {
                                bgNormal: BerColor.Brighter,
                                bgDown: BerColor.BtnBright,
                                txtNormal: BerColor.BtnBright,
                                txtDown: BerColor.Brighter,
                                txtScale: 0.2,
                            },
                            box: {
                                // overflowX: "scroll",
                                overflowY: "scroll"
                            }
                        },
                        opts: {
                            zIndex: 99,
                            visible: false
                        }
                    }, [
                        { name: "AAA" },
                        { name: "BBB" },
                        { name: "CCC" },
                    ]);

                    statusPicker.on("itemClicked", function(item) {
                        // console.log(item);
                        statusPicker.hide();
                        this.items.status.Value = item.data.name;

                        // //
                        // clear();
                        // reqList();

                    });

                }
            }

            // == events ==
            this.items.supplier.on("itemClicked", function() {
                showSupplierPicker();
            });

            this.items.bookingDate.on("itemClicked", function() {
                showDatePicker(_this.items.bookingDate);
            });
            this.items.deadline.on("itemClicked", function() {
                showDatePicker(_this.items.deadline);
            });
            this.items.checkIn.on("itemClicked", function() {
                showDatePicker(_this.items.checkIn);
            });
            this.items.checkOut.on("itemClicked", function() {
                showDatePicker(_this.items.checkOut);
            });

            this.items.status.on("itemClicked", function() {
                showStatusPicker();
            });

            this.items.reset.on("itemClicked", function() {
                console.log("reset");
            });
            this.items.search.on("itemClicked", function() {
                console.log("search");
            });


        }

        init() {
            super.init();

        }

    }


    //
    return RecordSearchPanel;
})();


//
module.exports = RecordSearchPanel;
