
var BerUI = require("../../index");
var TextBoxRow = require("../TextBoxRow");
var SetPanel = require("../SetPanel");
var TextTextRow = require("../TextTextRow");

// ====

var SearchPanelBase = (function() {
    //

    //
    class SearchPanelBase extends BerUI.BasePanel {
        constructor(parentView, options, itemSettings) {
            super(parentView, options);
            var _this = this;


            // == ui ==
            this.items = {};
            if(itemSettings.row_0) initItems(this.row_0, itemSettings.row_0);
            if(itemSettings.row_1) initItems(this.row_1, itemSettings.row_1);
            if(itemSettings.row_2) initItems(this.row_2, itemSettings.row_2);
            if(itemSettings.row_2_control) initItems(this.row_2, itemSettings.row_2_control);


            // == methods ==
            function initItems(parentView, data) {
                var COL = data.colNum;
                var ITEM_W = 100 / COL;
                var refX = 0;
                var cell, item;

                if(data.fromRight) {
                    refX = ITEM_W;
                    for(var i=0; i<data.items.length; i++) {
                        cell = new BerUI.BasePanel(parentView, {
                            rect: { x: (100 - refX), w: ITEM_W },
                            look: { padding: 3 }
                        });

                        item = createItem(cell, data.items[i]);

                        _this.items[data.items[i].id] = item;
                        refX += ITEM_W;

                    }

                } else {
                    for(var i=0; i<data.items.length; i++) {
                        cell = new BerUI.BasePanel(parentView, {
                            rect: { x: refX, w: ITEM_W },
                            look: { padding: 3 }
                        });

                        item = createItem(cell, data.items[i]);

                        _this.items[data.items[i].id] = item;
                        refX += ITEM_W;

                    }

                }

            }

            function createItem(cell, itemData) {
                var item;
                switch (itemData.type) { // TODO: 支援更多類型
                    case "textbox":
                        item = new TextBoxRow(cell, {
                            look: {
                                title: itemData.title
                            }
                        });
                        if(itemData.textType) item.setOpts({ textType: itemData.textType });
                        break;

                    case "setPanel":
                        item = new SetPanel(cell, {
                            look: {
                                title: itemData.title,
                                value: itemData.value
                            }
                        });
                        break;

                    case "text":
                        item = new TextTextRow(cell, {
                            look: {
                                title: itemData.title,
                                value: itemData.value
                            }
                        });
                        break;

                    default:

                }
                return item;
            }


            // == events ==


        }

        init() {
            super.init();

            var panel = new BerUI.BasePanel(this, {
                look: { padding: 8 }
            });

            //
            var row_0 = new BerUI.BasePanel(panel);
            var row_1 = new BerUI.BasePanel(panel);
            var row_2 = new BerUI.BasePanel(panel);

            //
            this.row_0 = row_0;
            this.row_1 = row_1;
            this.row_2 = row_2;


        }

        redrawRect() {
            super.redrawRect();

            var BASE_H = 60;

            this.row_0.setRect({
                h: BASE_H + "px"
            });

            this.row_1.setRect({
                y: BASE_H + "px",
                h: BASE_H + "px"
            });

            this.row_2.setRect({
                y: BASE_H * 2 + "px",
                h: BASE_H + "px"
            });

        }

    }


    //
    return SearchPanelBase;
})();


//
module.exports = SearchPanelBase;
