
var BerUI = require("../index");

// ====

var SetPanel = (function() {
    //

    //
    class SetPanel extends BerUI.BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var target;


            // == ui ==
            this.txtTitle.Value = this.look.title;
            this.txtTitle.setLook({
                // bgColor: this.look.normalBgColor,
                txtColor: this.look.titleTxtColor,
                txt: this.look.title
            });
            this.txtValue.setLook({
                bgColor: this.look.normalBgColor,
                txtColor: this.look.activeBgColor,
                txt: this.look.value
            });


            // == methods ==
            function reset() {
                _this.txtValue.Value = this.look.value;
            }

            function onTouchStart(event) {
                if(_this.opts.disabled) return;
                //
                _this.txtValue.setLook({
                    bgColor: _this.look.activeBgColor,
                    txtColor: _this.look.normalBgColor
                });
                target = event.target;
            }
            function onTouchMove(event) {
                if(_this.opts.disabled) return;
                //
                var moveTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target !== moveTarget) { // NOTE leave
                    _this.txtValue.setLook({
                        bgColor: _this.look.normalBgColor,
                        txtColor: _this.look.activeBgColor
                    });
                    target = null;
                }
            }
            function onTouchEnd(event) {
                if(_this.opts.disabled) return;
                _this.txtValue.setLook({
                    bgColor: _this.look.normalBgColor,
                    txtColor: _this.look.activeBgColor
                });
                //
                var endTarget = document.elementFromPoint(
                    event.changedTouches[0].pageX,
                    event.changedTouches[0].pageY
                );
                if(target === endTarget) {
                    if(event) {
                        // if(consumeEvent) {
                            event.preventDefault();
                            event.stopPropagation();
                        // }
                        _this.emit("itemClicked", _this.txtValue.Value);
                    }
                }

            }

            function onMouseDown() {
                if(_this.opts.disabled) return;
                _this.txtValue.setLook({
                    bgColor: _this.look.activeBgColor,
                    txtColor: _this.look.normalBgColor
                });
            }
            function onMouseEnter() {
                if(_this.opts.disabled) return;
                // _this.txt.setLook({ bgColor: _this.look.bgNormal }); // TODO add EnterStyle
            }
            function onMouseLeave() {
                if(_this.opts.disabled) return;
                _this.txtValue.setLook({
                    bgColor: _this.look.normalBgColor,
                    txtColor: _this.look.activeBgColor
                });
            }
            function onMouseUp() {
                if(_this.opts.disabled) return;
                _this.txtValue.setLook({
                    bgColor: _this.look.normalBgColor,
                    txtColor: _this.look.activeBgColor
                });
            }
            function onClick() {
                if(_this.opts.disabled) return;
                _this.emit("itemClicked", _this.txtValue.Value);
            }

            //
            this.reset = reset;


            // == events ==
            if("ontouchstart" in window) {
                this.txtValue.domOn("touchstart", onTouchStart, false);
                this.txtValue.domOn("touchmove", onTouchMove, false);
                this.txtValue.domOn("touchend", onTouchEnd, false);
            } else {
                this.txtValue.domOn("mousedown", onMouseDown);
                this.txtValue.domOn("mouseenter", onMouseEnter);
                this.txtValue.domOn("mouseleave", onMouseLeave);
                this.txtValue.domOn("mouseup", onMouseUp);
                this.txtValue.domOn("click", onClick);
            }


            // //
            // setTimeout(function() {
            //     _this.redraw(true);
            // }, 100);

        }


        // == init ==
        init() {
            super.init();
            
            // extends
            this.look.normalBgColor = BerColor.BtnBg;
            this.look.activeBgColor = BerColor.BtnText;
            this.look.title = "----";
            this.look.value = "----";
            this.look.titleTxtColor = BerColor.Mid;


            // == ui ==
            var panel = new BerUI.BasePanel(this, {
                look: { padding: 2 }
            });

            var cellTitle = new BerUI.BasePanel(panel, {
                rect: { h: 30 },
                // look: { padding: 1 }
            });
            var txtTitle = new BerUI.TextView(cellTitle, {
                look: {
                    // txtColor: BerColor.Mid,
                    txtScale: 0.7,
                    txtAlign: "left",
                    txt: this.look.title
                }
            });

            var cellValue = new BerUI.BasePanel(panel, {
                rect: { y: 30, h: 70 },
                look: { padding: 2 }
            });
            var txtValue = new BerUI.TextView(cellValue, {
                look: {
                    borderRadius: 4,
                    bgColor: this.look.normalBgColor,
                    txtColor: BerColor.Dark,
                    txtScale: 0.4,
                    txtAlign: "center",
                    txt: this.look.value
                }
            });

            //
            this.txtTitle = txtTitle;
            this.txtValue = txtValue;

        }

        //
        set Value(val) {
            this.txtValue.Value = val;
        }
        get Value() {
            return this.txtValue.Value;
        }


    }


    //
    return SetPanel;
})();


//
module.exports = SetPanel;
