
var TextView = require("./TextView");

// ====

var ButtonView = (function() {
    //

    //
    class ButtonView extends TextView {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            //
            function updateLook() {
                if(_this.anim.isAnimating()) return; // NOTE 不然 bgColor 和 txtColor 的動畫會被下面的設定干擾
                //
                if(_this.opts.disabled) {
                    _this.setLook({
                        bgColor: _this.look.bgDisabled,
                        txtColor: _this.look.txtDisabled,
                        bgImage: _this.look.imgDisabled
                    });

                } else {
                    switch (_this.state) {
                        case "enter":
                            _this.setLook({
                                bgColor: _this.look.bgEnter,
                                txtColor: _this.look.txtEnter,
                                bgImage: _this.look.imgEnter
                            });
                            break;

                        case "down":
                            _this.setLook({
                                bgColor: _this.look.bgDown,
                                txtColor: _this.look.txtDown,
                                bgImage: _this.look.imgDown
                            });
                            break;

                        default:   // normal
                            _this.setLook({
                                bgColor: _this.look.bgNormal,
                                txtColor: _this.look.txtNormal,
                                bgImage: _this.look.imgNormal
                            });

                    }

                }

            }


            // == events ==
            // mouse
            this.domOn("click", function() {
                if(_this.opts.disabled) return;
                _this.emit("click", _this); // NOTE mouse 跟 touch 統一在這邊轉換 (總之會都從這裡發出)
            });
            this.domOn("mouseleave", function() {
                if(_this.opts.disabled) return;
                _this.mouseover = false;
                _this.state = "normal";
                updateLook();
            });
            this.domOn("mouseenter", function() {
                if(_this.opts.disabled) return;
                _this.mouseover = true;
                _this.state = "enter";
                updateLook();
            });
            this.domOn("mousedown", function() {
                if(_this.opts.disabled) return;
                _this.state = "down";
                updateLook();
                // _this.emit("click", _this);
            });
            this.domOn("mouseup", function() {
                if(_this.opts.disabled) return;
                if(_this.mouseover) {
                    _this.state = "enter";
                } else {
                    _this.state = "normal";
                }
                updateLook();
            });

            // touch
            this.domOn("touchstart", function() {
                if(_this.opts.disabled) return;
                _this.state = "down";
                updateLook();
                // _this.emit("click", _this);  // NOTE touch 系列在 CoreView 有針對 click 事件做轉換, 故在此不用管 (否則會變成送出兩次)
            });
            this.domOn("touchend", function() {
                if(_this.opts.disabled) return;
                _this.mouseover = false;
                _this.state = "normal";
                updateLook();
            });
            this.domOn("touchmove", function() {
                if(_this.opts.disabled) return;
                _this.mouseover = false;
                _this.state = "normal";
                updateLook();
            });

            //
            this.on("anim_end", function() {
                _this.redraw();
            });

        }

        // == init ==
        init(options) {
            super.init();

            // extend
            this.look.bgNormal = "rgb(106, 192, 215)";
            this.look.bgEnter = "rgb(200, 200, 200)";
            this.look.bgDown = "rgb(100, 100, 100)";
            this.look.bgDisabled = "rgb(200, 200, 200)";
            this.look.txtNormal = "rgb(240, 240, 240)";
            this.look.txtEnter = "rgb(100, 100, 100)";
            this.look.txtDown = "rgb(200, 200, 200)";
            this.look.txtDisabled = "rgb(220, 220, 220)";
            this.look.imgNormal = null;
            this.look.imgEnter = null;
            this.look.imgDown = null;
            this.look.imgDisabled = null;

            // inherit
            this.opts.cursor = "pointer";   // 改預設值, 非新增

            //
            this.state = "normal";          // normal, enter, down
            this.mouseover = false;

        }


        // == redraw ==

        // look
        redrawLook() {
            if(this.anim.isAnimating()) { // NOTE 不然 bgColor 和 txtColor 的動畫會被下面的設定干擾
                super.redrawLook();

            } else {
                //
                if(this.opts.disabled) {
                    this.look.bgColor = this.look.bgDisabled;
                    this.look.txtColor = this.look.txtDisabled;
                    this.look.bgImage = this.look.imgDisabled;

                } else {
                    switch (this.state) {
                        case "enter":
                            this.look.bgColor = this.look.bgEnter;
                            this.look.txtColor = this.look.txtEnter;
                            this.look.bgImage = this.look.imgEnter;
                            break;

                        case "down":
                            this.look.bgColor = this.look.bgDown;
                            this.look.txtColor = this.look.txtDown;
                            this.look.bgImage = this.look.imgDown;
                            break;

                        default:   // normal
                            this.look.bgColor = this.look.bgNormal;
                            this.look.txtColor = this.look.txtNormal;
                            this.look.bgImage = this.look.imgNormal;

                    }
                }

                //
                super.redrawLook();
            }

        }


        //
        get Value() {
            return this.look.txt;
        }
        set Value(val) {
            this.setLook({ txt: val });
        }


        // ====
        show() {
            this.setOpts({ visible: true });
        }
        hide() {
            this.setOpts({ visible: false });
        }

        setDisabled(disabled) {
            this.setOpts({ disabled: disabled });
            this.redraw(true);
        }

        setEnabled(enabled) {
            this.setOpts({ disabled: !enabled });
            this.redraw(true);
        }

    }


    //
    return ButtonView;
})();


//
module.exports = ButtonView;
