
var CoreView = require("../core/CoreView");

// ====

var CanvasVideoView = (function() {
    //

    //
    class CanvasVideoView extends CoreView {
        constructor(parentView, options) {
            super("canvas", parentView, options);
            var _this = this;

            var ms_per_frame = 1000 / this.opts.fps;
            var looper = new Looper(updateFrame, ms_per_frame);

            //
            var context2D =  this.core.getContext("2d");
            var videoView = this.opts.video;
            videoView.core.crossOrigin = "anonymous";// TEST


            // == methods ==
            // NOTE 只公開使用 none 與 auto
            function update(mode) {
                if(!videoView) return;

                //
                console.log(videoView.core.videoWidth, videoView.core.videoHeight);

                var video = videoView.core;   // NOTE 只是用來調整畫面, 所以直接取代舊版
                var VIDEO_W = videoView.core.videoWidth;
                var VIDEO_H = videoView.core.videoHeight;

                switch (mode) {
                    case "fit_w": //符合元件寬度, 調整高度以符合比例
                        var viewSize = { w: parentView.PxW, h: parentView.PxH }; // NOTE 直接參考 parentView 的尺寸, 避免自己上次尺寸變動造成基準值不同.

                        var picSize = { w: VIDEO_W, h: VIDEO_H };
                        var newSize = {};
                        newSize.w = viewSize.w;
                        newSize.h = viewSize.w / (picSize.w / picSize.h);

                        // NOTE 若調整完的圖片高度超過元件, 則再做一次高度調整
                        if(newSize.h > viewSize.h) {
                            update("fit_h");
                            break;
                        }

                        _this.setRect({
                            x: (viewSize.w - newSize.w) / 2,
                            y: (viewSize.h - newSize.h) / 2,
                            w: newSize.w,
                            h: newSize.h,
                            mode: "px"
                        });

                        _this.core.width = VIDEO_W;
                        _this.core.height = VIDEO_H;
                        context2D.drawImage(video, 0, 0);
                        break;

                    case "fit_h": //符合元件高度, 調整寬度以符合比例
                        var viewSize = { w: parentView.PxW, h: parentView.PxH }; // NOTE 直接參考 parentView 的尺寸, 避免自己上次尺寸變動造成基準值不同.

                        var picSize = { w: VIDEO_W, h: VIDEO_H };
                        var newSize = {};
                        newSize.w = viewSize.h / (picSize.h / picSize.w);
                        newSize.h = viewSize.h;

                        // NOTE 若調整完的圖片寬度超過元件, 則再做一次寬度調整
                        if(newSize.w > viewSize.w) {
                            update("fit_w");
                            break;
                        }

                        _this.setRect({
                            x: (viewSize.w - newSize.w) / 2,
                            y: (viewSize.h - newSize.h) / 2,
                            w: newSize.w,
                            h: newSize.h,
                            mode: "px"
                        });

                        _this.core.width = VIDEO_W;
                        _this.core.height = VIDEO_H;
                        context2D.drawImage(video, 0, 0);
                        break;

                    case "auto":
                        if(VIDEO_W >= VIDEO_H) {
                            update("fit_w");
                        } else {
                            update("fit_h");
                        }
                        break;


                    default: // origin
                        console.log("origin");
                        _this.core.width = _this.PxW;
                        _this.core.height = _this.PxH;
                        context2D.drawImage(video, 0, 0);
                        break;
                }
            }
            function clear() {
                context2D.clearRect(0, 0, _this.core.width, _this.core.height);
            }
            function fillColor(color) {
                context2D.fillStyle = color;
                context2D.fillRect(0, 0, _this.core.width, _this.core.height);
            }

            function updateFrame() {
                context2D.drawImage(videoView.core, 0, 0);

                _this.emit("frame", _this.core.toDataURL());
                // _this.emit("frame", _this.core.toDataURL("image/jpeg", 0.85));

            }
            function start() {
                looper.start();
            }
            function stop() {
                looper.stop();
            }
            //
            this.clear = clear;
            this.fillColor = fillColor;

            this.start = start;
            this.stop = stop;


            // == events ==
            // this.on("redraw", function() {
            //     update(_this.look.fillMode);
            // });
            videoView.domOn("loadeddata", function() {
                console.log("loadeddata", videoView.core.videoWidth, videoView.core.videoHeight);
                _this.emit("loadeddata", videoView.core.videoWidth, videoView.core.videoHeight);

                // update(_this.look.fillMode);

                // NOTE: 因為不需要管本體的 canvas 怎麼顯示, 所以只要在這調整一次符合 video 的尺寸即可
                _this.setRect({
                    x: 0,
                    y: 0,
                    w: videoView.core.videoWidth,
                    h: videoView.core.videoHeight,
                    mode: "px"
                });
                _this.core.width = videoView.core.videoWidth;
                _this.core.height = videoView.core.videoHeight;
                context2D.drawImage(videoView.core, 0, 0);

            });

        }

        // == init ==
        init(options) {
            super.init();

            // // look
            // this.look.src = "";
            this.look.fillMode = "auto";       // none, fit_w, fit_h, auto >> NOTE 目前只公開使用 none 與 auto

            //
            this.opts.fps = 10;
            this.opts.video = null;

        }


    }


    //
    return CanvasVideoView;
})();


//
module.exports = CanvasVideoView;
