
var CoreView = require("../core/CoreView");

// ====

var DivView = (function() {
    //

    //
    class DivView extends CoreView {
        constructor(parentView, options) {
            super("div", parentView, options);
            var _this = this;

            this.isScrolling = false;
            var scrollingChecker = new Timer(function() {
                _this.isScrolling = false;
                // console.log(_this.isScrolling);
            }, 300);



            // NOTE events
            // [atRight] 捲到最右時 emit
            // [atBottom] 捲到底時 emit
            this.domOn("scroll", function() {
                //
                scrollingChecker.start();
                _this.isScrolling = true;

                //
                _this.scroll.x = _this.core.scrollLeft;
                _this.scroll.y = _this.core.scrollTop;
                _this.emit("scroll", _this.scroll);

                // left
                if(_this.core.scrollLeft == 0) {
                    if(!_this.isAtLeft) {
                        _this.isAtLeft = true;
                        _this.emit("atLeft");
                    }

                } else {
                    _this.isAtLeft = false;
                }

                // top
                if(_this.core.scrollTop == 0) {
                    if(!_this.isAtTop) {
                        _this.isAtTop = true;
                        _this.emit("atTop");
                    }

                } else {
                    _this.isAtTop = false;
                }

                // right
                if((_this.core.scrollWidth - (_this.core.clientWidth + _this.core.scrollLeft)) < 2) {
                    if(!_this.isAtRight) {
                        _this.isAtRight = true;
                        _this.emit("atRight");
                    }

                } else {
                    _this.isAtRight = false;
                }

                // bottom
                if(( _this.core.scrollHeight - (_this.core.clientHeight + _this.core.scrollTop) ) < 2) {
                    if(!_this.isAtBottom) {
                        _this.isAtBottom = true;
                        _this.emit("atBottom");
                    }

                } else {
                    _this.isAtBottom = false;
                }

            });

        }

        // == init ==
        init() {
            super.init();

            //
            this.core.style.overflowX = "hidden";
            this.core.style.overflowY = "hidden";

            //
            // NOTE visible 要兩個一起才有用, 否則單個 visible 會變成 scroll
            this.overflow = {
                x: "hidden",    // hidden, visible, scroll
                y: "hidden"
            };
            this.scroll = {
                x: 0,
                y: 0
            };

            this.isAtTop = true;
            this.isAtLeft = true;
            this.isAtRight = false;
            this.isAtBottom = false;

        }
        setOptions(options) {
            super.setOptions(options);
            //
            // NOTE 自己專有的部分直接執行就好
            if(isUsable(options)) {
                if(isUsable(options.overflow)) this.setOverflow(options.overflow);
                if(isUsable(options.scroll)) this.setScroll(options.scroll);
            }
        }

        // == redraw ==

        // overflow // NOTE visible 要兩個一起才有用, 否則單個 visible 會變成 scroll
        setOverflow(overflow, setOnly) {
            assignToTargetOwnProps(overflow, this.overflow);
            if(!(isUsable(setOnly) && setOnly)) this.redrawOverflow();
        }
        redrawOverflow() {
            this.core.style.overflowX = this.overflow.x;
            this.core.style.overflowY = this.overflow.y;
        }

        // scroll
        setScroll(scroll, setOnly) {
            assignToTargetOwnProps(scroll, this.scroll);
            if(!(isUsable(setOnly) && setOnly)) this.redrawScroll();
        }
        redrawScroll() {
            // x
            if(this.scroll.x == "max") {
                this.scrollToRight();
            } else {
                this.core.scrollLeft = this.scroll.x;
            }

            // y
            if(this.scroll.y == "max") {
                this.scrollToBottom();
            } else {
                this.core.scrollTop = this.scroll.y;
            }

        }
        scrollToTop() {
            this.core.scrollTop = 0;
        }
        scrollToBottom() {
            this.core.scrollTop = this.core.scrollHeight;
        }
        scrollToLeft() {
            this.core.scrollLeft = 0;
        }
        scrollToRight() {
            this.core.scrollLeft = this.core.scrollWidth;
        }


        // ==
        redraw(redrawChild) {
            super.redraw(redrawChild);

            this.redrawOverflow();
            this.redrawScroll();

        }


    }


    //
    return DivView;
})();


//
module.exports = DivView;
