
var CoreView = require("../core/CoreView");

// ====

var FileView = (function() {
    // NOTE 就算是多選模式, 也只會轉換第一個檔案
    // NOTE 檔案類型檢查現在只有 audio/* image/* video/* , 可能需要更多
    // TODO 需要更好的檔案類型檢查方式

    //
    class FileView extends CoreView {
        constructor(parentView, options) {
            super("file", parentView, options);
            var _this = this;


            // == methods ==
            function fileToDataURL(file, callback) {
                var reader = new FileReader();
                reader.onload = function (e) {
                    var dataURL = e.target.result;
                    callback(dataURL);
                }
                reader.readAsDataURL(file);
            }


            // == events ==
            this.domOn("change", function() {
                var typeChecked = true;

                if(_this.typeCheckingString != "") {
                    for(var i=0; i<_this.core.files.length; i++) {
                        if(!_this.core.files[i].type.toString().startsWith(_this.typeCheckingString)) {
                            typeChecked = false;
                            break;
                        }
                    }

                }

                //
                if(typeChecked) {
                    if(_this.core.files[0]) {
                        fileToDataURL(_this.core.files[0], function(dataURL) {
                            _this.dataURL = dataURL;
                            _this.emit("load", null, dataURL, _this.core.files[0]);
                        });
                    }

                } else {
                    _this.dataURL = "";
                    _this.emit("load", "type_err");

                }

                //
                _this.emit("change", _this.core.files);

            });


        }

        // == init ==
        init(options) {
            super.init();

            // opts
            this.opts.accept = "";      // file_extension, audio/*, image/*, video/*, media_type  >> ref: https://www.w3schools.com/tags/att_input_accept.aspt
            this.opts.multiple = false;

            //
            this.typeCheckingString = "";
            this.dataURL = "";

        }


        // == redraw ==

        // opts
        redrawOpts() {
            super.redrawOpts();

            //
            this.core.accept = this.opts.accept;
            switch (this.opts.accept) {
                case "audio/*":
                    this.typeCheckingString = "audio/";
                    break;

                case "image/*":
                    this.typeCheckingString = "image/";
                    break;

                case "video/*":
                    this.typeCheckingString = "video/";
                    break;

                default:
                    this.typeCheckingString = "";

            }

            //
            if (this.opts.multiple) {
                this.core.setAttribute("multiple", "multiple");
            } else {
                this.core.removeAttribute("multiple");
            }

        }

        // ====
        click() {
            this.core.click();
        }
        clear() {
            this.core.value = "";
        }


    }


    //
    return FileView;
})();


//
module.exports = FileView;
