
var EventEmitter = require("./EventEmitter");

// ====

// NOTE transform 的 translate 系列, 元件的 rect 必須設定在顯示中該出現的位置. 若預設是藏起來的, 則在初始時用 transform 來藏.

var Animation = (function() {
    //
    function getRGBA(colorStr) {
        var color = {
            r: 0,
            g: 0,
            b: 0,
            a: 1
        };
        var str = colorStr.trim().toLowerCase();
        var temp = str.split("(")[1];
        temp = temp.split(")")[0];
        temp = temp.split(",");

        if(str.startsWith("rgba")) {
            if(temp.length != 4) return null;
            color.r = parseInt(temp[0]);
            color.g = parseInt(temp[1]);
            color.b = parseInt(temp[2]);
            color.a = parseInt(temp[3]);

        } else if(str.startsWith("rgb")) {
            if(temp.length != 3) return null;
            color.r = parseInt(temp[0]);
            color.g = parseInt(temp[1]);
            color.b = parseInt(temp[2]);

        } else {
            return null;
        }

        return color;

        // TODO 現在只能處理一種字串 ( rgb(x,x,x) & rgba(x,x,x,x) ) , 待擴充
    }

    //
    class Animation extends EventEmitter {
        constructor(view) {
            super();

            var _this = this;
            var view = view;

            var run = false;
            var duration = 500;
            var startTime = null;
            var passingTime = null;
            var progress = null;       // 0~1

            var current;
            var queue = [];

            var loop = false;

            // ====

            function setAnimaion(anim) {
                // core
                var newAnim = {};
                newAnim.duration = (isUsable(anim.duration) ? anim.duration : 500);

                // NOTE 當 anim 裡的某組值, 至少一個有被設定, 而其他沒被設定時(例: anim.transform.translateX 有值, 但 translateY 及 translateZ 沒有), 沒被設定的值需要從 view 取.
                //      但 animation 有可能是透過 setAnimaions 一次設定多組.
                //      此時上述沒被設定的值, 若在此處透過 view 來取, 就只能取到第一個 animation 執行前的值. 可能會不符合實際狀況.
                //      因此發生上述狀況時, 要在 animate 處才取值.
                // NOTE 上述的一組, 指的是像 anim.visibility 或 anim.transform, 各自為一組.

                // visibility
                if(isUsable(anim.visibility)) {
                    newAnim.visibility = {};
                    newAnim.visibility.toValue = anim.visibility;
                }

                // transform
                if(isUsable(anim.transform)) {
                    newAnim.transform = {};

                    // translate
                    if(isUsable(anim.transform.translateX)) newAnim.transform.toTranslateX = anim.transform.translateX;
                    if(isUsable(anim.transform.translateY)) newAnim.transform.toTranslateY = anim.transform.translateY;
                    if(isUsable(anim.transform.translateZ)) newAnim.transform.toTranslateZ = anim.transform.translateZ;

                    // rotate
                    if(isUsable(anim.transform.rotateX)) newAnim.transform.toRotateX = anim.transform.rotateX;
                    if(isUsable(anim.transform.rotateY)) newAnim.transform.toRotateY = anim.transform.rotateY;
                    if(isUsable(anim.transform.rotateZ)) newAnim.transform.toRotateZ = anim.transform.rotateZ;

                    // scale
                    if(isUsable(anim.transform.scale)) newAnim.transform.toScale = anim.transform.scale;

                    // skew
                    if(isUsable(anim.transform.skewX)) newAnim.transform.toSkewX = anim.transform.skewX;
                    if(isUsable(anim.transform.skewY)) newAnim.transform.toSkewY = anim.transform.skewY;

                }

                // bgColor
                if(isUsable(anim.bgColor)) {
                    var color = getRGBA(anim.bgColor);
                    if(color) {
                        newAnim.bgColor = {};
                        newAnim.bgColor.toR = color.r;
                        newAnim.bgColor.toG = color.g;
                        newAnim.bgColor.toB = color.b;
                        newAnim.bgColor.toA = color.a;
                    } else {
                        console.warn("[Animation] invalid backgroundColor");
                    }
                }

                // txtColor
                if(isUsable(anim.txtColor)) {
                    var color = getRGBA(anim.txtColor);
                    if(color) {
                        newAnim.txtColor = {};
                        newAnim.txtColor.toR = color.r;
                        newAnim.txtColor.toG = color.g;
                        newAnim.txtColor.toB = color.b;
                        newAnim.txtColor.toA = color.a;
                    } else {
                        console.warn("[Animation] invalid textColor");
                    }
                }

                // boxShadow
                if(isUsable(anim.boxShadow)) {
                    newAnim.boxShadow = {};

                    // h
                    if(isUsable(anim.boxShadow.h)) newAnim.boxShadow.toH = anim.boxShadow.h;

                    // v
                    if(isUsable(anim.boxShadow.v)) newAnim.boxShadow.toV = anim.boxShadow.v;

                    // blur
                    if(isUsable(anim.boxShadow.blur)) newAnim.boxShadow.toBlur = anim.boxShadow.blur;

                    // spread
                    if(isUsable(anim.boxShadow.spread)) newAnim.boxShadow.toSpread = anim.boxShadow.spread;

                    // color
                    if(isUsable(anim.boxShadow.color)) {
                        var color = getRGBA(anim.boxShadow.color);
                        if(color) {
                            newAnim.boxShadow.color = {};
                            newAnim.boxShadow.color.toR = color.r;
                            newAnim.boxShadow.color.toG = color.g;
                            newAnim.boxShadow.color.toB = color.b;
                            newAnim.boxShadow.color.toA = color.a;
                        } else {
                            console.warn("[Animation] invalid boxShadow color");
                        }
                    }

                    // inset 不納入動畫

                }

                // scroll
                if(isUsable(anim.scroll)) {
                    newAnim.scroll = {};

                    // x
                    if(isUsable(anim.scroll.x)) newAnim.scroll.toX = anim.scroll.x;

                    // y
                    if(isUsable(anim.scroll.y)) newAnim.scroll.toY = anim.scroll.y;

                }

                // core
                queue.push(newAnim);

            }
            function setAnimaions(anims) {
                for(var i=0; i<anims.length; i++) {
                    setAnimaion(anims[i]);
                }
            }

            function play(anim) {
                if(Array.isArray(anim)) {
                    setAnimaions(anim);
                } else {
                    setAnimaion(anim);
                }

                //
                run = true;
                animate();
            }
            function animate() {
                // ====
                // TEMP: dev0323 視窗不可見時, 直接把動畫執行到最後結尾. 測試中.
                if(document.visibilityState == "hidden") {
                    return end();
                }
                // ====

                // core
                if(!current) {
                    if(queue.length == 0) return;
                    current = queue.shift();
                    startTime = new Date().getTime();
                }
                passingTime = (new Date().getTime() - startTime);
                progress = (passingTime / current.duration);

                // visibility
                if(current.visibility) {
                    // 因為要從 view 取值, 當有 queue 時 view 的值可能因為其他段動畫有變動, 所以必須在這裡取
                    if(!isUsable(current.visibility.fromValue)) current.visibility.fromValue = parseFloat(view.core.style.opacity);
                    if(!isUsable(current.visibility.totalOffset)) current.visibility.totalOffset = (current.visibility.toValue - current.visibility.fromValue);
                    //
                    if(current.visibility.fromValue == 0 && current.visibility.toValue != 0) view.core.style.display = "block";
                    // NOTE 底下註解掉的部分, 是剛要開始做 "當目標值跟現在一樣時, 不再做無謂的計算"
                    // TODO 應該對效能有幫助, 待完成, 並套用到類似地方
                    // if(current.visibility.totalOffset == 0) {
                    //     view.core.style.opacity = current.visibility.toValue;
                    // } else {
                        view.core.style.opacity = current.visibility.fromValue + (current.visibility.totalOffset * progress);
                    // }
                }

                // transform
                if(current.transform) {
                    // translateX
                    if(!isUsable(current.transform.toTranslateX)) current.transform.toTranslateX = view.transform.translateX; //
                    if(!isUsable(current.transform.fromTranslateX)) current.transform.fromTranslateX = view.transform.translateX;
                    if(!isUsable(current.transform.totalOffsetTranslateX)) current.transform.totalOffsetTranslateX = (current.transform.toTranslateX - current.transform.fromTranslateX);
                    var translateX = (current.transform.fromTranslateX + (current.transform.totalOffsetTranslateX * progress));

                    // translateY
                    if(!isUsable(current.transform.toTranslateY)) current.transform.toTranslateY = view.transform.translateY; //
                    if(!isUsable(current.transform.fromTranslateY)) current.transform.fromTranslateY = view.transform.translateY;
                    if(!isUsable(current.transform.totalOffsetTranslateY)) current.transform.totalOffsetTranslateY = (current.transform.toTranslateY - current.transform.fromTranslateY);
                    var translateY = (current.transform.fromTranslateY + (current.transform.totalOffsetTranslateY * progress));

                    // translateZ
                    if(!isUsable(current.transform.toTranslateZ)) current.transform.toTranslateZ = view.transform.translateZ; //
                    if(!isUsable(current.transform.fromTranslateZ)) current.transform.fromTranslateZ = view.transform.translateZ;
                    if(!isUsable(current.transform.totalOffsetTranslateZ)) current.transform.totalOffsetTranslateZ = (current.transform.toTranslateZ - current.transform.fromTranslateZ);
                    var translateZ = (current.transform.fromTranslateZ + (current.transform.totalOffsetTranslateZ * progress));

                    // rotateX
                    if(!isUsable(current.transform.toRotateX)) current.transform.toRotateX = view.transform.rotateX; //
                    if(!isUsable(current.transform.fromRotateX)) current.transform.fromRotateX = view.transform.rotateX;
                    if(!isUsable(current.transform.totalOffsetRotateX)) current.transform.totalOffsetRotateX = (current.transform.toRotateX - current.transform.fromRotateX);
                    var rotateX = (current.transform.fromRotateX + (current.transform.totalOffsetRotateX * progress));

                    // rotateY
                    if(!isUsable(current.transform.toRotateY)) current.transform.toRotateY = view.transform.rotateY; //
                    if(!isUsable(current.transform.fromRotateY)) current.transform.fromRotateY = view.transform.rotateY;
                    if(!isUsable(current.transform.totalOffsetRotateY)) current.transform.totalOffsetRotateY = (current.transform.toRotateY - current.transform.fromRotateY);
                    var rotateY = (current.transform.fromRotateY + (current.transform.totalOffsetRotateY * progress));

                    // rotateZ
                    if(!isUsable(current.transform.toRotateZ)) current.transform.toRotateZ = view.transform.rotateZ; //
                    if(!isUsable(current.transform.fromRotateZ)) current.transform.fromRotateZ = view.transform.rotateZ;
                    if(!isUsable(current.transform.totalOffsetRotateZ)) current.transform.totalOffsetRotateZ = (current.transform.toRotateZ - current.transform.fromRotateZ);
                    var rotateZ = (current.transform.fromRotateZ + (current.transform.totalOffsetRotateZ * progress));

                    // scale
                    if(!isUsable(current.transform.toScale)) current.transform.toScale = view.transform.scale; //
                    if(!isUsable(current.transform.fromScale)) current.transform.fromScale = view.transform.scale;
                    if(!isUsable(current.transform.totalOffsetScale)) current.transform.totalOffsetScale = (current.transform.toScale - current.transform.fromScale);
                    var scale = (current.transform.fromScale + (current.transform.totalOffsetScale * progress));

                    // skewX
                    if(!isUsable(current.transform.toSkewX)) current.transform.toSkewX = view.transform.skewX; //
                    if(!isUsable(current.transform.fromSkewX)) current.transform.fromSkewX = view.transform.skewX;
                    if(!isUsable(current.transform.totalOffsetSkewX)) current.transform.totalOffsetSkewX = (current.transform.toSkewX - current.transform.fromSkewX);
                    var skewX = (current.transform.fromSkewX + (current.transform.totalOffsetSkewX * progress));

                    // skewY
                    if(!isUsable(current.transform.toSkewY)) current.transform.toSkewY = view.transform.skewY; //
                    if(!isUsable(current.transform.fromSkewY)) current.transform.fromSkewY = view.transform.skewY;
                    if(!isUsable(current.transform.totalOffsetSkewY)) current.transform.totalOffsetSkewY = (current.transform.toSkewY - current.transform.fromSkewY);
                    var skewY = (current.transform.fromSkewY + (current.transform.totalOffsetSkewY * progress));

                    //
                    view.setTransform({
                        translateX: translateX,
                        translateY: translateY,
                        translateZ: translateZ,
                        rotateX: rotateX,
                        rotateY: rotateY,
                        rotateZ: rotateZ,
                        scale: scale,
                        skewX: skewX,
                        skewY: skewY
                    });

                }

                // bgColor
                if(current.bgColor) {
                    var fromColor;
                    if(!isUsable(current.bgColor.fromR) || !isUsable(current.bgColor.fromG) || !isUsable(current.bgColor.fromB) || !isUsable(current.bgColor.fromA)) {
                        if(view.look.bgColor) {
                            fromColor = getRGBA(view.look.bgColor);
                        } else {
                            // 若 view 沒設定過 bgColor, 就從透明開始
                            fromColor = {
                                r: current.bgColor.toR,
                                g: current.bgColor.toG,
                                b: current.bgColor.toB,
                                a: 0
                            }
                        }
                    }

                    // r
                    if(!isUsable(current.bgColor.fromR)) current.bgColor.fromR = fromColor.r;
                    if(!isUsable(current.bgColor.totalOffsetR)) current.bgColor.totalOffsetR = (current.bgColor.toR - current.bgColor.fromR);
                    var r = parseInt(current.bgColor.fromR + (current.bgColor.totalOffsetR * progress));

                    // g
                    if(!isUsable(current.bgColor.fromG)) current.bgColor.fromG = fromColor.g;
                    if(!isUsable(current.bgColor.totalOffsetG)) current.bgColor.totalOffsetG = (current.bgColor.toG - current.bgColor.fromG);
                    var g = parseInt(current.bgColor.fromG + (current.bgColor.totalOffsetG * progress));

                    // b
                    if(!isUsable(current.bgColor.fromB)) current.bgColor.fromB = fromColor.b;
                    if(!isUsable(current.bgColor.totalOffsetB)) current.bgColor.totalOffsetB = (current.bgColor.toB - current.bgColor.fromB);
                    var b = parseInt(current.bgColor.fromB + (current.bgColor.totalOffsetB * progress));

                    // a
                    if(!isUsable(current.bgColor.fromA)) current.bgColor.fromA = fromColor.a;
                    if(!isUsable(current.bgColor.totalOffsetA)) current.bgColor.totalOffsetA = (current.bgColor.toA - current.bgColor.fromA);
                    var a = (current.bgColor.fromA + (current.bgColor.totalOffsetA * progress));

                    //
                    view.setLook({
                        bgColor: "rgba(" + r + ", " + g + ", " + b + ", " + a + ")"
                    });

                }

                // txtColor
                if(current.txtColor) {
                    var fromColor;
                    if(!isUsable(current.txtColor.fromR) || !isUsable(current.txtColor.fromG) || !isUsable(current.txtColor.fromB) || !isUsable(current.txtColor.fromA)) {
                        if(view.look.txtColor) {
                            fromColor = getRGBA(view.look.txtColor);
                        } else {
                            // 若 view 沒設定過 txtColor, 就從透明開始
                            fromColor = {
                                r: current.txtColor.toR,
                                g: current.txtColor.toG,
                                b: current.txtColor.toB,
                                a: 0
                            }
                        }
                    }

                    // r
                    if(!isUsable(current.txtColor.fromR)) current.txtColor.fromR = fromColor.r;
                    if(!isUsable(current.txtColor.totalOffsetR)) current.txtColor.totalOffsetR = (current.txtColor.toR - current.txtColor.fromR);
                    var r = parseInt(current.txtColor.fromR + (current.txtColor.totalOffsetR * progress));

                    // g
                    if(!isUsable(current.txtColor.fromG)) current.txtColor.fromG = fromColor.g;
                    if(!isUsable(current.txtColor.totalOffsetG)) current.txtColor.totalOffsetG = (current.txtColor.toG - current.txtColor.fromG);
                    var g = parseInt(current.txtColor.fromG + (current.txtColor.totalOffsetG * progress));

                    // b
                    if(!isUsable(current.txtColor.fromB)) current.txtColor.fromB = fromColor.b;
                    if(!isUsable(current.txtColor.totalOffsetB)) current.txtColor.totalOffsetB = (current.txtColor.toB - current.txtColor.fromB);
                    var b = parseInt(current.txtColor.fromB + (current.txtColor.totalOffsetB * progress));

                    // a
                    if(!isUsable(current.txtColor.fromA)) current.txtColor.fromA = fromColor.a;
                    if(!isUsable(current.txtColor.totalOffsetA)) current.txtColor.totalOffsetA = (current.txtColor.toA - current.txtColor.fromA);
                    var a = (current.txtColor.fromA + (current.txtColor.totalOffsetA * progress));

                    //
                    view.setLook({
                        txtColor: "rgba(" + r + ", " + g + ", " + b + ", " + a + ")"
                    });

                }

                // boxShadow
                if(current.boxShadow) {
                    // h
                    if(!isUsable(current.boxShadow.toH)) current.boxShadow.toH = view.boxShadow.h; //
                    if(!isUsable(current.boxShadow.fromH)) current.boxShadow.fromH = view.boxShadow.h;
                    if(!isUsable(current.boxShadow.totalOffsetH)) current.boxShadow.totalOffsetH = (current.boxShadow.toH - current.boxShadow.fromH);
                    var h = (current.boxShadow.fromH + (current.boxShadow.totalOffsetH * progress));

                    // v
                    if(!isUsable(current.boxShadow.toV)) current.boxShadow.toV = view.boxShadow.v; //
                    if(!isUsable(current.boxShadow.fromV)) current.boxShadow.fromV = view.boxShadow.v;
                    if(!isUsable(current.boxShadow.totalOffsetV)) current.boxShadow.totalOffsetV = (current.boxShadow.toV - current.boxShadow.fromV);
                    var v = (current.boxShadow.fromV + (current.boxShadow.totalOffsetV * progress));

                    // blur
                    if(!isUsable(current.boxShadow.toBlur)) current.boxShadow.toBlur = view.boxShadow.blur; //
                    if(!isUsable(current.boxShadow.fromBlur)) current.boxShadow.fromBlur = view.boxShadow.blur;
                    if(!isUsable(current.boxShadow.totalOffsetBlur)) current.boxShadow.totalOffsetBlur = (current.boxShadow.toBlur - current.boxShadow.fromBlur);
                    var blur = (current.boxShadow.fromBlur + (current.boxShadow.totalOffsetBlur * progress));

                    // spread
                    if(!isUsable(current.boxShadow.toSpread)) current.boxShadow.toSpread = view.boxShadow.spread; //
                    if(!isUsable(current.boxShadow.fromSpread)) current.boxShadow.fromSpread = view.boxShadow.spread;
                    if(!isUsable(current.boxShadow.totalOffsetSpread)) current.boxShadow.totalOffsetSpread = (current.boxShadow.toSpread - current.boxShadow.fromSpread);
                    var spread = (current.boxShadow.fromSpread + (current.boxShadow.totalOffsetSpread * progress));

                    // color
                    var bsColor;
                    if(current.boxShadow.color) {
                        var fromColor;
                        if(!isUsable(current.boxShadow.color.fromR) || !isUsable(current.boxShadow.color.fromG) || !isUsable(current.boxShadow.color.fromB) || !isUsable(current.boxShadow.color.fromA)) {
                            if(view.boxShadow.color) {
                                fromColor = getRGBA(view.boxShadow.color);
                            } else {
                                // 若 boxShadow 沒設定過 color, 就從透明開始
                                fromColor = {
                                    r: current.boxShadow.color.toR,
                                    g: current.boxShadow.color.toG,
                                    b: current.boxShadow.color.toB,
                                    a: 0
                                }
                            }
                        }

                        // r
                        if(!isUsable(current.boxShadow.color.fromR)) current.boxShadow.color.fromR = fromColor.r;
                        if(!isUsable(current.boxShadow.color.totalOffsetR)) current.boxShadow.color.totalOffsetR = (current.boxShadow.color.toR - current.boxShadow.color.fromR);
                        var r = parseInt(current.boxShadow.color.fromR + (current.boxShadow.color.totalOffsetR * progress));

                        // g
                        if(!isUsable(current.boxShadow.color.fromG)) current.boxShadow.color.fromG = fromColor.g;
                        if(!isUsable(current.boxShadow.color.totalOffsetG)) current.boxShadow.color.totalOffsetG = (current.boxShadow.color.toG - current.boxShadow.color.fromG);
                        var g = parseInt(current.boxShadow.color.fromG + (current.boxShadow.color.totalOffsetG * progress));

                        // b
                        if(!isUsable(current.boxShadow.color.fromB)) current.boxShadow.color.fromB = fromColor.b;
                        if(!isUsable(current.boxShadow.color.totalOffsetB)) current.boxShadow.color.totalOffsetB = (current.boxShadow.color.toB - current.boxShadow.color.fromB);
                        var b = parseInt(current.boxShadow.color.fromB + (current.boxShadow.color.totalOffsetB * progress));

                        // a
                        if(!isUsable(current.boxShadow.color.fromA)) current.boxShadow.color.fromA = fromColor.a;
                        if(!isUsable(current.boxShadow.color.totalOffsetA)) current.boxShadow.color.totalOffsetA = (current.boxShadow.color.toA - current.boxShadow.color.fromA);
                        var a = (current.boxShadow.color.fromA + (current.boxShadow.color.totalOffsetA * progress));

                        //
                        bsColor = "rgba(" + r + ", " + g + ", " + b + ", " + a + ")";

                    } else {
                        bsColor = view.boxShadow.color;
                    }

                    //
                    view.setBoxShadow({
                        visible: true,
                        h: h,
                        v: v,
                        blur: blur,
                        spread: spread,
                        color: bsColor
                    });

                }

                // scroll
                if(current.scroll) {
                    // x
                    if(!isUsable(current.scroll.toX)) current.scroll.toX = view.scroll.x; //
                    if(!isUsable(current.scroll.fromX)) current.scroll.fromX = view.scroll.x;
                    if(!isUsable(current.scroll.totalOffsetX)) current.scroll.totalOffsetX = (current.scroll.toX - current.scroll.fromX);
                    var x = (current.scroll.fromX + (current.scroll.totalOffsetX * progress));

                    // y
                    if(!isUsable(current.scroll.toY)) current.scroll.toY = view.scroll.y; //
                    if(!isUsable(current.scroll.fromY)) current.scroll.fromY = view.scroll.y;
                    if(!isUsable(current.scroll.totalOffsetY)) current.scroll.totalOffsetY = (current.scroll.toY - current.scroll.fromY);
                    var y = (current.scroll.fromY + (current.scroll.totalOffsetY * progress));

                    //
                    view.setScroll({
                        x: x,
                        y: y
                    });

                }

                // core
                _this.emit("frame");

                if(passingTime >= current.duration) {
                    //
                    if(loop) {
                        queue.push(current);
                    }
                    //
                    stop();
                }

                if(run) requestAnimationFrame(animate);

            }

            function gotoCurrent() {
                // opacity
                if(current.visibility) {
                    if (current.visibility.toValue == 0) {
                        view.core.style.opacity = 0;
                        view.core.style.display = "none";
                        view.opts.visible = false;
                    } else {
                        view.core.style.opacity = current.visibility.toValue;
                        view.core.style.display = "block";
                        view.opts.visible = true;
                    }
                }

                // transform
                if(current.transform) {
                    view.setTransform({
                        translateX: current.transform.toTranslateX,
                        translateY: current.transform.toTranslateY,
                        translateZ: current.transform.toTranslateZ,
                        rotateX: current.transform.toRotateX,
                        rotateY: current.transform.toRotateY,
                        rotateZ: current.transform.toRotateZ,
                        scale: current.transform.toScale,
                        skewX: current.transform.toSkewX,
                        skewY: current.transform.toSkewY
                    });
                }

                // bgColor
                if(current.bgColor) {
                    view.setLook({
                        bgColor: "rgba(" + current.bgColor.toR + ", " + current.bgColor.toG + ", " + current.bgColor.toB + ", " + current.bgColor.toA + ")"
                    });
                }

                // txtColor
                if(current.txtColor) {
                    view.setLook({
                        txtColor: "rgba(" + current.txtColor.toR + ", " + current.txtColor.toG + ", " + current.txtColor.toB + ", " + current.txtColor.toA + ")"
                    });
                }

                // boxShadow
                if(current.boxShadow) {
                    view.setBoxShadow({
                        visible: true,
                        h: current.boxShadow.toH,
                        v: current.boxShadow.toV,
                        blur: current.boxShadow.toBlur,
                        spread: current.boxShadow.toSpread,
                        color: "rgba(" + current.boxShadow.color.toR + ", " + current.boxShadow.color.toG + ", " + current.boxShadow.color.toB + ", " + current.boxShadow.color.toA + ")"
                    });
                }

                // scroll
                if(current.scroll) {
                    view.setScroll({
                        x: current.scroll.toX,
                        y: current.scroll.toY
                    });
                }

            }
            function stop() {
                if(!current) return;

                gotoCurrent();

                // core
                current = null;
                startTime = null;
                passingTime = null;
                progress = null;

                if(queue.length == 0) {
                    run = false;
                    _this.emit("end");
                }

            }
            function end() {
                if(!current) {
                    if(queue.length > 0) {
                        current = queue[queue.length-1];
                        queue = [];
                    } else {
                        return;
                    }
                }

                gotoCurrent();

                // core
                current = null;
                startTime = null;
                passingTime = null;
                progress = null;

                run = false;
                _this.emit("end");

            }

            function setLoop(lp) {
                loop = lp;
            }

            function isAnimating() {
                return run;
            }

            //
            this.play = play;
            this.stop = stop;
            this.end = end;
            this.setLoop = setLoop;

            this.isAnimating = isAnimating;

        }

    }


    //
    return Animation;
})();




//
module.exports = Animation;
