
var EventEmitter = require("./EventEmitter");
var Animation = require("./Animation");

// ====

var CoreView = (function() {
    //
    var ua = window.navigator.userAgent.toLowerCase();
    window.BerIsSafari = ((ua.indexOf('safari')!=-1) && (ua.indexOf('chrome')  == -1)); // NOTE: 這是為了某些瀏覽器的字體問題加的, 但在開發環境不適用

    //
    class CoreView extends EventEmitter {
        constructor(elementType, parentView, options) {
            super();

            var DEFAULT_ANIM_DURATION = 500;

            var _this = this;
            this.core = createCore(elementType);
            this.parentView = parentView;
            this.options = options;

            //
            this.childViews = [];
            this.parentView.addChildView(this);

            //
            this.core.style.position = "absolute";
            this.core.style.display = "block";
            this.core.style.opacity = 1;
            this.core.style.backgroundRepeat = "no-repeat";
            this.core.style.backgroundPosition = "center";
            this.core.style.webkitTapHighlightColor = "rgba(0, 0, 0, 0)"; // 移除 touch 造成的 highlight
            this.core.style.outline = "none"; // 移除 focus/active 時自動出現的框
            this.core.style.boxSizing = "border-box"; // ref http://www.w3schools.com/jsref/prop_style_boxsizing.asp
            this.core.style.padding = 0;

            //
            function createCore(elementType) {
                var type;
                switch (elementType.toLowerCase()) {
                    case "text":
                        type = elementType;
                        elementType = "input";
                        break;

                    case "file":
                        type = elementType;
                        elementType = "input";
                        break;

                }

                var core = document.createElement(elementType);
                if(type) core.setAttribute("type", type);

                return core;
            }


            // == hierarchy ==
            this.addChildView = function(view) {
                _this.core.appendChild(view.core);
                _this.childViews.push(view);
                // _this.redraw();
            }
            this.removeChildView = function (view) {
                _this.core.removeChild(view.core);
                var idx = _this.childViews.indexOf(view);
                if(idx > -1) _this.childViews.splice(idx, 1);
                // _this.redrawChildViews();
            }
            this.redrawChildViews = function() {
                for(var i=0; i<_this.childViews.length; i++) {
                    if(_this.childViews[i].redraw) _this.childViews[i].redraw(true);
                }
            }


            // == ... ==
            this.setClassName = function(className) {
                _this.core.className = className;
            }


            // == events ==
            this.domOn = function (type, listener, consumeEvent) {
                var consumeEvent = (isUsable(consumeEvent)) ? consumeEvent : true;

                //
                _this.core.addEventListener(type, function(event) {
                    if(event) {
                        if(consumeEvent) {
                            event.preventDefault();
                            event.stopPropagation();
                        }

                        if(listener) {
                            listener(event);
                        } else {
                            // console.warn("no listener");
                        }

                    }
                }, false);

                //
                if(type == "click") {
                    // NOTE for touch control
                    var target = null;
                    _this.core.addEventListener("touchstart", function (event) {
                        // console.log("[start]", event);
                        target = event.target;
                    }, false);
                    _this.core.addEventListener("touchmove", function (event) {
                        // console.log("[move]", event);
                        var moveTarget = document.elementFromPoint(
                            event.changedTouches[0].pageX,
                            event.changedTouches[0].pageY
                        );
                        if(target !== moveTarget) { // NOTE leave
                            target = null;
                        }

                    }, false);
                    _this.core.addEventListener("touchend", function (event) {
                        // console.log("[end]", event);

                        var endTarget = document.elementFromPoint(
                            event.changedTouches[0].pageX,
                            event.changedTouches[0].pageY
                        );
                        // console.log("[end]", endTarget);

                        //====
                        if(target === endTarget) {
                            if (event) {
                                if (consumeEvent) {
                                    event.preventDefault();
                                    event.stopPropagation();
                                }

                                if (listener) {
                                    listener(event);
                                } else {
                                    console.warn("no listener");
                                }

                            }
                        }

                    }, false);

                    //
                    _this.core.style.cursor = "pointer";
                }

            }


            // == animation ==
            this.anim = new Animation(this);
            // this.anim.on("frame", function() {
            //     console.log("frame");
            // });
            this.anim.on("end", function() {
                _this.emit("anim_end");
            });


            // == set options ==
            this.init();
            this.setOptions(options);
            this.redraw();

            // NOTE example:1
            // NOTE 但仍有點奇怪的是, 上面的那個 redraw 沒有辦法觸發這個 on
            // this.on("redraw", function() {
            //     console.log("[core][on]");
            // });

        }


        //
        // ex() {
        //     super.ex.apply(this, arguments);
        // }

        // == init ==
        init() {
            this.rect = {
                x: 0,
                y: 0,
                w: 100,
                h: 100,
                mode: "%"
            };
            this.look = {
                bgSize: null,           // default 100% 100%
                bgImage: null,
                bgColor: null,
                borderRadius: 0,
                padding: 0,             // NOTE padding 是用 [boxSizing = "border-box"] 的 border 模擬的
                paddingScale: 0,        // TODO 測試實用性
                paddingR: 0             // TODO 反轉 paddingScale 參考的尺寸
            };
            this.opts = {
                cursor: "default",
                disabled: false,
                visible: true,
                position: "absolute",   // absolute, fixed
                zIndex: 0,
                userSelect: "none"      // none, text
            };
            this.transform = {
                translateX: 0,
                translateY: 0,
                translateZ: 0,
                rotateX: 0,
                rotateY: 0,
                rotateZ: 0,
                scale: 1,
                skewX: 0,
                skewY: 0
            };
            this.boxShadow = {
                visible: false,
                h: 0,
                v: 0,
                blur: 1,
                spread: 0,
                color: "",//"black",
                inset: false
            };
        }
        setOptions(options, setOnly) {
            var _setOnly = (isUsable(setOnly) ? setOnly : false);
            //
            if(isUsable(options)) {
                // NOTE 底下各設定都不 redraw, 後面才一次呼叫 this.redraw()
                if(isUsable(options.rect)) this.setRect(options.rect, _setOnly);
                if(isUsable(options.look)) this.setLook(options.look, _setOnly);
                if(isUsable(options.opts)) this.setOpts(options.opts, _setOnly);
                if(isUsable(options.transform)) this.setTransform(options.transform, _setOnly);
                if(isUsable(options.boxShadow)) this.setBoxShadow(options.boxShadow, _setOnly);
            }
        }

        // == redraw ==

        // rect
        setRect(rect, setOnly) {
            assignToTargetOwnProps(rect, this.rect);
            if(!(isUsable(setOnly) && setOnly)) this.redrawRect();
        }
        redrawRect() {
            // NOTE 實驗中
            if(typeof(this.rect.x) === "string") {
                this.core.style.left = this.rect.x;
            } else {
                this.core.style.left = this.rect.x + this.rect.mode;
            }
            if(typeof(this.rect.y) === "string") {
                this.core.style.top = this.rect.y;
            } else {
                this.core.style.top = this.rect.y + this.rect.mode;
            }
            if(typeof(this.rect.w) === "string") {
                this.core.style.width = this.rect.w;
            } else {
                this.core.style.width = this.rect.w + this.rect.mode;
            }
            if(typeof(this.rect.h) === "string") {
                this.core.style.height = this.rect.h;
            } else {
                this.core.style.height = this.rect.h + this.rect.mode;
            }
            // this.core.style.left = this.rect.x + this.rect.mode;
            // this.core.style.top = this.rect.y + this.rect.mode;
            // this.core.style.width = this.rect.w + this.rect.mode;
            // this.core.style.height = this.rect.h + this.rect.mode;
        }

        // look
        setLook(look, setOnly) {
            assignToTargetOwnProps(look, this.look);
            if(!(isUsable(setOnly) && setOnly)) this.redrawLook();
        }
        redrawLook() {
            // bgImage
            if(this.look.bgSize) {
                this.core.style.backgroundSize = this.look.bgSize + "px";
            } else {
                this.core.style.backgroundSize = "100% 100%";
            }
            this.core.style.backgroundImage = this.look.bgImage;//

            // bgColor
            this.core.style.backgroundColor = this.look.bgColor;

            // borderRadius
            this.core.style.borderRadius = this.look.borderRadius + "px";

            // sPadding (implemented by border)
            if(this.look.padding) {
                // if(this.parentView.look.bgColor) {
                //     this.core.style.border = this.look.padding + "px solid " + this.parentView.look.bgColor;    // NOTE 顯示出 parentView 的背景色, 模擬本 view 的尺寸縮小 >> NOTE 應該直接透明就好了吧
                // } else {
                    this.core.style.border = this.look.padding + "px solid rgba(0, 0, 0, 0)";
                // }
            } else {

                if(this.look.paddingR) {
                    var refSize = (window.innerWidth > window.innerHeight) ? window.innerWidth : window.innerHeight;
                    var BASE_VALUE_FOR_1080 = 1;
                    var padding = (BASE_VALUE_FOR_1080 * refSize / 1080) * this.look.paddingR;

                    this.core.style.border = padding + "px solid rgba(0, 0, 0, 0)";

                } else if(this.look.paddingScale) {
                    var refSize = (window.innerWidth > window.innerHeight) ? window.innerHeight : window.innerWidth;
                    var BASE_VALUE_FOR_1080 = 1;
                    var padding = (BASE_VALUE_FOR_1080 * refSize / 1080) * this.look.paddingScale;

                    this.core.style.border = padding + "px solid rgba(0, 0, 0, 0)";

                } else {
                    // no padding, no paddingScale
                    this.core.style.border = "hidden";
                }

            }

        }

        // opts
        setOpts(opts, setOnly) {
            assignToTargetOwnProps(opts, this.opts);
            if(!(isUsable(setOnly) && setOnly)) this.redrawOpts();
        }
        redrawOpts() {
            var redrawAll = false;
            //
            this.core.style.cursor = this.opts.cursor;
            this.core.style.disabled = this.opts.disabled;
            if(this.opts.visible == "hidden") {
                this.core.style.visibility = "hidden";
            } else if(this.opts.visible) {
                this.core.style.visibility = "visible";
                if(this.core.style.display == "none") redrawAll = true;  // NOTE 這情況需要重新參考尺寸, 所以必須強制重繪
                this.core.style.opacity = 1;
                this.core.style.display = "block";
            } else {
                this.core.style.visibility = "visible";
                this.core.style.opacity = 0;
                this.core.style.display = "none";
            }
            this.core.style.position = this.opts.position;
            this.core.style.zIndex = this.opts.zIndex;
            this.core.style.userSelect = this.opts.userSelect;
            //
            if(redrawAll) this.redraw(true);
        }

        // transform
        setTransform(transform, setOnly) {
            assignToTargetOwnProps(transform, this.transform);
            if(!(isUsable(setOnly) && setOnly)) this.redrawTransfrom();
        }
        resetTransform() {
            this.transform = {
                translateX: 0,
                translateY: 0,
                translateZ: 0,
                rotateX: 0,
                rotateY: 0,
                rotateZ: 0,
                scale: 1,
                skewX: 0,
                skewY: 0
            };
            this.redrawTransfrom();
        }
        redrawTransfrom() {
            var trans = "";

            if(this.transform.translateX != 0) trans += "translateX(" + this.transform.translateX + "px) ";
            if(this.transform.translateY != 0) trans += "translateY(" + this.transform.translateY + "px) ";
            if(this.transform.translateZ != 0) trans += "translateZ(" + this.transform.translateZ + "px) ";
            if(this.transform.rotateX != 0) trans += "rotateX(" + this.transform.rotateX + "deg) ";
            if(this.transform.rotateY != 0) trans += "rotateY(" + this.transform.rotateY + "deg) ";
            if(this.transform.rotateZ != 0) trans += "rotateZ(" + this.transform.rotateZ + "deg) ";
            if(this.transform.scale != 1) trans += "scale(" + this.transform.scale + ") ";
            if(this.transform.skewX != 0) trans += "skewX(" + this.transform.skewX + "deg) ";
            if(this.transform.skewY != 0) trans += "skewY(" + this.transform.skewY + "deg) ";

            this.core.style.transform = trans;

        }

        // boxShadow
        setBoxShadow(boxShadow, setOnly) {
            assignToTargetOwnProps(boxShadow, this.boxShadow);
            if(!(isUsable(setOnly) && setOnly)) this.redrawBoxShadow();
        }
        redrawBoxShadow() {
            var bs = "";
            if (this.boxShadow.visible) {
                bs += this.boxShadow.h + "px ";
                bs += this.boxShadow.v + "px ";
                bs += this.boxShadow.blur + "px ";
                bs += this.boxShadow.spread + "px ";
                bs += this.boxShadow.color;
                if(this.boxShadow.inset) bs += " inset";
            }
            this.core.style.boxShadow = bs;
        }

        //
        redraw(redrawChild) {
            this.redrawRect()
            this.redrawLook();
            this.redrawOpts();
            this.redrawTransfrom();
            this.redrawBoxShadow();
            this.emit("redraw");
            // console.log("[redraw][core]", redrawChild);

            // NOTE 關於這個 emit
            // 此 class 的 instance 宣告的 on, 只有該 instance 呼叫的 redraw (裡面的 emit) 會觸發
            // 當 redraw 在此 class 的 constructor 呼叫時, 只有宣告在 constructor 裡面的 on 會觸發 (參照上面的 example:1)


            // ====
            if(isUsable(redrawChild) && redrawChild == true) this.redrawChildViews();

        }


        // == animation ==
        animate(anim) {
            this.anim.play(anim);
        }
        endAnimation() {
            this.anim.end();
        }
        setAnimaionLoop(loop) {
            this.anim.setLoop(loop);
        }


        // == getter ==

        // NOTE 為了在 .style.display == "none" 時還能取到值, 做了以下 workaround
        // TODO 確認在實際製作介面時會否有問題, 尤其時效能低的裝置上

        get PxX() {
            // return this.core.offsetLeft;
            if(this.core.style.display == "block") {
                return this.core.offsetLeft;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetLeft;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxY() {
            // return this.core.offsetTop;
            if(this.core.style.display == "block") {
                return this.core.offsetTop;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetTop;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxW() {
            // return this.core.offsetWidth;
            if(this.core.style.display == "block") {
                return this.core.offsetWidth;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetWidth;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxH() {
            // return this.core.offsetHeight;
            if(this.core.style.display == "block") {
                return this.core.offsetHeight;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetHeight;
                this.core.style.display = "none";
                return ret;
            }
        }
        get RectPx() {
            // return {
            //     x: this.PxX,
            //     y: this.PxY,
            //     w: this.PxW,
            //     h: this.PxH
            // };
            if(this.core.style.display == "block") {
                return {
                    x: this.core.offsetLeft,
                    y: this.core.offsetTop,
                    w: this.core.offsetWidth,
                    h: this.core.offsetHeight
                };
            } else {
                this.core.style.display = "block";
                var ret = {
                    x: this.core.offsetLeft,
                    y: this.core.offsetTop,
                    w: this.core.offsetWidth,
                    h: this.core.offsetHeight
                };
                this.core.style.display = "none";
                return ret;
            }
        }
        // NOTE: postfix = 2 的系列, 主動計算 this.look.padding 造成的影響
        get PxX2() {
            // return this.core.offsetLeft;
            if(this.core.style.display == "block") {
                return this.core.offsetLeft + this.look.padding;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetLeft + this.look.padding;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxY2() {
            // return this.core.offsetTop;
            if(this.core.style.display == "block") {
                return this.core.offsetTop + this.look.padding;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetTop + this.look.padding;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxW2() {
            // return this.core.offsetWidth;
            if(this.core.style.display == "block") {
                return this.core.offsetWidth - this.look.padding * 2;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetWidth - this.look.padding * 2;
                this.core.style.display = "none";
                return ret;
            }
        }
        get PxH2() {
            // return this.core.offsetHeight;
            if(this.core.style.display == "block") {
                return this.core.offsetHeight - this.look.padding * 2;
            } else {
                this.core.style.display = "block";
                var ret = this.core.offsetHeight - this.look.padding * 2;
                this.core.style.display = "none";
                return ret;
            }
        }
        get RectPx2() {
            // return {
            //     x: this.PxX,
            //     y: this.PxY,
            //     w: this.PxW,
            //     h: this.PxH
            // };
            if(this.core.style.display == "block") {
                return {
                    x: this.core.offsetLeft + this.look.padding,
                    y: this.core.offsetTop + this.look.padding,
                    w: this.core.offsetWidth - this.look.padding * 2,
                    h: this.core.offsetHeight - this.look.padding * 2
                };
            } else {
                this.core.style.display = "block";
                var ret = {
                    x: this.core.offsetLeft + this.look.padding,
                    y: this.core.offsetTop + this.look.padding,
                    w: this.core.offsetWidth - this.look.padding * 2,
                    h: this.core.offsetHeight - this.look.padding * 2
                };
                this.core.style.display = "none";
                return ret;
            }
        }

        get ScrollTop() {
            return this.core.scrollTop;
        }
        set ScrollTop(scrollTop) {
            this.core.scrollTop = scrollTop;
        }
        get ScrollLeft() {
            return this.core.scrollLeft;
        }
        set ScrollLeft(scrollLeft) {
            this.core.scrollLeft = scrollLeft;
        }


        // ====
        removeFromParent() {
            if(this.core.parentNode) {
                this.core.parentNode.removeChild(this.core);
                this.parentView = null;
            }
        }
        setParentView(parentView) {
            this.removeFromParent();
            this.parentView = parentView;
            this.parentView.addChildView(this);
        }


    }


    //
    return CoreView
})();


//
module.exports = CoreView;
