
var TextView = require("../basic/TextView");
var ButtonView = require("../basic/ButtonView");
var BasePanel = require("./BasePanel");

// ====

var AlertPanel = (function() {
    //

    //
    class AlertPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == events ==
            this.domOn("click", function() {
                // console.log("mask clicked");
                _this.hide();
            });
            this.boxPanel.domOn("click", function() {
                // do nothing (只是為了吃掉事件)
            });
            this.btnDone.on("click", function(btn) {
                // console.log("[done]", btn);
                _this.emit("result");
                _this.hide();
            });

        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = "rgba(200, 200, 200, 0.7)";

            this.look.txtDone = "確定";
            this.look.panelBgColor = "rgb(100, 100, 100)";
            this.look.panelTxtColor = "rgb(255, 255, 255)";
            this.look.panelSubTxtColor = "rgb(180, 180, 180)";
            this.look.btnBgColorEnter = "rgb(150, 150, 150)";
            this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
            this.look.btnBgColorDown = "rgb(200, 200, 200)";
            this.look.btnTxtColorDown = "rgb(80, 80, 80)";


            // == ui ==
            //
            this.boxPanel = new BasePanel(this, {
                rect: { x: 5, y: 30, w: 90, h: 40 },
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    visible: true
                }
            })

            // msg
            this.msgPanel = new BasePanel(this.boxPanel, {
                rect: { h: 70 }
                // look: { padding: 4 }
            });
            this.txtSingle = new TextView(this.msgPanel, {
                rect: { y: 25, h: 50 },
                look: {
                    txtColor: this.look.panelTxtColor,
                    txtAlign: "center"
                }
            });
            this.txtDoubleTitle = new TextView(this.msgPanel, {
                rect: { y: 10, h: 50 },
                look: {
                    txtColor: this.look.panelTxtColor,
                    txtAlign: "center"
                }
            });
            this.txtDoubleMsg = new TextView(this.msgPanel, {
                rect: { y: 60, h: 30 },
                look: {
                    txtColor: this.look.panelSubTxtColor,
                    txtAlign: "center"
                }
            });

            // btn
            this.btnPanel = new BasePanel(this.boxPanel, {
                rect: { y: 70, h: 30 },
                look: {
                    bgColor: "white"
                }
            });
            this.btnDone = new ButtonView(this.btnPanel, {
                // rect: { x: 50, w: 50 },
                look: {
                    // txtShadow: {
                    //     color: "rgba(0, 0, 0, 1)"
                    // },
                    bgNormal: this.look.panelBgColor,
                    bgEnter: this.look.btnBgColorEnter,
                    bgDown: this.look.btnBgColorDown,
                    txtNormal: this.look.panelTxtColor,
                    txtEnter: this.look.btnTxtColorEnter,
                    txtDown: this.look.btnTxtColorDown,
                    // imgNormal: "url('my.png')",
                    // imgEnter: "url('my2.jpg')",
                    // imgDown: "url('my3.jpg')",
                }
            });

        }


        // == redraw ==

        // look
        redrawLook() {
            super.redrawLook();

            this.btnDone.setLook({
                txt: this.look.txtDone,
                bgNormal: this.look.panelBgColor,
                bgEnter: this.look.btnBgColorEnter,
                bgDown: this.look.btnBgColorDown,
                txtNormal: this.look.panelTxtColor,
                txtEnter: this.look.btnTxtColorEnter,
                txtDown: this.look.btnTxtColorDown,
            });

        }


        // ====
        show(data) {
            super.show();

            if(isUsable(data.title)) {
                this.txtDoubleTitle.setLook({ txt: data.title });
                this.txtDoubleMsg.setLook({ txt: data.msg });
                this.txtSingle.setOpts({ visible: false });
                this.txtDoubleTitle.setOpts({ visible: true });
                this.txtDoubleMsg.setOpts({ visible: true });
            } else {
                this.txtSingle.setLook({ txt: data.msg });
                this.txtSingle.setOpts({ visible: true });
                this.txtDoubleTitle.setOpts({ visible: false });
                this.txtDoubleMsg.setOpts({ visible: false });
            }

            this.btnPanel.redraw(true);

        }


    }


    //
    return AlertPanel;
})();


//
module.exports = AlertPanel;
