
var ButtonView = require("../basic/ButtonView");
var ToggleView = require("../basic/ToggleView");
var TextView = require("../basic/TextView");
var BasePanel = require("./BasePanel");
var ListPanel = require("./ListPanel");
var ListItem = require("./ListItem");

// ====

// NOTE TODO 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.

var COLOR_BTN_BRIGHT = "rgb(56, 147, 170)";

var MonthSelector = (function() {
    //
    class ControlPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            this.year;


            // == methods ==
            function setYear(year) {
                _this.year = year;
                _this.btnYear.setLook({ txt: year + " 年" });
            }
            //
            this.setYear = setYear;


            // == events ==
            this.btnYear.on("click", function() {
                _this.emit("yearClicked");
            });
            this.btnUp.on("click", function() {
                _this.year--;
                setYear(_this.year);
                _this.emit("yearChanged", _this.year);
            });
            this.btnDown.on("click", function() {
                _this.year++;
                setYear(_this.year);
                _this.emit("yearChanged", _this.year);
            });


        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = COLOR_BTN_BRIGHT;
            //
            this.look.btnBgColor = COLOR_BTN_BRIGHT;
            this.look.btnTxtColor = "rgb(240, 240, 240)";
            this.look.btnTxtColor_down = "rgb(200, 200, 200)";

            //
            // == ui ==
            var UNIT_W = 100 / 10;

            var cellYear = new BasePanel(this, {
                rect: { x: UNIT_W * 0, w: UNIT_W * 6 }
            });
            var btnYear = new ButtonView(cellYear, {
                look: {
                    txtScale: 0.4,
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    txtNormal: this.look.btnTxtColor,
                    txtEnter: this.look.btnTxtColor,
                    txtDown: this.look.btnTxtColor_down
                }
            });

            var cellUp = new BasePanel(this, {
                rect: { x: UNIT_W * 6, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnUp = new ButtonView(cellUp, {
                look: {
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    imgNormal: "url('img/sys/up_white.png')",
                    imgEnter: "url('img/sys/up_white.png')",
                    imgDown: "url('img/sys/up_light_gray.png')"
                }
            });

            var cellDown = new BasePanel(this, {
                rect: { x: UNIT_W * 8, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnDown = new ButtonView(cellDown, {
                look: {
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    imgNormal: "url('img/sys/down_white.png')",
                    imgEnter: "url('img/sys/down_white.png')",
                    imgDown: "url('img/sys/down_light_gray.png')"
                }
            });


            //
            this.cellUp = cellUp;

            this.btnYear = btnYear;
            this.btnUp = btnUp;
            this.btnDown = btnDown;

        }

        //
        redrawRect() {
            super.redrawRect();

            var refRect = this.cellUp.RectPx;
            var refSize = ((refRect.w > refRect.h) ? refRect.h : refRect.w) * 0.8;

            this.btnUp.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

            this.btnDown.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

        }


    }

    //
    class MonthPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==


            // == events ==
            // this.btnYear.on("click", function() {
            //     _this.emit("yearMonthClicked");
            // });
            for(var i=0; i<12; i++) {
                (function(I) {

                    _this.btns[I].on("click", function() {
                        _this.emit("monthClicked", I);
                    });

                })(i);
            }

        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.btnBgColor = COLOR_BTN_BRIGHT;
            this.look.btnTxtColor = "rgb(240, 240, 240)";
            this.look.btnTxtColor_down = "rgb(200, 200, 200)";

            //
            // == ui ==
            var UNIT_W = 100 / 4;
            var UNIT_H = 100 / 3;
            var cell, btn;
            // this.cells = [];
            this.btns = [];

            for(var i=0; i<12; i++) {
                cell = new BasePanel(this, {
                    rect: {
                        x: UNIT_W * (i % 4),
                        y: UNIT_H * (Math.floor(i / 4)),
                        w: UNIT_W,
                        h: UNIT_H
                    }
                });
                btn = new ButtonView(cell, {
                    look: {
                        txtScale: 0.2,
                        bgNormal: this.look.btnBgColor,
                        bgEnter: this.look.btnBgColor,
                        bgDown: this.look.btnBgColor,
                        txtNormal: this.look.btnTxtColor,
                        txtEnter: this.look.btnTxtColor,
                        txtDown: this.look.btnTxtColor_down,
                        txt: (i+1) + "月"
                    }
                });
                this.btns.push(btn);
            }



            //
            // this.btnYear = btnYear;

        }


    }


    //
    class MonthSelector extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var yearSelector;


            // == methods ==
            function initYearSelector() {
                yearSelector = new YearSelector(_this);
                yearSelector.on("yearSelected", function(year) {
                    _this.year = year;
                    _this.controlPanel.setYear(year);
                    yearSelector.hide();
                });

            }
            function showYearSelector() {
                if(!yearSelector) initYearSelector();
                yearSelector.show(_this.year);
            }


            // == events ==
            this.controlPanel.on("yearClicked", function() {
                showYearSelector();
            });
            this.controlPanel.on("yearChanged", function(year) {
                _this.year = year;
            });
            this.monthPanel.on("monthClicked", function(month) {
                _this.month = month;
                _this.emit("monthSelected", _this.year, _this.month);
            });

        }


        // == init ==
        init() {
            super.init();

            var now = new Date();
            this.year = now.getFullYear();
            this.month = now.getMonth();

            //
            var _this = this;
            var MASK_COLOR = "rgba(50, 50, 50, 0.7)";

            // look
            this.look.bgColor = MASK_COLOR;//"rgba(200, 200, 200, 0.7)";
            //
            this.look.txtBack = "Back";
            this.look.txtDone = "Done";
            this.look.panelBgColor = "rgb(150, 150, 150)";
            this.look.panelTxtColor = "rgb(255, 255, 255)";
            this.look.panelSubTxtColor = "rgb(180, 180, 180)";
            this.look.btnBgColorEnter = "rgb(150, 150, 150)";
            this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
            this.look.btnBgColorDown = "rgb(200, 200, 200)";
            this.look.btnTxtColorDown = "rgb(80, 80, 80)";

            // opts
            this.opts.visible = false;

            //
            // == ui ==
            var UNIT_H = 100 / 7.5;

            var controlPanel = new ControlPanel(this, {
                rect: { y: UNIT_H * 0, h: UNIT_H * 1 }
            });

            var monthPanel = new MonthPanel(this, {
                rect: { y: UNIT_H * 1, h: UNIT_H * 6.5 }
            });


            //
            this.controlPanel = controlPanel;
            this.monthPanel = monthPanel;

        }


        //
        show(year) {
            super.show();

            this.year = year;
            this.controlPanel.setYear(year);

        }

    }




    // ====
    return MonthSelector;
})();

var YearSelector = (function() {
    //
    class ControlPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            this.year;


            // == methods ==


            // == events ==
            // this.btnYear.on("click", function() {
            //     _this.emit("yearClicked");
            // });
            this.btnUp.on("click", function() {
                _this.emit("yearUp");
            });
            this.btnDown.on("click", function() {
                _this.emit("yearDown");
            });


        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = COLOR_BTN_BRIGHT;
            //
            this.look.btnBgColor = COLOR_BTN_BRIGHT;


            // == ui ==
            var UNIT_W = 100 / 10;

            var cellUp = new BasePanel(this, {
                rect: { x: UNIT_W * 6, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnUp = new ButtonView(cellUp, {
                look: {
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    imgNormal: "url('img/sys/up_white.png')",
                    imgEnter: "url('img/sys/up_white.png')",
                    imgDown: "url('img/sys/up_light_gray.png')"
                }
            });

            var cellDown = new BasePanel(this, {
                rect: { x: UNIT_W * 8, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnDown = new ButtonView(cellDown, {
                look: {
                    bgNormal: this.look.btnBgColor,
                    bgEnter: this.look.btnBgColor,
                    bgDown: this.look.btnBgColor,
                    imgNormal: "url('img/sys/down_white.png')",
                    imgEnter: "url('img/sys/down_white.png')",
                    imgDown: "url('img/sys/down_light_gray.png')"
                }
            });


            //
            this.cellUp = cellUp;

            this.btnUp = btnUp;
            this.btnDown = btnDown;

        }

        //
        redrawRect() {
            super.redrawRect();

            var refRect = this.cellUp.RectPx;
            var refSize = ((refRect.w > refRect.h) ? refRect.h : refRect.w) * 0.8;

            this.btnUp.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

            this.btnDown.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

        }


    }

    //
    class YearPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var fromYear;


            // == methods ==
            function setYears(from) {
                fromYear = from - 8;
                for(var i=0; i<12; i++) {
                    _this.btns[i].setLook({ txt: (fromYear + i) });
                }
            }
            //
            this.setYears = setYears;


            // == events ==
            for(var i=0; i<12; i++) {
                (function(I) {

                    _this.btns[I].on("click", function() {
                        _this.emit("yearClicked", (fromYear + I));
                    });

                })(i);
            }

        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.btnBgColor = COLOR_BTN_BRIGHT;
            this.look.btnTxtColor = "rgb(240, 240, 240)";
            this.look.btnTxtColor_down = "rgb(200, 200, 200)";

            //
            // == ui ==
            var UNIT_W = 100 / 4;
            var UNIT_H = 100 / 3;
            var cell, btn;
            // this.cells = [];
            this.btns = [];

            for(var i=0; i<12; i++) {
                cell = new BasePanel(this, {
                    rect: {
                        x: UNIT_W * (i % 4),
                        y: UNIT_H * (Math.floor(i / 4)),
                        w: UNIT_W,
                        h: UNIT_H
                    }
                });
                btn = new ButtonView(cell, {
                    look: {
                        txtScale: 0.2,
                        bgNormal: this.look.btnBgColor,
                        bgEnter: this.look.btnBgColor,
                        bgDown: this.look.btnBgColor,
                        txtNormal: this.look.btnTxtColor,
                        txtEnter: this.look.btnTxtColor,
                        txtDown: this.look.btnTxtColor_down,
                        txt: (i+1) + "月"
                    }
                });
                this.btns.push(btn);
            }


        }


    }


    //
    class YearSelector extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==


            // == events ==
            // this.controlPanel.on("yearClicked", function() {
            //     console.log("yearClicked");
            // });
            this.controlPanel.on("yearUp", function() {
                _this.year -= 12;
                _this.yearPanel.setYears(_this.year);
            });
            this.controlPanel.on("yearDown", function() {
                _this.year += 12;
                _this.yearPanel.setYears(_this.year);
            });
            this.yearPanel.on("yearClicked", function(year) {
                _this.emit("yearSelected", year);
            });

        }


        // == init ==
        init() {
            super.init();

            var now = new Date();
            this.year = now.getFullYear();
            this.month = now.getMonth();

            //
            var _this = this;
            var MASK_COLOR = "rgba(50, 50, 50, 0.7)";

            // look
            this.look.bgColor = MASK_COLOR;//"rgba(200, 200, 200, 0.7)";
            //
            this.look.txtBack = "Back";
            this.look.txtDone = "Done";
            this.look.panelBgColor = "rgb(150, 150, 150)";
            this.look.panelTxtColor = "rgb(255, 255, 255)";
            this.look.panelSubTxtColor = "rgb(180, 180, 180)";
            this.look.btnBgColorEnter = "rgb(150, 150, 150)";
            this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
            this.look.btnBgColorDown = "rgb(200, 200, 200)";
            this.look.btnTxtColorDown = "rgb(80, 80, 80)";

            // opts
            this.opts.visible = false;

            //
            // == ui ==
            var UNIT_H = 100 / 7.5;

            var controlPanel = new ControlPanel(this, {
                rect: { y: UNIT_H * 0, h: UNIT_H * 1 }
            });

            var yearPanel = new YearPanel(this, {
                rect: { y: UNIT_H * 1, h: UNIT_H * 6.5 }
            });


            //
            this.controlPanel = controlPanel;
            this.yearPanel = yearPanel;

        }


        //
        show(year) {
            super.show();

            this.year = year;
            this.yearPanel.setYears(year);

        }

    }


    // ====
    return YearSelector;
})();

//
var Calendar = (function() {
    //

    //
    class ControlPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==
            function setYearMonth(year, month) {
                _this.btnYearMonth.setLook({ txt: year + "年 " + month + "月" });
            }
            //
            this.setYearMonth = setYearMonth;


            // == events ==

            this.btnYearMonth.on("click", function() {
                _this.emit("yearMonthClicked");
            });
            this.btnUp.on("click", function() {
                _this.emit("upClicked");
            });
            this.btnDown.on("click", function() {
                _this.emit("downClicked");
            });

        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.btnBgNormal = COLOR_BTN_BRIGHT;
            this.look.btnBgEnter = "rgb(200, 200, 200)";
            this.look.btnBgDown = "rgb(100, 100, 100)";
            this.look.btnTxtNormal = "rgb(240, 240, 240)";
            this.look.btnTxtEnter = "rgb(100, 100, 100)";
            this.look.btnTxtDown = "rgb(200, 200, 200)";


            //
            // == ui ==
            var UNIT_W = 100 / 10;

            var cellYearMonth = new BasePanel(this, {
                rect: { x: UNIT_W * 0, w: UNIT_W * 6 }
            });
            var btnYearMonth = new ButtonView(cellYearMonth, {
                look: {
                    txtScale: 0.4,
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    txtNormal: this.look.btnTxtNormal,
                    txtEnter: this.look.btnTxtNormal,
                    txtDown: this.look.btnTxtDown
                }
            });

            var cellUp = new BasePanel(this, {
                rect: { x: UNIT_W * 6, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnUp = new ButtonView(cellUp, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/up_white.png')",
                    imgEnter: "url('img/sys/up_white.png')",
                    imgDown: "url('img/sys/up_light_gray.png')"
                }
            });

            var cellDown = new BasePanel(this, {
                rect: { x: UNIT_W * 8, w: UNIT_W * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnDown = new ButtonView(cellDown, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/down_white.png')",
                    imgEnter: "url('img/sys/down_white.png')",
                    imgDown: "url('img/sys/down_light_gray.png')"
                }
            });


            //
            this.cellUp = cellUp;

            this.btnYearMonth = btnYearMonth;
            this.btnUp = btnUp;
            this.btnDown = btnDown;

        }

        //
        redrawRect() {
            super.redrawRect();

            var refRect = this.cellUp.RectPx;
            var refSize = ((refRect.w > refRect.h) ? refRect.h : refRect.w) * 0.8;

            this.btnUp.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

            this.btnDown.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

        }

    }

    class WeekHeader extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==


            // == events ==

        }


        // == init ==
        init() {
            super.init();

            // look
            this.look.txtBgNormal = "rgb(200, 200, 200)";
            this.look.txtTxtNormal = "rgb(50, 50, 50)";


            //
            // == ui ==
            this.txts = ["日", "一", "二", "三", "四", "五", "六"];
            this.days = [];

            var COL_W = 100 / 7;
            var refX = 0;
            var cell, txtView;
            for(var i=0; i<7; i++) {
                cell = new BasePanel(this, {
                    rect: { x: refX, w: COL_W }
                });
                txtView = new TextView(cell, {
                    look: {
                        txtScale: 0.4,
                        bgColor: this.look.txtBgNormal,
                        txtColor: this.look.txtTxtNormal,
                        txt: this.txts[i]
                    }
                });
                this.days.push(txtView);
                //
                refX += COL_W;
            }

        }

    }


    var DayButtonPanel = (function() {

        class WeekRow extends BasePanel {
            constructor(parentView, options) {
                super(parentView, options);
                var _this = this;


                // == methods ==


                // == events ==
                for(var i=0; i<7; i++) {
                    (function(I) {

                        _this.days[I].on("click", function() {
                            _this.emit("dateSelected", _this.days[I].dataDate);
                        });

                    })(i);
                }

            }


            // == init ==
            init() {
                super.init();

                // look
                this.look.bgColorThisMonth = COLOR_BTN_BRIGHT;
                this.look.txtColorThisMonth = "rgb(240, 240, 240)";
                this.look.txtColorThisMonth_enter = "rgb(200, 200, 200)";
                this.look.bgColorOtherMonth = "rgb(100, 100, 100)";
                this.look.txtColorOtherMonth = "rgb(240, 240, 240)";
                this.look.txtColorOtherMonth_enter = "rgb(200, 200, 200)";


                //
                // == ui ==
                this.days = [];

                var COL_W = 100 / 7;
                var refX = 0;
                var cell, btn;
                for(var i=0; i<7; i++) {
                    cell = new BasePanel(this, {
                        rect: { x: refX, w: COL_W },
                        look: { padding: 1 }
                    });
                    btn = new ButtonView(cell, {
                        look: {
                            txtScale: 0.4,
                            bgNormal: this.look.bgColorThisMonth,
                            bgEnter: this.look.bgColorThisMonth,
                            bgDown: this.look.bgColorThisMonth,
                            txtNormal: this.look.txtColorThisMonth,
                            txtEnter: this.look.txtColorThisMonth_enter,
                            txtDown: this.look.txtColorThisMonth
                        }
                    });
                    this.days.push(btn);
                    //
                    refX += COL_W;
                }

            }

            //
            initDates(year, month, week) {
                var firstDate = new Date(year, month-1, 1);
                var firstDateDay = firstDate.getDay();
                var firstWeekDays = 7 - firstDateDay;
                var date;

                if(week == 0) {
                    //
                    date = 1;
                    for(var i=firstDateDay; i<7; i++) {
                        this.days[i].setLook({
                            bgNormal: this.look.bgColorThisMonth,
                            bgEnter: this.look.bgColorThisMonth,
                            bgDown: this.look.bgColorThisMonth,
                            txtNormal: this.look.txtColorThisMonth,
                            txtEnter: this.look.txtColorThisMonth_enter,
                            txtDown: this.look.txtColorThisMonth,
                            txt: date
                        });
                        this.days[i].dataDate = new Date(year, month-1, date);//
                        date++;
                    }

                    // 若前面有空格則填入上月剩下天數 (要取得上月有幾天)
                    // var lastDateInLastMonth = new Date(firstDate.getFullYear(), firstDate.getMonth(), 0).getDate(); // NOTE date 設 0 會取到上個月的最後一天
                    var tempDate;
                    var cnt = 1;
                    // var lastMonthDate = lastDateInLastMonth;
                    for(var i=firstDateDay-1; i>=0; i--) {
                        tempDate = firstDate.addDays(-cnt);
                        this.days[i].setLook({
                            bgNormal: this.look.bgColorOtherMonth,
                            bgEnter: this.look.bgColorOtherMonth,
                            bgDown: this.look.bgColorOtherMonth,
                            txtNormal: this.look.txtColorOtherMonth,
                            txtEnter: this.look.txtColorOtherMonth_enter,
                            txtDown: this.look.txtColorOtherMonth,
                            txt: tempDate.getDate()//lastMonthDate
                        });
                        cnt++;//lastMonthDate--;
                        this.days[i].dataDate = new Date(tempDate.getTime());//
                    }

                } else { // other weeks
                    //
                    var firstDateOfThisWeek = new Date(year, month-1, firstWeekDays + 7*(week-1) + 1);
                    var tempDate;
                    var thisMonth = new Date(year, month-1, 1).getMonth();
                    for(var i=0; i<7; i++) {
                        tempDate = firstDateOfThisWeek.addDays(i);
                        if(tempDate.getMonth() != thisMonth) {
                            this.days[i].setLook({
                                bgNormal: this.look.bgColorOtherMonth,
                                bgEnter: this.look.bgColorOtherMonth,
                                bgDown: this.look.bgColorOtherMonth,
                                txtNormal: this.look.txtColorOtherMonth,
                                txtEnter: this.look.txtColorOtherMonth_enter,
                                txtDown: this.look.txtColorOtherMonth,
                                txt: tempDate.getDate()
                            });

                        } else {
                            this.days[i].setLook({
                                bgNormal: this.look.bgColorThisMonth,
                                bgEnter: this.look.bgColorThisMonth,
                                bgDown: this.look.bgColorThisMonth,
                                txtNormal: this.look.txtColorThisMonth,
                                txtEnter: this.look.txtColorThisMonth_enter,
                                txtDown: this.look.txtColorThisMonth,
                                txt: tempDate.getDate()
                            });

                        }

                        this.days[i].dataDate = new Date(tempDate.getTime());//

                    }

                }

            }


        }

        class DayButtonPanel extends BasePanel {
            constructor(parentView, options) {
                super(parentView, options);
                var _this = this;


                // == methods ==


                // == events ==
                for(var i=0; i<6; i++) {
                    (function(I) {

                        _this.weeks[I].on("dateSelected", function(date) {
                            _this.emit("dateSelected", date);
                        });

                    })(i);
                }

            }


            // == init ==
            init() {
                super.init();

                // look
                this.look.bgColor = "rgb(150, 150, 150)";
                this.look.padding = 2;
                //
                this.look.panelBgColor = "rgb(100, 100, 100)";
                this.look.panelTxtColor = "rgb(255, 255, 255)";
                this.look.panelSubTxtColor = "rgb(180, 180, 180)";
                this.look.btnBgColorEnter = "rgb(150, 150, 150)";
                this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
                this.look.btnBgColorDown = "rgb(200, 200, 200)";
                this.look.btnTxtColorDown = "rgb(80, 80, 80)";


                //
                // == ui ==
                var ROW_H = 100 / 6;
                var refY = 0;

                //
                this.weeks = [];
                var cell, weekRow;
                for(var i=0; i<6; i++) {
                    cell = new BasePanel(this, {
                        rect: { y: refY, h: ROW_H }
                    });
                    weekRow = new WeekRow(cell, {
                        // look: {
                        //     bgNormal: this.look.btnBgNormal,
                        //     bgEnter: this.look.btnBgColorEnter,
                        //     bgDown: this.look.btnBgColorDown,
                        //     txtNormal: this.look.btnTxtNormal,
                        //     txtEnter: this.look.btnTxtColorEnter,
                        //     txtDown: this.look.btnTxtColorDown
                        // }
                    });
                    this.weeks.push(weekRow);
                    //
                    refY += ROW_H;
                }

            }


            // ====
            initDates(year, month) {
                for(var i=0; i<6; i++) {
                    this.weeks[i].initDates(year, month, i);
                }

            }


        }


        // ====
        return DayButtonPanel;
    })();

    var DayTogglePanel = (function() {

        class WeekRow extends BasePanel {
            constructor(parentView, options) {
                super(parentView, options);
                var _this = this;


                // == methods ==


                // == events ==
                for(var i=0; i<7; i++) {
                    (function(I) {

                        _this.days[I].on("toggle", function(active) {
                            _this.emit("dateToggled", active, _this.days[I].dataDate.getTime());
                            if(active) {
                                _this.emit("dateSelected", _this.days[I].dataDate);
                            } else {
                                _this.emit("dateUnselected", _this.days[I].dataDate);
                            }
                        });

                    })(i);
                }

            }


            // == init ==
            init() {
                super.init();

                // look
                this.look.thisMonth = {
                    inactive: {
                        bgColor: "rgb(200, 200, 200)",
                        txtColor: "rgb(100, 100, 100)"
                    },
                    active: {
                        bgColor: COLOR_BTN_BRIGHT,
                        txtColor: "rgb(240, 240, 240)"
                    }
                };
                this.look.otherMonth = {
                    inactive: {
                        bgColor: "rgb(100, 100, 100)",
                        txtColor: "rgb(200, 200, 200)"
                    },
                    active: {
                        bgColor: COLOR_BTN_BRIGHT,
                        txtColor: "rgb(240, 240, 240)"
                    }
                };


                // == ui ==
                this.days = [];

                var COL_W = 100 / 7;
                var refX = 0;
                var cell, btn;
                for(var i=0; i<7; i++) {
                    cell = new BasePanel(this, {
                        rect: { x: refX, w: COL_W },
                        look: { padding: 1 }
                    });
                    btn = new ToggleView(cell, {
                        look: { txtScale: 0.4 }
                    });
                    this.days.push(btn);
                    //
                    refX += COL_W;
                }


            }

            //
            initDates(year, month, week, activeDates) {
                var firstDate = new Date(year, month-1, 1);
                var firstDateDay = firstDate.getDay();
                var firstWeekDays = 7 - firstDateDay;
                var date;

                if(week == 0) {
                    //
                    date = 1;
                    for(var i=firstDateDay; i<7; i++) {
                        this.look.thisMonth.inactive.txt = date;
                        this.look.thisMonth.active.txt = date;
                        this.days[i].setLook({
                            inactive: this.look.thisMonth.inactive,
                            active: this.look.thisMonth.active
                        });
                        this.days[i].dataDate = new Date(year, month-1, date);//
                        date++;
                    }

                    // 若前面有空格則填入上月剩下天數 (要取得上月有幾天)
                    var tempDate;
                    var cnt = 1;
                    for(var i=firstDateDay-1; i>=0; i--) {
                        tempDate = firstDate.addDays(-cnt);
                        this.look.otherMonth.inactive.txt = tempDate.getDate();
                        this.look.otherMonth.active.txt = tempDate.getDate();
                        this.days[i].setLook({
                            inactive: this.look.otherMonth.inactive,
                            active: this.look.otherMonth.active
                        });
                        cnt++;
                        this.days[i].dataDate = new Date(tempDate.getTime());//
                    }

                } else { // other weeks
                    //
                    var firstDateOfThisWeek = new Date(year, month-1, firstWeekDays + 7*(week-1) + 1);
                    var tempDate;
                    var thisMonth = new Date(year, month-1, 1).getMonth();
                    for(var i=0; i<7; i++) {
                        tempDate = firstDateOfThisWeek.addDays(i);
                        if(tempDate.getMonth() != thisMonth) {
                            this.look.otherMonth.inactive.txt = tempDate.getDate();
                            this.look.otherMonth.active.txt = tempDate.getDate();
                            this.days[i].setLook({
                                inactive: this.look.otherMonth.inactive,
                                active: this.look.otherMonth.active
                            });

                        } else {
                            this.look.thisMonth.inactive.txt = tempDate.getDate();
                            this.look.thisMonth.active.txt = tempDate.getDate();
                            this.days[i].setLook({
                                inactive: this.look.thisMonth.inactive,
                                active: this.look.thisMonth.active
                            });

                        }

                        this.days[i].dataDate = new Date(tempDate.getTime());//

                    }

                }



                // ====
                this.load(activeDates);

            }

            load(activeDates) {
                for(var i=0; i<7; i++) {
                    if(activeDates[this.days[i].dataDate.getTime()]) {
                        this.days[i].setActive(true, "quiet");
                    } else {
                        this.days[i].setActive(false, "quiet");
                    }
                }
            }
            clear() {
                for(var i=0; i<7; i++) {
                    this.days[i].setActive(false, "quiet");
                }
            }

        }


        class DayTogglePanel extends BasePanel {
            constructor(parentView, options) {
                super(parentView, options);
                var _this = this;

                this.activeDates = {};


                // == methods ==


                // == events ==
                for(var i=0; i<6; i++) {
                    (function(I) {

                        _this.weeks[I].on("dateToggled", function(active, timeIndex) {
                            if(active) {
                                if(!_this.activeDates[timeIndex]) _this.activeDates[timeIndex] = 1;
                            } else {
                                if(_this.activeDates[timeIndex]) delete _this.activeDates[timeIndex];
                            }
                        });

                        _this.weeks[I].on("dateSelected", function(date) {
                            _this.emit("dateSelected", date);
                        });
                        _this.weeks[I].on("dateUnselected", function(date) {
                            _this.emit("dateUnselected", date);
                        });

                    })(i);
                }

            }


            // == init ==
            init() {
                super.init();


                // look
                // this.look.bgColor = "rgb(150, 150, 150)";
                this.look.padding = 2;


                // == ui ==
                var ROW_H = 100 / 6;
                var refY = 0;

                //
                this.weeks = [];
                var cell, weekRow;
                for(var i=0; i<6; i++) {
                    cell = new BasePanel(this, {
                        rect: { y: refY, h: ROW_H }
                    });
                    weekRow = new WeekRow(cell);
                    this.weeks.push(weekRow);
                    //
                    refY += ROW_H;
                }

            }

            // ====
            initDates(year, month) {
                //
                for(var i=0; i<6; i++) {
                    this.weeks[i].initDates(year, month, i, this.activeDates);
                }
            }

            getData() {
                return this.activeDates;
            }
            load(newActiveDates) {
                if(isUsable(newActiveDates)) {
                    var timeIndex;
                    for(var i=0; i<newActiveDates.length; i++) {
                        timeIndex = newActiveDates[i].getTime();
                        if(!this.activeDates[timeIndex]) this.activeDates[timeIndex] = 1;
                    }
                    for(var i=0; i<6; i++) {
                        this.weeks[i].load(this.activeDates);
                    }
                }
            }
            clear() {
                delete this.activeDates;
                this.activeDates = {};
                for(var i=0; i<6; i++) {
                    this.weeks[i].clear();
                }
            }

        }

        // ====
        return DayTogglePanel;
    })();


    //
    class Calendar extends BasePanel {
        constructor(parentView, options, type) {
            super(parentView, options);
            var _this = this;
            var _type = (isUsable(type) ? type : "button");


            // == ui ==
            var monthSelector;
            var dayPanel;
            var UNIT_H = 100 / 7.5;

            switch(_type) {
                case "toggle":
                    dayPanel = new DayTogglePanel(this.boxPanel, {
                        rect: { y: UNIT_H * 1.5, h: UNIT_H * 6 }
                    });
                    break;

                default: // "button"
                    dayPanel = new DayButtonPanel(this.boxPanel, {
                        rect: { y: UNIT_H * 1.5, h: UNIT_H * 6 }
                    });

            }
            this.dayPanel = dayPanel;



            // == methods ==
            function initMonthSelector() {
                monthSelector = new MonthSelector(_this.boxPanel);
                monthSelector.on("monthSelected", function(year, month) {
                    _this.year = year;
                    _this.month = month;
                    _this.setDate(_this.year, _this.month+1);
                    monthSelector.hide();
                });
            }
            function showMonthSelector() {
                if(!monthSelector) initMonthSelector();
                monthSelector.show(_this.year);
            }


            // == events ==
            this.controlPanel.on("upClicked", function() {
                var now = new Date(_this.year, _this.month-1, 1);
                var lastMonth = now.addMonths(-1);
                _this.setDate(lastMonth.getFullYear(), lastMonth.getMonth()+1);

            });
            this.controlPanel.on("downClicked", function() {
                var now = new Date(_this.year, _this.month-1, 1);
                var lastMonth = now.addMonths(1);
                _this.setDate(lastMonth.getFullYear(), lastMonth.getMonth()+1);

                // // NOTE test clear()
                // _this.dayPanel.clear();

            });
            this.controlPanel.on("yearMonthClicked", function() {
                showMonthSelector();

                // // NOTE test getDate()
                // console.log(_this.dayPanel.getData());
                // var data = _this.dayPanel.getData();
                // for(var id in data) {
                //     console.log(new Date(parseInt(id.toString())).toYYYYMMdd());
                // }

            });

            this.dayPanel.on("dateSelected", function(date) {
                // console.log(date.toYYYYMMdd());
                _this.emit("dateSelected", date);
            });
            this.dayPanel.on("dateUnselected", function(date) {
                // console.log(date.toYYYYMMdd());
                _this.emit("dateUnselected", date);
            });

            this.boxPanel.domOn("click", function() {});
            this.domOn("click", function() {
                _this.emit("maskClicked");
            });


            // ==
            if(isUsable(options) && isUsable(options.rect) && (isUsable(options.rect.constW) || isUsable(options.rect.constH))) {
                if(isUsable(options.rect.constW)) _this.rect.constW = options.rect.constW;
                if(isUsable(options.rect.constH)) _this.rect.constH = options.rect.constH;
                _this.redraw(true);
            }

        }


        // == init ==
        init() {
            super.init();

            var now = new Date();
            this.year = now.getFullYear();
            this.month = now.getMonth();

            //
            var _this = this;
            var MASK_COLOR = "rgba(50, 50, 50, 0.7)";

            // look
            this.look.bgColor = MASK_COLOR;
            //
            this.look.txtBack = "Back";
            this.look.txtDone = "Done";
            this.look.panelBgColor = "rgb(150, 150, 150)";
            this.look.panelTxtColor = "rgb(255, 255, 255)";
            this.look.panelSubTxtColor = "rgb(180, 180, 180)";
            this.look.btnBgColorEnter = "rgb(150, 150, 150)";
            this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
            this.look.btnBgColorDown = "rgb(200, 200, 200)";
            this.look.btnTxtColorDown = "rgb(80, 80, 80)";

            // opts
            this.opts.visible = false;


            // == ui ==

            var boxPanel = new BasePanel(this, {
                // rect: { x: 5, y: 5, w: 90, h: 90 },
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    visible: true
                }
            })

            var UNIT_H = 100 / 7.5;

            var controlPanel = new ControlPanel(boxPanel, {
                rect: { y: UNIT_H * 0, h: UNIT_H * 1 }
            });

            var weekHeader = new WeekHeader(boxPanel, {
                rect: { y: UNIT_H * 1, h: UNIT_H * 0.5 }
            });

            // var rightPanel = new BasePanel(this, {
            //     look: {
            //         bgColor: this.look.panelBgColor,
            //         borderRadius: 5
            //     },
            //     boxShadow: {
            //         visible: true
            //     }
            // });
            // var bottomPanel = new BasePanel(this, {
            //     look: {
            //         bgColor: this.look.panelBgColor,
            //         borderRadius: 5
            //     },
            //     boxShadow: {
            //         visible: true
            //     }
            // });


            //
            this.controlPanel = controlPanel;
            this.boxPanel = boxPanel;
            // this.dayPanel = dayPanel;
            // this.rightPanel = rightPanel;
            // this.bottomPanel = bottomPanel;

        }


        // == redraw ==

        // rect
        redrawRect() {
            // NOTE 新增屬性的預設值必須在這設
            this.rect.constW = (this.rect.constW ? this.rect.constW : 700);
            this.rect.constH = (this.rect.constH ? this.rect.constH : 750);

            //
            var parentRect = this.parentView.RectPx;
            var BOX_X = (parentRect.w - this.rect.constW) / 2;
            var BOX_Y = (parentRect.h - this.rect.constH) / 2;
            var BOX_W = this.rect.constW;
            var BOX_H = this.rect.constH;
            this.boxPanel.setRect({
                x: BOX_X,
                y: BOX_Y,
                w: BOX_W,
                h: BOX_H,
                mode: "px"
            });

            // var SPACE = 4;
            //
            // this.rightPanel.setRect({
            //     x: BOX_X + BOX_W + SPACE,
            //     y: BOX_Y,
            //     w: BOX_W * (2/7),
            //     h: BOX_H,
            //     mode: "px"
            // });
            //
            // this.bottomPanel.setRect({
            //     x: BOX_X,
            //     y: BOX_Y + BOX_H + SPACE,
            //     w: BOX_W * (1 + 2/7) + SPACE,
            //     h: BOX_H * (1/7.5),
            //     mode: "px"
            // });


            super.redrawRect();
        }

        // look
        // redrawLook() {
        //     super.redrawLook();
        //
        //     this.btnBack.setLook({
        //         txt: this.look.txtBack,
        //         bgNormal: this.look.panelBgColor,
        //         bgEnter: this.look.btnBgColorEnter,
        //         bgDown: this.look.btnBgColorDown,
        //         txtNormal: this.look.panelTxtColor,
        //         txtEnter: this.look.btnTxtColorEnter,
        //         txtDown: this.look.btnTxtColorDown,
        //     });
        //     this.btnDone.setLook({
        //         txt: this.look.txtDone,
        //         bgNormal: this.look.panelBgColor,
        //         bgEnter: this.look.btnBgColorEnter,
        //         bgDown: this.look.btnBgColorDown,
        //         txtNormal: this.look.panelTxtColor,
        //         txtEnter: this.look.btnTxtColorEnter,
        //         txtDown: this.look.btnTxtColorDown,
        //     });
        //
        //
        //     this.boxPanel.redraw(true); // NOTE 看來很多介面不會自己動的問題, 都是因為沒像這樣要求 child redraw
        //
        // }

        show() {
            super.show();
            if(this.parentView.PxW < 500) {
                var newW = this.parentView.PxW - 8;
                var newH = (this.parentView.PxW - 8) / 350 * 375;
                this.setRect({
                    constW: newW,
                    constH: newH
                });
            }
        }


        setDate(year, month) {
            this.year = year;
            this.month = month;
            this.controlPanel.setYearMonth(year, month);
            this.dayPanel.initDates(year, month);
        }

        loadActiveDates(activeDates) {
            this.dayPanel.load(activeDates);
        }
        clearActiveDates() {
            this.dayPanel.clear();
        }


    }


    // ====
    return Calendar;
})();


// ====
module.exports = Calendar;
