
var ButtonView = require("../basic/ButtonView");
var TextView = require("../basic/TextView");
var BasePanel = require("./BasePanel");

var Calendar = require("./Calendar");

// ====

// NOTE TODO 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.


var CalendarNightSelector = (function() {
    //

    //
    class NightsPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==
            function setNights(nights) {
                _this.nights = nights;
                if(_this.nights < 1) _this.nights = 1;
                _this.txtNight.setLook({ txt: _this.nights + " 晚"});
            }
            //
            this.setNights = setNights;


            // == events ==
            this.btnUp.on("click", function() {
                _this.nights--;
                if(_this.nights < 1) _this.nights = 1;
                _this.txtNight.setLook({ txt: _this.nights + " 晚"});
                //
                _this.emit("nightsChanged", _this.nights);

            });
            this.btnDown.on("click", function() {
                _this.nights++;
                _this.txtNight.setLook({ txt: _this.nights + " 晚"});
                //
                _this.emit("nightsChanged", _this.nights);
            });

        }


        // == init ==
        init() {
            super.init();

            this.nights = 1;

            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.btnBgNormal = "rgb(106, 192, 215)";
            this.look.btnBgEnter = "rgb(200, 200, 200)";
            this.look.btnBgDown = "rgb(100, 100, 100)";
            this.look.btnTxtNormal = "rgb(240, 240, 240)";
            this.look.btnTxtEnter = "rgb(100, 100, 100)";
            this.look.btnTxtDown = "rgb(200, 200, 200)";


            // == ui ==
            var UNIT_H = 100 / 7.5;

            var cellUp = new BasePanel(this, {
                rect: { h: UNIT_H * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnUp = new ButtonView(cellUp, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/up_white.png')",
                    imgEnter: "url('img/sys/up_gray.png')",
                    imgDown: "url('img/sys/up_white.png')"
                }
            });

            var cellNight = new BasePanel(this, {
                rect: { y: UNIT_H * 2, h: (100 - UNIT_H * 4) }
            });
            var txtNight = new TextView(cellNight, {
                look: {
                    txtScale: 0.15,
                    // bgColor: this.look.txtBgNormal,
                    txtColor: "white",//this.look.txtTxtNormal,
                    txt: this.nights + " 晚"
                }
            });

            var cellDown = new BasePanel(this, {
                rect: { y: (100 - UNIT_H * 2), h: UNIT_H * 2 },
                look: { bgColor: this.look.btnBgNormal }
            });
            var btnDown = new ButtonView(cellDown, {
                look: {
                    bgNormal: this.look.btnBgNormal,
                    bgEnter: this.look.btnBgNormal,
                    bgDown: this.look.btnBgNormal,
                    imgNormal: "url('img/sys/down_white.png')",
                    imgEnter: "url('img/sys/down_gray.png')",
                    imgDown: "url('img/sys/down_white.png')"
                }
            });


            //
            this.cellUp = cellUp;

            // this.btnYearMonth = btnYearMonth;
            this.btnUp = btnUp;
            this.btnDown = btnDown;
            this.txtNight = txtNight;

        }

        //
        redrawRect() {
            super.redrawRect();

            var refRect = this.cellUp.RectPx;
            var refSize = ((refRect.w > refRect.h) ? refRect.h : refRect.w) * 0.8;

            this.btnUp.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

            this.btnDown.setRect({
                x: (refRect.w - refSize) / 2,
                y: (refRect.h - refSize) / 2,
                h: refSize,
                w: refSize,
                mode: "px"
            });

        }

    }

    class DateStringPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            setTitle(this.look.title);
            setValue(this.look.value);


            // == methods ==
            function setTitle(title) {
                _this.txtTitle.setLook({ txt: title });
            }
            function setValue(value) {
                _this.txtValue.setLook({ txt: value });
            }
            //
            this.setTitle = setTitle;
            this.setValue = setValue;

            // == events ==

        }


        // == init ==
        init() {
            super.init();


            // look
            this.look.bgColor = "rgb(150, 150, 150)";
            //
            this.look.title = "";
            this.look.value = "";


            // == ui ==
            var UNIT_W = 100 / 9.5;

            var cellTitle = new BasePanel(this, {
                rect: { w: UNIT_W * 2 }
            });
            var txtTitle = new TextView(cellTitle, {
                look: {
                    txtScale: 0.4,
                    txtColor: "rgb(220, 220, 220)"
                }
            });

            var cellValue = new BasePanel(this, {
                rect: { x: UNIT_W * 2, w: (100 - UNIT_W * 2) }
            });
            var txtValue = new TextView(cellValue, {
                look: {
                    txtScale: 0.4,
                    txtColor: "white"
                }
            });


            //
            this.txtTitle = txtTitle;
            this.txtValue = txtValue;

        }

    }


    //
    class CalendarNightSelector extends Calendar {
        constructor(parentView, options) {
            super(parentView, options, "toggle");
            var _this = this;

            this.nights = 1;
            this.inDate;
            this.outDate;


            // == methods ==
            function selectNights(date) {
                _this.clearActiveDates();
                //
                var newActiveDates = [date];
                for(var i=1; i<=_this.nights; i++) {
                    newActiveDates.push(date.addDays(i));
                }
                _this.loadActiveDates(newActiveDates);
                //
                _this.inDate = date;
                _this.outDate = newActiveDates[newActiveDates.length-1];
                _this.inPanel.setValue(_this.inDate.toYYYYMMdd());
                _this.outPanel.setValue(_this.outDate.toYYYYMMdd());
            }
            function setNights(nights) {
                _this.nights = nights;
                if(_this.nights < 1) _this.nights = 1;
                _this.nightsPanel.setNights(_this.nights);
            }
            //
            this.setNights = setNights;


            // == events ==
            this.on("dateSelected", selectNights);
            this.on("dateUnselected", selectNights);
            this.nightsPanel.on("nightsChanged", function(nights) {
                _this.nights = nights;
                if(_this.inDate) selectNights(_this.inDate);
            });


        }


        // == init ==
        init() {
            super.init();


            // == ui ==
            var rightPanel = new BasePanel(this, {
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    blur: 4,
                    visible: true
                }
            });
            var nightsPanel = new NightsPanel(rightPanel);


            var bottomPanel = new BasePanel(this, {
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    blur: 4,
                    visible: true
                }
            });
            var inPanel = new DateStringPanel(bottomPanel, {
                rect: { h: 50 },
                look: { title: "入住" }
            });
            var outPanel = new DateStringPanel(bottomPanel, {
                rect: { y: 50, h: 50 },
                look: { title: "退房" }
            });

            // ====
            this.rightPanel = rightPanel;
            this.bottomPanel = bottomPanel;

            this.nightsPanel = nightsPanel;

            this.inPanel = inPanel;
            this.outPanel = outPanel;

        }

        //
        redrawLook() {
            super.redrawLook();
            this.boxPanel.setBoxShadow({ blur: 4 });
        }
        redrawRect() {
            super.redrawRect();

            var BOX_X = this.boxPanel.PxX;
            var BOX_Y = this.boxPanel.PxY;
            var BOX_W = this.boxPanel.PxW;
            var BOX_H = this.boxPanel.PxH;

            var SPACE = 4;

            this.rightPanel.setRect({
                x: BOX_X + BOX_W + SPACE,
                y: BOX_Y,
                w: BOX_W * (2/7),
                h: BOX_H,
                mode: "px"
            });

            this.bottomPanel.setRect({
                x: BOX_X,
                y: BOX_Y + BOX_H + SPACE,
                w: BOX_W * (1 + 2/7) + SPACE,
                h: BOX_H * (2/7.5),
                mode: "px"
            });

        }

        //
        setDate(year, month) {
            super.setDate(year, month);
        }

        loadActiveDates(activeDates) {
            super.loadActiveDates(activeDates);
        }
        clearActiveDates() {
            super.clearActiveDates();
        }



    }



    // ====
    return CalendarNightSelector;
})();


// ====
module.exports = CalendarNightSelector;
