
var VideoView = require('../basic/VideoView');
var CanvasVideoView = require('../basic/CanvasVideoView');
var BasePanel = require("./BasePanel");

// ====

var CapturePanel = (function() {
    //
    var getUserMedia_supported = false;
    function initGetUserMedia() {
        // Test browser support
        window.navigator = window.navigator || {};
        navigator.getUserMedia = navigator.getUserMedia       ||
                                 navigator.webkitGetUserMedia ||
                                 navigator.mozGetUserMedia    ||
                                 null;
        if (navigator.getUserMedia === null) {
           console.log("GG getUserMedia GG");
        } else {
           // Opera <= 12.16 supports video only.
           var audioContext = window.AudioContext       ||
                              window.webkitAudioContext ||
                              null;
           if (audioContext === null) {
              console.log("GG audioContext GG");
           }

           getUserMedia_supported = true;

       }

    }
    initGetUserMedia();

    //
    class CapturePanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var videoStream = null;


            // == methods ==

            function start(opts) {
                if(!getUserMedia_supported) return;

                //
                if(opts && isUsable(opts.deviceIdx)) {
                    // NOTE: 根據 deviceIdx 選取可用的 videoinput device
                    // TODO: 把設定方式跟原本的整合
                    navigator.mediaDevices.enumerateDevices().then(function(devices) {

                        var videos = [];
                        devices.forEach(function(device) {
                            if(device.kind == "videoinput") {
                                videos.push(device.deviceId);
                            }
                        });

                        console.log("[video input]", videos.length);

                        if(videos.length == 0) return console.error("video device not found");
                        if(!isUsable(videos[opts.deviceIdx])) return console.error("deviceIdx out of range");

                        console.log("[deviceId]", videos[opts.deviceIdx]);

                        // var width = (opts.width ? opts.width : { min: 640, ideal: 640, max: 1920 });
                        // var height = (opts.height ? opts.height : { min: 480, ideal: 480, max: 1080 });
                        // var frameRate = (opts.frameRate ? opts.frameRate : { ideal: 10, max: 10 });

                        var _videoOpts = {
                            width: (opts.width ? opts.width : { min: 640, ideal: 1080, max: 1920 }),
                            height: (opts.height ? opts.height : { min: 480, ideal: 720, max: 1080 }),
                            frameRate: (opts.frameRate ? opts.frameRate : { ideal: 10, max: 10 }),
                            // deviceId: { exact: videos[opts.deviceIdx] },
                        };
                        if(videos[opts.deviceIdx]) _videoOpts.deviceId = { exact: videos[opts.deviceIdx] };

                        var _opts = {
                            // video: {
                            //     // mandatory: {
                            //     //     minWidth: (opts.minWidth ? opts.minWidth : 320),
                            //     //     maxWidth: (opts.maxWidth ? opts.maxWidth : 640),
                            //     //     minHeight: (opts.minHeight ? opts.minHeight : 240),
                            //     //     maxHeight: (opts.maxHeight ? opts.maxHeight : 480),
                            //     //     minFrameRate: (opts.minFrameRate ? opts.minFrameRate : 10),
                            //     // },
                            //     width: width,         // TODO: 套用這種設定方式到所有使用到的地方, 或是出個新版本
                            //     height: height,
                            //     frameRate: frameRate,
                            //     deviceId: { exact: videos[opts.deviceIdx] },
                            // },
                            // deviceId: { exact: videos[opts.deviceIdx] },
                            video: _videoOpts,
                            audio: false
                        };
                        console.log("[_opts]", _opts);
                        navigator.mediaDevices.getUserMedia(_opts).then(function(stream) {
                            videoStream = stream;
                            // _this.videoView.setLook({ src: createSrc(stream) });
                            _this.videoView.core.srcObject = stream;                     // NOTE: 這寫法似乎只適用 Chrome // TODO:
                            _this.videoView.play();
                            _this.canvasView.start();

                        }).catch(function(err) {
                            console.log('Video capture error: ' + err.code);
                            _this.emit("cam_err", err);

                        });

                    }).catch(function(err) {
                        console.log(err.name + ": " + err.message);
                    });


                } else {
                    //
                    navigator.getUserMedia(_this.opts.capture, function(stream) {
                    // navigator.mediaDevices.getUserMedia(_this.opts.capture, function(stream) {
                        videoStream = stream;
                        // _this.videoView.setLook({ src: createSrc(stream) });
                        _this.videoView.core.srcObject = stream;                     // NOTE: 這寫法似乎只適用 Chrome // TODO:
                        _this.videoView.play();
                        _this.canvasView.start();

                    }, function(error) {
                        console.log('Video capture error: ' + error.code);
                        _this.emit("cam_err", error);

                    });

                }

            }

            function stop() {
                _this.videoView.pause();
                _this.canvasView.stop();

                if(videoStream && videoStream.getTracks) {
                    var tracks = videoStream.getTracks();

                    for(var i=0; i<tracks.length; i++) {
                        tracks[i].stop();
                    }
                }

            }
            function clear() {
                _this.canvasView.clear();
            }

            //
            this.start = start;
            this.stop = stop;
            this.clear = clear;


            // == events ==
            this.canvasView.on("frame", function(dataURL) {
                _this.emit("frame", dataURL);
            });
            this.canvasView.on("loadeddata", function(videoWidth, videoHeight) {
                _this.emit("loadeddata", videoWidth, videoHeight);
            });


        }


        // == init ==
        init(options) {
            super.init();


            //
            this.opts.capture = {
                video: true,
                audio: false
            };


            //
            var videoView = new VideoView(this, {
                // opts: { visible: false }
            });

            var canvasCell = new BasePanel(this);
            var canvasView = new CanvasVideoView(canvasCell, {
                opts: {
                    visible: false,
                    video: videoView
                },
            });


            //
            this.videoView = videoView;
            this.canvasView = canvasView;


        }


    }





    //
    return CapturePanel;
})();


//
module.exports = CapturePanel;
