
var ButtonView = require("../basic/ButtonView");
var TextView = require("../basic/TextView");
var BasePanel = require("./BasePanel");
var ListPanel = require("./ListPanel");
var ListItem = require("./ListItem");

// ====

var DatePicker = (function() {
    //
    // NOTE TODO 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.


    //
    class DatePicker extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;


            // == methods ==
            function init(initDate) {
                initWheel(_this.yearWheel, initDate.fromYear, initDate.toYear);

                //
                initWheel(_this.monthWheel, 1, 12, "00");
                initWheel(_this.dateWheel, 1, 31, "00");

            }
            function initWheel(targetList, fromVal, toVal, paddingStr) {
                var arr = [];
                arr.push({ name: "" });
                for(var i=fromVal; i<=toVal; i++) {
                    if(paddingStr) {
                        arr.push({
                            name: i.toString().paddingLeft(paddingStr)
                        });
                    } else {
                        arr.push({
                            name: i.toString()
                        });
                    }
                }
                arr.push({ name: "" });
                targetList.load(arr);
            }

            // == events ==
            this.btnBack.on("click", function(btn) {
                // console.log("[done]", btn);
                // _this.emit("result");
                _this.hide();
            });
            this.btnDone.on("click", function(btn) {
                // console.log("[done]", btn);
                _this.emit("done", _this.getDate());
                _this.hide();
            });


            // ====
            if(isUsable(options) && isUsable(options.initDate)) {
                init(options.initDate);
            } else {
                init({
                    fromYear: parseInt(new Date().getFullYear()),
                    toYear: parseInt(new Date().getFullYear()) + 3
                });
            }

        }


        // == init ==
        init() {
            super.init();

            var _this = this;
            var MASK_COLOR = "rgba(50, 50, 50, 0.7)";

            // look
            this.look.bgColor = "rgba(200, 200, 200, 0.7)";
            //
            this.look.txtBack = "Back";
            this.look.txtDone = "Done";
            this.look.panelBgColor = "rgb(100, 100, 100)";
            this.look.panelTxtColor = "rgb(255, 255, 255)";
            this.look.panelSubTxtColor = "rgb(180, 180, 180)";
            this.look.btnBgColorEnter = "rgb(150, 150, 150)";
            this.look.btnTxtColorEnter = "rgb(255, 255, 255)";
            this.look.btnBgColorDown = "rgb(200, 200, 200)";
            this.look.btnTxtColorDown = "rgb(80, 80, 80)";

            // opts
            this.opts.visible = false;

            //
            // == ui ==
            //
            this.boxPanel = new BasePanel(this, {
                rect: { x: 5, y: 30, w: 90, h: 40 },
                look: {
                    bgColor: this.look.panelBgColor,
                    borderRadius: 5
                },
                boxShadow: {
                    visible: true
                }
            })

            // row wheel
            var rowWheel = new BasePanel(this.boxPanel, {
                rect: { h: 85 }
            });

            var colW = 100 / 3;

            var colYear = new BasePanel(rowWheel, {
                rect: { w: colW }
            });
            this.yearWheel = new WheelPicker(colYear);

            var colMonth = new BasePanel(rowWheel, {
                rect: { x: colW, w: colW }
            });
            this.monthWheel = new WheelPicker(colMonth);

            var colDate = new BasePanel(rowWheel, {
                rect: { x: colW * 2, w: colW }
            });
            this.dateWheel = new WheelPicker(colDate);

            //
            var maskH = 100 / 3;
            var topMask = new BasePanel(rowWheel, {
                rect: { h: maskH },
                look: {
                    bgColor: MASK_COLOR
                }
            });
            var bottomMask = new BasePanel(rowWheel, {
                rect: { y: maskH * 2, h: maskH },
                look: {
                    bgColor: MASK_COLOR
                }
            });


            // row control
            var rowControl = new BasePanel(this.boxPanel, {
                rect: { y: 85, h: 15 }
            });

            var colBack = new BasePanel(rowControl, {
                rect: { w: 50 }
            });
            this.btnBack = new ButtonView(colBack, {
                // rect: { x: 50, w: 50 },
                look: {
                    // txtShadow: {
                    //     color: "rgba(0, 0, 0, 1)"
                    // },
                    bgNormal: this.look.panelBgColor,
                    bgEnter: this.look.btnBgColorEnter,
                    bgDown: this.look.btnBgColorDown,
                    txtNormal: this.look.panelTxtColor,
                    txtEnter: this.look.btnTxtColorEnter,
                    txtDown: this.look.btnTxtColorDown,
                    txt: this.look.txtBack
                    // imgNormal: "url('my.png')",
                    // imgEnter: "url('my2.jpg')",
                    // imgDown: "url('my3.jpg')",
                }
            });

            var colDone = new BasePanel(rowControl, {
                rect: { x: 50, w: 50 }
            });
            this.btnDone = new ButtonView(colDone, {
                // rect: { x: 50, w: 50 },
                look: {
                    // txtShadow: {
                    //     color: "rgba(0, 0, 0, 1)"
                    // },
                    bgNormal: this.look.panelBgColor,
                    bgEnter: this.look.btnBgColorEnter,
                    bgDown: this.look.btnBgColorDown,
                    txtNormal: this.look.panelTxtColor,
                    txtEnter: this.look.btnTxtColorEnter,
                    txtDown: this.look.btnTxtColorDown,
                    txt: this.look.txtDone
                    // imgNormal: "url('my.png')",
                    // imgEnter: "url('my2.jpg')",
                    // imgDown: "url('my3.jpg')",
                }
            });

        }


        // == redraw ==

        // look
        redrawLook() {
            super.redrawLook();

            this.btnBack.setLook({
                txt: this.look.txtBack,
                bgNormal: this.look.panelBgColor,
                bgEnter: this.look.btnBgColorEnter,
                bgDown: this.look.btnBgColorDown,
                txtNormal: this.look.panelTxtColor,
                txtEnter: this.look.btnTxtColorEnter,
                txtDown: this.look.btnTxtColorDown,
            });
            this.btnDone.setLook({
                txt: this.look.txtDone,
                bgNormal: this.look.panelBgColor,
                bgEnter: this.look.btnBgColorEnter,
                bgDown: this.look.btnBgColorDown,
                txtNormal: this.look.panelTxtColor,
                txtEnter: this.look.btnTxtColorEnter,
                txtDown: this.look.btnTxtColorDown,
            });


            this.boxPanel.redraw(true); // NOTE 看來很多介面不會自己動的問題, 都是因為沒像這樣要求 child redraw

        }


        // ====
        show(date) {
            super.show();
            this.setDate(date);
        }

        setDate(date) {
            this.yearWheel.setScrollToValue(date.getFullYear());
            this.monthWheel.setScrollToValue(date.getMonth() + 1);
            this.dateWheel.setScrollToValue(date.getDate());
        }
        getDate() {
            var year = this.yearWheel.getSelectedData().name;
            var month = this.monthWheel.getSelectedData().name;
            var date = this.dateWheel.getSelectedData().name;
            return year + "/" + month + "/" + date;
        }


    }

    //
    var WheelPicker = (function() {
        var ROW_NUM = 3;
        var ROW_H = 100 / ROW_NUM;

        //
        class PickerPanel extends ListPanel {
            constructor(parentView, options) {
                super(parentView, options);
                var _this = this;
                var _idx = -1;

                var scrollTimer = new Timer(function() {
                    var unitH = _this.PxH / 3;
                    var idx = Math.round(_this.ScrollTop / unitH);
                    _this.scrollTo(idx, true, 100);

                    _idx = idx + 1; // NOTE 因為 wheel 第一個 item 是空白的, 故取值時要 +1 (_idx 為取值用)

                }, 300);


                // == methods ==
                //
                function setScrollToValue(val) {
                    var items = _this.itemArr;
                    var idx = null;
                    var data;
                    for(var i in items) {
                        data = items[i].data;
                        if(data.name == val) {
                            idx = parseInt(i) - 1;
                            break;
                        }
                    }
                    if(idx != null) {
                        _this.scrollTo(idx);
                        _idx = idx + 1;
                    }
                }
                function getSelectedData() {
                    return _this.getDataOf(_idx);
                }
                //
                this.setScrollToValue = setScrollToValue;
                this.getSelectedData = getSelectedData;

                // == events ==
                this.domOn("scroll", function() {
                    scrollTimer.start();
                });

            }

            init() {
                super.init();

            }

            load(dataList) {
                super.load(PickerItem, ROW_H, dataList);
            }


        }

        //
        class PickerItem extends ListItem {
            constructor(parentView, options, data) {
                super(parentView, options, data);
                var _this = this;

                // == ui ==
                this.txt.setLook({ txt: data.name });


                // == events ==

            }

            init() {
                super.init();

                //
                this.look.itemBgColor = "rgb(100, 100, 100)";
                this.look.itemTxtColor = "rgb(255, 255, 255)";
                //
                this.txt = new TextView(this, {
                    look: {
                        bgColor: this.look.itemBgColor,
                        txtColor: this.look.itemTxtColor,
                        txtAlign: "center",
                        // txt: this.data.name
                    }
                });

            }


        }


        // ====
        return PickerPanel;
    })();



    //
    return DatePicker;
})();


// ====
module.exports = DatePicker;
