
var ButtonView = require("../basic/ButtonView");
var BasePanel = require("./BasePanel");
var ListItem = require("./ListItem");

// ====

var COLOR_BTN_BRIGHT = "rgb(56, 147, 170)";

var GenericPicker = (function() {
    //
    // NOTE 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.

    //
    class GenericPicker extends BasePanel {
        constructor(parentView, options, data) {
            super(parentView, options);
            var _this = this;
            var _idx = -1;

            this.ROW = this.look.rowNum;
            this.COL = this.look.colNum;
            this.CELL_W = 100 / this.COL;
            this.CELL_H = 100 / this.ROW;


            // == ui ==
            this.panel.setRect({
                x: (100 - this.rect.box.w) / 2,
                y: (100 - this.rect.box.h) / 2,
                w: this.rect.box.w,
                h: this.rect.box.h,
            })
            this.panel.setOverflow({
                x: this.look.box.overflowX,
                y: this.look.box.overflowY
            })
            createItems(data);


            // == methods ==
            function createItems(data) {
                var newItem;
                for(var i=0; i<data.length; i++) {
                    newItem = new PickerItem(_this.panel, {
                        look: {
                            itemTxtScale: _this.look.itemTxtScale
                        }
                    }, data[i]);
                    newItem.on("itemClicked", function() {
                        _this.emit("itemClicked", this);
                    });
                    _this.items.push(newItem);
                }
                _this.redrawPanel();
            }


            // == events ==
            this.domOn("click", function() {
                _this.hide();
            });


        }

        init() {
            super.init();

            this.items = [];

            //
            this.rect.box = {
                w: 100,
                h: 100
            };

            //
            this.look.bgColor = "rgba(50, 50, 50, 0.5)";
            //
            this.look.rowNum = 4;
            this.look.colNum = 1;
            this.look.itemTxtScale = 0.5;
            this.look.box = {
                overflowX: "hidden",
                overflowY: "hidden"
            };

            //
            this.panel = new BasePanel(this);

        }

        redrawPanel() {
            for(var i=0; i<this.items.length; i++) {
                this.items[i].setRect({
                    x: this.CELL_W * (i % this.COL),
                    y: this.CELL_H * Math.floor(i / this.COL),
                    w: this.CELL_W,
                    h: this.CELL_H
                });
            }
            this.redraw(true);
        }

    }

    //
    class PickerItem extends ListItem {
        constructor(parentView, options, data) {
            super(parentView, options, data);
            var _this = this;

            // == ui ==
            this.btn.setLook({
                bgNormal: this.look.itemBgNormal,
                bgEnter: this.look.itemBgEnter,
                bgDown: this.look.itemBgDown,
                txtNormal: this.look.itemTxtNormal,
                txtEnter: this.look.itemTxtEnter,
                txtDown: this.look.itemTxtDown,
                txtScale: this.look.itemTxtScale,
                borderRadius: this.look.itemBorderRadius,
                txt: data.name
            });


            // == events ==
            this.btn.on("click", function() {
                _this.emit("itemClicked", _this);
            });

        }

        init() {
            super.init();

            // inherit
            this.look.padding = 1;
            // extend
            this.look.itemBgNormal = COLOR_BTN_BRIGHT;//"rgb(106, 192, 215)";
            this.look.itemBgEnter = "rgb(200, 200, 200)";
            this.look.itemBgDown = "rgb(100, 100, 100)";
            this.look.itemTxtNormal = "rgb(240, 240, 240)";
            this.look.itemTxtEnter = "rgb(100, 100, 100)";
            this.look.itemTxtDown = "rgb(200, 200, 200)";
            this.look.itemTxtScale = 0.4;
            this.look.itemBorderRadius = 4;

            //
            this.btn = new ButtonView(this);

        }


    }




    //
    return GenericPicker;
})();


// ====
module.exports = GenericPicker;
