
var BasePanel = require("./BasePanel");

// ====

var ListPanel = (function() {
    //
    function createTargetView(ctx, TargetView, options, data) {
        var newItem = new TargetView(ctx, options, data);
        newItem.on("click", function() {
            ctx.emit("itemClicked", this);
        });
        newItem.on("dblclick", function() {
            ctx.emit("itemDblClicked", this);
        });
        return newItem;
    }

    //
    class ListPanel extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

        }

        // == init ==
        init() {
            super.init();

            // opts
            this.opts.orientation = "vertical";

            //
            this.itemArr = [];

        }


        // == redraw ==

        // opts
        redrawOpts() {
            super.redrawOpts();

            if(this.opts.orientation == "horizontal") {
                this.setOverflow({
                    x: "scroll",
                    y: "hidden"
                });
            } else {                    // vertical
                this.setOverflow({
                    x: "hidden",
                    y: "scroll"
                });
            }

        }



        // ====

        //
        // NOTE TargetView must be extended from ListItem
        // NOTE options.rect 只有 w 或 h 有效. (VERTICAL 的 w 與 HORIZONTAL 的 h 一定是 100)
        // NOTE >> 前項的值改成 size
        createNewItems(TargetView, size, data) {
            var _this = this;

            var options;
            if(this.opts.orientation == "horizontal") {
                options = {
                    rect: {
                        w: size,
                        h: 100,
                        mode: "%"
                    }
                };
            } else {                    // vertical
                options = {
                    rect: {
                        w: 100,
                        h: size,
                        mode: "%"
                    }
                };
            }

            var newItem;
            var newItems = [];
            if(Array.isArray(data)) {
                for(var i=0; i<data.length; i++) {
                    newItems.push(createTargetView(_this, TargetView, options, data[i]));
                }
            } else {
                newItems.push(createTargetView(_this, TargetView, options, data));
            }
            return newItems;
        }
        addToHead(TargetView, options, data) {
            var newItems = this.createNewItems(TargetView, options, data);
            this.itemArr = newItems.concat(this.itemArr);
            this.redrawItems();
        }
        addToTail(TargetView, options, data) {
            var newItems = this.createNewItems(TargetView, options, data);
            this.itemArr = this.itemArr.concat(newItems);
            this.redrawItems();
        }
        redrawItems() {
            var ref = 0;

            for(var i=0; i < this.itemArr.length; i++) {
                this.itemArr[i].Index = i;

                if(this.opts.orientation == "horizontal") {
                    this.itemArr[i].setRect({
                        x: ref,
                        y: 0
                    });
                    ref += this.itemArr[i].rect.w;

                } else {                    // vertical
                    this.itemArr[i].setRect({
                        x: 0,
                        y: ref
                    });
                    ref += this.itemArr[i].rect.h;

                }

            }

        }

        remove(idx) {
            var targetItem = this.itemArr[idx];
            if(targetItem) {
                targetItem.dispose();
                this.itemArr.splice(idx, 1);
                this.redrawItems();
            }
        }
        clear() {
            for(var i=0; i<this.itemArr.length; i++) {
                this.itemArr[i].dispose();
            }
            this.itemArr = [];
        }
        load(TargetView, size, dataList) {  // alias of addToTail
            this.addToTail(TargetView, size, dataList);
        }

        scrollTo(idx, animate, duration) {
            // NOTE 若沒空間捲了, 不一定會到頂
            var _animate = (isUsable(animate) ? animate : false);
            var _duration = (isUsable(duration) ? duration : 500);

            var ref = 0;
            if(this.opts.orientation == "horizontal") {
                for(var i=0; i<this.itemArr.length; i++) {
                    if(i < idx) {
                        ref += this.itemArr[i].PxW;
                    } else {
                        break;
                    }
                }

                if(_animate) {
                    this.animate({ scroll: { x: ref }, duration: _duration });
                } else {
                    this.core.scrollleft = ref;
                }

            } else {                    // vertical
                for(var i=0; i<this.itemArr.length; i++) {
                    if(i < idx) {
                        ref += this.itemArr[i].PxH;
                    } else {
                        break;
                    }
                }

                if(_animate) {
                    this.animate({ scroll: { y: ref }, duration: _duration });
                } else {
                    this.core.scrollTop = ref;
                }

            }

        }

        getDataOf(idx) {
            return (isUsable(this.itemArr[idx]) ? this.itemArr[idx].data : null);
        }

    }


    //
    return ListPanel;
})();


//
module.exports = ListPanel;
