
var BasePanel = require("./BasePanel");

// ====

var Slider = (function() {
    //
    // NOTE 新參數寫在 init 裡, 新參數造成的改變寫在 constructor 裡.

    //
    class Slider extends BasePanel {
        constructor(parentView, options) {
            super(parentView, options);
            var _this = this;

            var active = false;
            var refX_mouse, refX_indicator, refW_barFront;


            // == ui ==
            this.barBack.setLook({ bgColor: this.look.bgColorBack });
            this.barFront.setLook({ bgColor: this.look.bgColorFront });
            this.indicator.setLook({ bgColor: this.look.bgColorIndicator });


            // == methods ==
            function onStart(clientX) {
                active = true;
                refX_mouse = clientX;
                refX_indicator = _this.indicator.rect.x;
                refW_barFront = _this.barFront.rect.w;
            }
            function onMoving(clientX) {
                if(active) {
                    var offset = (clientX - refX_mouse) / _this.barPanel.PxW * 100;
                    var newIndicatorX = refX_indicator + offset;
                    var newBarFrontW = refW_barFront + offset;
                    if(newIndicatorX < 0) {
                        newIndicatorX = 0;
                    } else if(newIndicatorX > 95) {
                        newIndicatorX = 95;
                    }

                    if(newBarFrontW <= 5) {
                        newBarFrontW = 5;
                    } else if(newBarFrontW >= 95) {
                        newBarFrontW = 100;
                    }

                    _this.indicator.setRect({ x: newIndicatorX });
                    _this.barFront.setRect({ w: newBarFrontW });

                    //
                    _this.emit("change", _this.Value);

                }
            }


            // == events ==

            // = mouse =
            this.barPanel.domOn("mouseup", function() {
                active = false;
            });
            this.barPanel.domOn("mouseleave", function(e) {
                active = false;
            });
            this.barPanel.domOn("mousemove", function(e) {
                onMoving(e.clientX);
            });
            this.indicator.domOn("mousedown", function(e) {
                onStart(e.clientX);
            });

            // = touch =
            this.barPanel.domOn("touchstart", function(e) {
                onStart(e.touches[0].clientX);
            });
            this.barPanel.domOn("touchend", function(e) {
                active = false;
            });
            this.indicator.domOn("touchmove", function(e) {
                onMoving(e.touches[0].clientX);
            });

        }

        init() {
            super.init();

            // extends
            this.look.bgColorBack = "rgb(63, 142, 163)";
            this.look.bgColorFront = "rgb(63, 142, 163)";
            this.look.bgColorIndicator = "rgb(106, 192, 215)";
            this.opts.max = 100;
            this.opts.min = 0;


            //
            var INDICATOR_W = 5;
            var INDICATOR_H = 50;
            var INDICATOR_X = (100 - INDICATOR_W) / 2;
            var INDICATOR_Y = (100 - INDICATOR_H) / 2;
            var BAR_H = 100 / 3;
            var BAR_FRONT_W = 50;

            var barPanel = new BasePanel(this, {
                rect: { x: 5, y: 35, w: 90, h: 30 }
            });

            var barBack = new BasePanel(barPanel, {
                rect: { y: BAR_H, h: BAR_H },
                look: {
                    borderRadius: 4,
                    bgColor: this.look.bgColorBack
                }
            });

            var barFront = new BasePanel(barPanel, {
                rect: { y: BAR_H, w: BAR_FRONT_W, h: BAR_H },
                look: {
                    borderRadius: 4,
                    bgColor: this.look.bgColorFront
                }
            });

            var indicator = new BasePanel(barPanel, {
                rect: { x: INDICATOR_X, y: INDICATOR_Y, w: INDICATOR_W, h: INDICATOR_H },
                look: {
                    borderRadius: 4,
                    bgColor: this.look.bgColorIndicator
                },
                opts: {
                    cursor: "pointer"
                },
                // boxShadow: {
                //     visible: true,
                //     blur: 2,
                //     spread: 2
                // }
            });

            //
            this.barPanel = barPanel;
            this.barBack = barBack;
            this.barFront = barFront;
            this.indicator = indicator;


        }


        // ====
        get Value() {
            return (this.indicator.rect.x / 95 * (this.opts.max - this.opts.min)) + this.opts.min;
        }


    }


    //
    return Slider;
})();


// ====
module.exports = Slider;
